@@ -1,15 +1,14 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
-// Copyright Bob Friesenhahn, 1999, 2000, 2001
+// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002, 2003
 //
 // Tests for setting/getting Magick::Image attributes
 //
 
+#include <Magick++.h>
 #include <string>
 #include <iostream>
 
-#include <Magick++.h>
-
 using namespace std;
 
 using namespace Magick;
@@ -174,7 +173,8 @@ int main( int /*argc*/, char ** argv)
       }
     else
       if ( string(image.backgroundColor()) != "#0000FF" &&
-	   string(image.backgroundColor()) != "#00000000FFFF" )
+	   string(image.backgroundColor()) != "#00000000FFFF" &&
+	   string(image.backgroundColor()) != "#0000000000000000FFFFFFFF" )
 	{
 	  ++failures;
 	  cout << "Line: " << __LINE__ << ", backgroundColor ("
@@ -193,7 +193,8 @@ int main( int /*argc*/, char ** argv)
       }
     else
       if ( string(image.backgroundColor()) != "#00AAFF" && 
-	   string(image.backgroundColor()) != "#0000AAAAFFFF" )
+	   string(image.backgroundColor()) != "#0000AAAAFFFF" && 
+	   string(image.backgroundColor()) != "#00000000AAAAAAAAFFFFFFFF" )
 	{
 	  ++failures;
 	  cout << "Line: " << __LINE__
@@ -257,8 +258,9 @@ int main( int /*argc*/, char ** argv)
     // baseFilename
     //
     // Base filename is color for xc images
-    if ( image.baseFilename() != "#FF0000" &&
-	 image.baseFilename() != "#FFFF00000000")
+    if ( image.baseFilename() != "xc:#FF0000" &&
+	 image.baseFilename() != "xc:#FFFF00000000" &&
+	 image.baseFilename() != "xc:#FFFFFFFF0000000000000000")
       {
 	++failures;
 	cout << "Line: " << __LINE__
@@ -346,7 +348,7 @@ int main( int /*argc*/, char ** argv)
     //
     {
       // Test default setting
-      float x, y;
+      double x, y;
       image.chromaBluePrimary( &x, &y );
       if ( x != 0 || y != 0 )
 	{
@@ -372,7 +374,7 @@ int main( int /*argc*/, char ** argv)
     //
     {
       // Test default setting
-      float x, y;
+      double x, y;
       image.chromaGreenPrimary( &x, &y );
       if ( x != 0 || y != 0 )
 	{
@@ -397,7 +399,7 @@ int main( int /*argc*/, char ** argv)
     //
     {
       // Test default setting
-      float x, y;
+      double x, y;
       image.chromaRedPrimary( &x, &y );
       if ( x != 0 || y != 0 )
 	{
@@ -422,7 +424,7 @@ int main( int /*argc*/, char ** argv)
     //
     {
       // Test default setting
-      float x, y;
+      double x, y;
       image.chromaWhitePoint( &x, &y );
       if ( x != 0 || y != 0 )
 	{
@@ -471,28 +473,6 @@ int main( int /*argc*/, char ** argv)
       }
     image.colorFuzz( 0 );
 
-    //
-    // colorMap
-    //
-    // Test default (first color is image canvas base color)
-    if ( image.colorMap(0) != canvasColor )
-      {
-	++failures;
-	cout << "Line: " << __LINE__ << ", colorMap(0) ("
-	     << string(image.colorMap(0))
-	     << ") is not canvas base color " << endl;
-      }
-
-    // Test setting/getting colorMap
-    image.colorMap(0,"#00AAFF");
-    if ( image.colorMap(0) != Color("#00AAFF") )
-      {
-	++failures;
-	cout << "Line: " << __LINE__
-             << ", failed to set/get colorMap(0)" << endl;
-      }
-    image.colorMap(0,canvasColor);
-
     //
     // columns
     //
@@ -545,8 +525,8 @@ int main( int /*argc*/, char ** argv)
       }
 
     // Test set/get
-    image.compressType(RunlengthEncodedCompression);
-    if ( image.compressType() != RunlengthEncodedCompression )
+    image.compressType(RLECompression);
+    if ( image.compressType() != RLECompression )
       {
 	++failures;
 	cout << "Line: " << __LINE__ << ", compressType set/get failed" << endl;
@@ -558,11 +538,11 @@ int main( int /*argc*/, char ** argv)
     //
     {
       // Test defaults
-      if ( image.density().isValid() )
+      if ( image.density() != Geometry(72,72) )
 	{
 	  ++failures;
 	  cout << "Line: " << __LINE__
-               << ", density default is incorrectly valid" << endl;
+               << ", density default is not 72x72 as expected" << endl;
 	}
       
       // Test set/get
@@ -586,24 +566,65 @@ int main( int /*argc*/, char ** argv)
 
     }
 
-    // Test setting invalid
-    image.density( Geometry() );
-    if ( image.density().isValid() )
+    //
+    // Format specific defines
+    //
+    if (image.defineSet("foo","bar"))
       {
-	++failures;
-	cout << "Line: " << __LINE__ << ", density failed set to inValid" << endl;
+        ++failures;
+        cout << "Line: " << __LINE__
+             << ", define for foo:bar incorrectly reports set."
+             << endl;
+      }
+
+    image.defineSet("foo","bar",true);
+    if (!image.defineSet("foo","bar"))
+      {
+        ++failures;
+        cout << "Line: " << __LINE__
+             << ", define for foo:bar incorrectly reports not set."
+             << endl;
+      }
+
+    image.defineSet("foo","bar",false);
+    if (image.defineSet("foo","bar"))
+      {
+        ++failures;
+        cout << "Line: " << __LINE__
+             << ", define for foo:bar incorrectly reports set."
+             << endl;
+      }
+
+    image.defineValue("foo","bar","value");
+    std::string value = image.defineValue("foo","bar");
+    if (image.defineValue("foo","bar") != "value")
+      {
+        ++failures;
+        cout << "Line: " << __LINE__
+             << ", define for foo:bar incorrectly reports value \""
+             << value << "\""
+             << endl;
+      }
+
+    image.defineSet("foo","bar",false);
+    if (image.defineSet("foo","bar"))
+      {
+        ++failures;
+        cout << "Line: " << __LINE__
+             << ", define for foo:bar incorrectly reports set."
+             << endl;
       }
 
     //
     // depth
     //
-    if ( image.depth() != 8 )
+    if ( image.depth() != QuantumDepth )
       {
 	++failures;
 	cout << "Line: " << __LINE__
              << ", depth ("
              << image.depth()
-             << ") is not equal to 8" << endl;
+             << ") is not equal to " << QuantumDepth << endl;
       }
 
     //
@@ -611,14 +632,19 @@ int main( int /*argc*/, char ** argv)
     //
     {
       // Since this is not a montage image, simply verify error report
-      cout << "Checking for working exceptions (may crash) ... ";
       bool caughtException = false;
+      cout << "Testing throwing and catching exceptions. A program crash or a message" << endl
+           << "that the exception was not caught indicates a test failure.  A properly" << endl
+           << "formatted exception message indicates success:" << endl;
       try
 	{
-	  image.directory();
+	  //image.directory();
+          Magick::Image bad_image("foo");
 	}
-      catch ( Exception )
+      catch ( Exception exception_)
 	{
+          cout << "Caught exception, good!:" << endl
+               << "  \"" << exception_.what() << "\"" << endl;
 	  caughtException = true;
 	}
       if ( caughtException != true )
@@ -626,11 +652,6 @@ int main( int /*argc*/, char ** argv)
 	  ++failures;
           cout << "failed to catch exception!" << endl;
 	}
-      else
-        {
-          cout << "exceptions are working!" << endl;
-        }
-
     }
 
     //
@@ -719,13 +740,13 @@ int main( int /*argc*/, char ** argv)
     // fontPointsize
     //
     // Test default
-    if ( image.fontPointsize() != 12 )
+    if ( image.fontPointsize() != 0 )
       {
 	++failures;
 	cout << "Line: " << __LINE__
              << ", fontPointsize ("
              << image.fontPointsize()
-             << ") is not default of 12 as expected"
+             << ") is not default of 0 as expected"
              << endl;
       }
 
@@ -1069,12 +1090,12 @@ int main( int /*argc*/, char ** argv)
     // page
     //
     // Test default
-    if ( image.page() != Geometry(0,0,0,0) )
+    if ( image.page() != Geometry(640,480,0,0) )
       {
 	++failures;
 	cout << "Line: " << __LINE__ << ", page default "
 	     << "(" << string(image.page()) << ")"
-	     << " is not empty as expected" << endl;
+	     << " is not 640x480 as expected" << endl;
       }
 
     // Test set/get
@@ -1090,11 +1111,11 @@ int main( int /*argc*/, char ** argv)
     // quality
     //
     // Test default
-    if ( image.quality() != 75 )
+    if ( image.quality() != 0 )
       {
 	++failures;
 	cout << "Line: " << __LINE__
-             << ", quality default is not 75 as expected" << endl;
+             << ", quality default is not 0 as expected" << endl;
       }
 
     // Test set/get
@@ -1104,7 +1125,7 @@ int main( int /*argc*/, char ** argv)
 	++failures;
 	cout << "Line: " << __LINE__ << ", quality set/get failed" << endl;
       }
-    image.quality(75);
+    image.quality(0);
 
     //
     // quantizeColors
@@ -1130,7 +1151,7 @@ int main( int /*argc*/, char ** argv)
     // quantizeColorSpace
     //
     // Test default
-    if ( image.quantizeColorSpace() != RGBColorspace )
+    if ( image.quantizeColorSpace() != UndefinedColorspace )
       {
 	++failures;
 	cout << "Line: " << __LINE__
@@ -1261,14 +1282,17 @@ int main( int /*argc*/, char ** argv)
     // signature
     //
 
-    if ( image.signature() != "81d5c520d4c23e608404e49eb4ac1dd67c546e571faf62bd66d8f89b348ee488" &&
-	 image.signature() != "b3d1dca044c5a015d422eecd7189a02c69db848675302af5770e5a364d4874fc" )
+    if ( image.signature() != "849077c0eb67c77cf3b06ff2419cc2b11dea5ff7d8f95466914f23731d1f1a68" &&
+	 image.signature() != "7fe11319c31622db67434ab155cd733afbb391e5dbc2657d08893ebea19ac6dc" &&
+	 image.signature() != "e073572dfa4ad28f2f8dd3c6d37dfb14585e60c94cfae910149e97eff2fd895f" &&
+	 image.signature() != "e12b9781b3a5025628567a4eabf970d16d42560e1b86189caceb03ec358dd8e6" &&
+         image.signature() != "56f66a46543e65e7f78022c1c5cb3bd76b82b987b0daaf5cb33fab12caf0563f")
       {
 	++failures;
 	cout << "Line: " << __LINE__ << ", signature ("
 	     << image.signature()
 	     << ") is incorrect" << endl;
-	// image.display();
+	image.display();
       }
 
     //
@@ -1370,11 +1394,14 @@ int main( int /*argc*/, char ** argv)
     //
     // type
     //
+    image.type(PaletteType);
     if ( image.type() != PaletteType )
       {
 	++failures;
 	cout << "Line: " << __LINE__
-             << ", type is not PaletteType as expected" << endl;
+             << ", type is not PaletteType as expected. Reported type "
+             << (int) image.type() << endl;
+
       }
 
     //
@@ -1442,23 +1469,24 @@ int main( int /*argc*/, char ** argv)
     //
     // xResolution
     //
-    if ( image.xResolution() != 0 )
+    if ( image.xResolution() != 72 )
       {
 	++failures;
 	cout << "Line: " << __LINE__
-             << ", xResolution default is not zero as expected" << endl;
+             << ", xResolution default (" << image.xResolution()
+             << ") is not zero as expected" << endl;
       }
 
     //
     // yResolution
     //
-    if ( image.yResolution() != 0 )
+    if ( image.yResolution() != 72 )
       {
 	++failures;
 	cout << "Line: " << __LINE__
-             << ", yResolution default is not zero as expected" << endl;
+             << ", yResolution default (" << image.yResolution()
+             << ") is not zero as expected" << endl;
       }
-
   }
   catch( Exception &error_ )
     {
