@@ -10,39 +10,26 @@
 %                            F      P      X   X                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                     Read/Write FlashPIX Image Format.                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,23 +39,80 @@
 /*
   Include declarations.
 */
-#include "magick.h"
-#include "define.h"
-#if defined(HasFPX)
-#if !defined(vms) && !defined(macintosh) && !defined(WIN32)
+#include "magick/studio.h"
+#include "magick/property.h"
+#include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/color.h"
+#include "magick/color-private.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
+#include "magick/geometry.h"
+#include "magick/image.h"
+#include "magick/image-private.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/pixel.h"
+#include "magick/quantum-private.h"
+#include "magick/static.h"
+#include "magick/string_.h"
+#include "magick/module.h"
+#if defined(MAGICKCORE_FPX_DELEGATE)
+#if !defined(vms) && !defined(macintosh) && !defined(__WINDOWS__)
 #include <fpxlib.h>
 #else
 #include "Fpxlib.h"
 #endif
 #endif
 
+#if defined(MAGICKCORE_FPX_DELEGATE)
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteFPXImage(const ImageInfo *,Image *);
+#endif
 
-#if defined(HasFPX)
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   I s F P X                                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  IsFPX() returns MagickTrue if the image format type, identified by the
+%  magick string, is FPX.
+%
+%  The format of the IsFPX method is:
+%
+%      MagickBooleanType IsFPX(const unsigned char *magick,const size_t length)
+%
+%  A description of each parameter follows:
+%
+%    o magick: This string is generally the first few bytes of an image file
+%      or blob.
+%
+%    o length: Specifies the length of the magick string.
+%
+*/
+static MagickBooleanType IsFPX(const unsigned char *magick,const size_t length)
+{
+  if (length < 4)
+    return(MagickFalse);
+  if (memcmp(magick,"\320\317\021\340",4) == 0)
+    return(MagickTrue);
+  return(MagickFalse);
+}
+
+#if defined(MAGICKCORE_FPX_DELEGATE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -80,7 +124,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadFPXImage reads a FlashPix image file and returns it.  It
+%  ReadFPXImage() reads a FlashPix image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.  This method was contributed by BillR@corbis.com.
 %
@@ -90,21 +134,13 @@ static unsigned int
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadFPXImage returns a pointer to the image after
-%      reading. A null image is returned if there is a memory shortage or if
-%      the image cannot be read.
-%
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  FILE
-    *file;
-
   FPXColorspace
     colorspace;
 
@@ -132,12 +168,12 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   IndexPacket
     index;
 
-  int
-    c;
-
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register IndexPacket
     *indexes;
 
@@ -154,55 +190,52 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *g,
     *r;
 
+  size_t
+    memory_limit;
+
   unsigned char
     *scanline;
 
   unsigned int
-    status,
-    subimage;
-
-  unsigned long
     height,
-    memory_limit,
     tile_width,
     tile_height,
     width;
 
+  unsigned long
+    scene;
+
   /*
     Open image.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenWarning,"Unable to open file",image);
-  /*
-    Copy image to temporary file.
-  */
-  TemporaryFilename((char *) image_info->filename);
-  file=fopen(image_info->filename,WriteBinaryType);
-  if (file == (FILE *) NULL)
-    ThrowReaderException(FileOpenWarning,"Unable to write file",image);
-  c=ReadBlobByte(image);
-  while (c != EOF)
-  {
-    (void) fputc(c,file);
-    c=ReadBlobByte(image);
-  }
-  (void) fclose(file);
-  (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
+  if (status == MagickFalse)
+    {
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   CloseBlob(image);
   /*
     Initialize FPX toolkit.
   */
+  fpx_status=FPX_InitSystem();
+  if (fpx_status != FPX_OK)
+    ThrowReaderException(CoderError,"UnableToInitializeFPXLibrary");
   memory_limit=20000000;
   fpx_status=FPX_SetToolkitMemoryLimit(&memory_limit);
   if (fpx_status != FPX_OK)
-    ThrowReaderException(DelegateWarning,"Unable to initialize FPX library",
-      image);
+    {
+      FPX_ClearSystem();
+      ThrowReaderException(CoderError,"UnableToInitializeFPXLibrary");
+    }
   tile_width=64;
   tile_height=64;
   flashpix=(FPXImageHandle *) NULL;
@@ -221,13 +254,20 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (fpx_status == FPX_LOW_MEMORY_ERROR)
     {
       FPX_ClearSystem();
-      ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
   if (fpx_status != FPX_OK)
     {
       FPX_ClearSystem();
-      ThrowReaderException(FileOpenWarning,"Unable to open FPX file",image);
+      ThrowFileException(exception,FileOpenError,"UnableToOpenFile",
+        image->filename);
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
+  if (colorspace.numberOfComponents == 0)
+    {
+      FPX_ClearSystem();
+      ThrowReaderException(CorruptImageError,"ImageTypeNotSupported");
     }
   if (image_info->view == (char *) NULL)
     {
@@ -240,8 +280,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       aspect_ratio=(float) width/height;
       fpx_status=FPX_GetImageResultAspectRatio(flashpix,&aspect_ratio);
       if (fpx_status != FPX_OK)
-        ThrowReaderException(DelegateWarning,"Unable to read aspect ratio",
-          image);
+        ThrowReaderException(DelegateError,"UnableToReadAspectRatio");
       if (width != (unsigned long) ((aspect_ratio*height)+0.5))
         Swap(width,height);
     }
@@ -249,7 +288,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (fpx_status != FPX_OK)
     {
       FPX_ClearSystem();
-      ThrowReaderException(FileOpenWarning,"Unable to read summary info",image);
+      ThrowReaderException(DelegateError,"UnableToReadSummaryInfo");
     }
   if (summary_info.title_valid)
     if ((summary_info.title.length != 0) &&
@@ -261,18 +300,19 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Note image label.
         */
-        label=(char *) AcquireMemory(summary_info.title.length+1);
+        label=(char *) NULL;
+        if (~summary_info.title.length >= MaxTextExtent)
+          label=(char *) AcquireQuantumMemory(summary_info.title.length+
+            MaxTextExtent,sizeof(*label));
         if (label == (char *) NULL)
           {
             FPX_ClearSystem();
-            ThrowReaderException(ResourceLimitWarning,
-              "Memory allocation failed",image);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
-        (void) strncpy(label,(char *) summary_info.title.ptr,
-          summary_info.title.length);
-        label[summary_info.title.length]='\0';
-        (void) SetImageAttribute(image,"label",label);
-        LiberateMemory((void **) &label);
+        (void) CopyMagickString(label,(char *) summary_info.title.ptr,
+          summary_info.title.length+1);
+        (void) SetImageProperty(image,"label",label);
+        label=DestroyString(label);
       }
   if (summary_info.comments_valid)
     if ((summary_info.comments.length != 0) &&
@@ -284,75 +324,80 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Note image comment.
         */
-        comments=(char *) AcquireMemory(summary_info.comments.length+1);
+        comments=(char *) NULL;
+        if (~summary_info.comments.length >= MaxTextExtent)
+          comments=(char *) AcquireQuantumMemory(summary_info.comments.length+
+            MaxTextExtent,sizeof(*comments));
         if (comments == (char *) NULL)
           {
             FPX_ClearSystem();
-            ThrowReaderException(ResourceLimitWarning,
-              "Memory allocation failed",image);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
-        (void) strncpy(comments,(char *) summary_info.comments.ptr,
-          summary_info.comments.length);
-        comments[summary_info.comments.length]='\0';
-        (void) SetImageAttribute(image,"comment",comments);
-        LiberateMemory((void **) &comments);
+        (void) CopyMagickString(comments,(char *) summary_info.comments.ptr,
+          summary_info.comments.length+1);
+        (void) SetImageProperty(image,"comment",comments);
+        comments=DestroyString(comments);
       }
   /*
-    Determine resolution by subimage specification.
+    Determine resolution by scene specification.
   */
   for (i=1; ; i++)
     if (((width >> i) < tile_width) || ((height >> i) < tile_height))
       break;
-  subimage=i;
-  if (image_info->subrange != 0)
-    while (subimage > image_info->subimage)
+  scene=i;
+  if (image_info->number_scenes != 0)
+    while (scene > image_info->scene)
     {
       width>>=1;
       height>>=1;
-      subimage--;
+      scene--;
     }
   if (image_info->size != (char *) NULL)
     while ((width > image->columns) || (height > image->rows))
     {
       width>>=1;
       height>>=1;
-      subimage--;
+      scene--;
     }
   image->depth=8;
   image->columns=width;
   image->rows=height;
   if ((colorspace.numberOfComponents % 2) == 0)
-    image->matte=True;
+    image->matte=MagickTrue;
   if (colorspace.numberOfComponents == 1)
     {
       /*
         Create linear colormap.
       */
-      if (!AllocateImageColormap(image,MaxRGB+1))
+      if (AllocateImageColormap(image,MaxColormapSize) == MagickFalse)
         {
           FPX_ClearSystem();
-          ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-            image);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
     }
-  if (image_info->ping)
+  if (image_info->ping != MagickFalse)
     {
       (void) FPX_CloseImage(flashpix);
       FPX_ClearSystem();
-      (void) remove(image->filename);
-      return(image);
+      return(GetFirstImageInList(image));
+    }
+  if (SetImageExtent(image,0,0) == MagickFalse)
+    {
+      (void) FPX_CloseImage(flashpix);
+      FPX_ClearSystem();
+      InheritException(exception,&image->exception);
+      return(DestroyImageList(image));
     }
   /*
     Allocate memory for the image and pixel buffer.
   */
-  scanline=(unsigned char *) AcquireMemory(colorspace.numberOfComponents*
-    image->columns*(tile_height+1));
+  scanline=(unsigned char *) AcquireQuantumMemory(image->columns,
+    (tile_height+1UL)*colorspace.numberOfComponents*sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
     {
       FPX_ClearSystem();
       (void) FPX_CloseImage(flashpix);
-      ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
   /*
     Initialize FlashPix image description.
@@ -395,7 +440,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         if (image_info->view != (char *) NULL)
           fpx_status=FPX_ReadImageRectangle(flashpix,0,y,image->columns,y+
-            tile_height-1,subimage,&fpx_info);
+            tile_height-1,scene,&fpx_info);
         else
           fpx_status=FPX_ReadImageTransformRectangle(flashpix,0.0F,
             (float) y/image->rows,(float) image->columns/image->rows,
@@ -403,11 +448,10 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             (long) tile_height,&fpx_info);
         if (fpx_status == FPX_LOW_MEMORY_ERROR)
           {
-            LiberateMemory((void **) &scanline);
+            scanline=(unsigned char *) RelinquishMagickMemory(scanline);
             (void) FPX_CloseImage(flashpix);
             FPX_ClearSystem();
-            ThrowReaderException(ResourceLimitWarning,
-              "Memory allocation failed",image);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
       }
     /*
@@ -421,43 +465,41 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       if (fpx_info.numberOfComponents > 2)
         {
-          q->red=Upscale(*r);
-          q->green=Upscale(*g);
-          q->blue=Upscale(*b);
+          q->red=ScaleCharToQuantum(*r);
+          q->green=ScaleCharToQuantum(*g);
+          q->blue=ScaleCharToQuantum(*b);
         }
       else
         {
-          index=Upscale(*r);
+          index=ScaleCharToQuantum(*r);
           indexes[x]=index;
           q->red=index;
           q->green=index;
           q->blue=index;
         }
-      if (image->matte)
-        q->opacity=MaxRGB-Upscale(*a);
+      if (image->matte != MagickFalse)
+        q->opacity=ScaleCharToQuantum(255-*a);
       q++;
       r+=red_component->columnStride;
       g+=green_component->columnStride;
       b+=blue_component->columnStride;
       a+=alpha_component->columnStride;
     }
-    if (!SyncImagePixels(image))
+    if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
-  LiberateMemory((void **) &scanline);
+  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   (void) FPX_CloseImage(flashpix);
   FPX_ClearSystem();
-  (void) remove(image->filename);
-  return(image);
-}
-#else
-static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
-{
-  ThrowException(exception,MissingDelegateWarning,
-    "FPX library is not available",image_info->filename);
-  return((Image *) NULL);
+  return(GetFirstImageInList(image));
 }
 #endif
 
@@ -472,7 +514,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterFPXImage adds attributes for the FPX image format to
+%  RegisterFPXImage() adds attributes for the FPX image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -481,21 +523,27 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterFPXImage method is:
 %
-%      RegisterFPXImage(void)
+%      unsigned long RegisterFPXImage(void)
 %
 */
-ModuleExport void RegisterFPXImage(void)
+ModuleExport unsigned long RegisterFPXImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("FPX");
-  entry->decoder=ReadFPXImage;
-  entry->encoder=WriteFPXImage;
-  entry->adjoin=False;
-  entry->description=AcquireString("FlashPix Format");
-  entry->module=AcquireString("FPX");
+#if defined(MAGICKCORE_FPX_DELEGATE)
+  entry->decoder=(DecodeImageHandler *) ReadFPXImage;
+  entry->encoder=(EncodeImageHandler *) WriteFPXImage;
+#endif
+  entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
+  entry->blob_support=MagickFalse;
+  entry->magick=(IsImageFormatHandler *) IsFPX;
+  entry->description=ConstantString("FlashPix Format");
+  entry->module=ConstantString("FPX");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -509,7 +557,7 @@ ModuleExport void RegisterFPXImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterFPXImage removes format registrations made by the
+%  UnregisterFPXImage() removes format registrations made by the
 %  FPX module from the list of supported formats.
 %
 %  The format of the UnregisterFPXImage method is:
@@ -522,7 +570,7 @@ ModuleExport void UnregisterFPXImage(void)
   (void) UnregisterMagickInfo("FPX");
 }
 
-#if defined(HasFPX)
+#if defined(MAGICKCORE_FPX_DELEGATE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -534,23 +582,18 @@ ModuleExport void UnregisterFPXImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteFPXImage writes an image in the FlashPix image format.  This
+%  WriteFPXImage() writes an image in the FlashPix image format.  This
 %  method was contributed by BillR@corbis.com.
 %
 %  The format of the WriteFPXImage method is:
 %
-%      unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o fpx_status: Method WriteFPXImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
-%    o image_info: Specifies a pointer to an ImageInfo structure.
-%
-%    o image:  A pointer to a Image structure.
+%    o image_info: The image info.
 %
+%    o image:  The image.
 %
 */
 
@@ -735,11 +778,8 @@ static void SetSaturation(double saturation,FPXColorTwistMatrix *color_twist)
   *color_twist=result;
 }
 
-static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
 {
-  char
-    filename[MaxTextExtent];
-
   FPXBackground
     background_color;
 
@@ -755,7 +795,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       }
     };
 
-  const ImageAttribute
+  const char
     *comment,
     *label;
 
@@ -777,17 +817,25 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  QuantumInfo
+    quantum_info;
+
+  register const PixelPacket
+    *p;
+
   register long
     i;
 
+  size_t
+    memory_limit;
+
   unsigned char
     *pixels;
 
   unsigned int
-    status;
-
-  unsigned long
-    memory_limit,
     tile_height,
     tile_width;
 
@@ -798,23 +846,26 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenWarning,"Unable to open file",image);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  if (status == MagickFalse)
+    return(status);
+  CloseBlob(image);
   /*
     Initialize FPX toolkit.
   */
   image->depth=8;
-  (void) TransformRGBImage(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   memory_limit=20000000;
   fpx_status=FPX_SetToolkitMemoryLimit(&memory_limit);
   if (fpx_status != FPX_OK)
-    ThrowWriterException(ResourceLimitWarning,
-      "Unable to initialize FPX library",image);
+    ThrowWriterException(DelegateError,"UnableToInitializeFPXLibrary");
   tile_width=64;
   tile_height=64;
   colorspace.numberOfComponents=3;
-  if (image->matte)
+  if (image->matte != MagickFalse)
     colorspace.numberOfComponents=4;
   if ((image_info->type != TrueColorType) &&
       IsGrayImage(image,&image->exception))
@@ -827,14 +878,8 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   background_color.color3_value=0;
   background_color.color4_value=0;
   compression=NONE;
-  if (image_info->compression == JPEGCompression)
+  if (image->compression == JPEGCompression)
     compression=JPEG_UNSPECIFIED;
-  (void) strncpy(filename,image->filename,MaxTextExtent-1);
-  if ((image->file == stdout) || image->pipet ||
-      (image->blob->data != (unsigned char *) NULL))
-    TemporaryFilename(filename);
-  else
-    CloseBlob(image);
   {
 #if defined(macintosh)
     FSSpec
@@ -843,90 +888,87 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
     FilenameToFSSpec(filename,&fsspec);
     fpx_status=FPX_CreateImageByFilename((const FSSpec &) fsspec,image->columns,
 #else
-    fpx_status=FPX_CreateImageByFilename(filename,image->columns,
+    fpx_status=FPX_CreateImageByFilename(image->filename,image->columns,
 #endif
       image->rows,tile_width,tile_height,colorspace,background_color,
       compression,&flashpix);
   }
   if (fpx_status != FPX_OK)
-    ThrowWriterException(FileOpenWarning,"Unable to open file",image);
-  if (image_info->compression == JPEGCompression)
+    return(status);
+  if (compression == JPEG_UNSPECIFIED)
     {
       /*
         Initialize the compression by quality for the entire image.
       */
-      fpx_status=
-        FPX_SetJPEGCompression(flashpix,(unsigned short) (image_info->quality));
+      fpx_status=FPX_SetJPEGCompression(flashpix,(unsigned short)
+        image->quality == UndefinedCompressionQuality ? 75 : image->quality);
       if (fpx_status != FPX_OK)
-        ThrowWriterException(DelegateWarning,"Unable to set JPEG level",image);
+        ThrowWriterException(DelegateError,"UnableToSetJPEGLevel");
     }
   /*
     Set image summary info.
   */
-  summary_info.title_valid=False;
-  summary_info.subject_valid=False;
-  summary_info.author_valid=False;
-  summary_info.comments_valid=False;
-  summary_info.keywords_valid=False;
-  summary_info.OLEtemplate_valid=False;
-  summary_info.last_author_valid=False;
-  summary_info.rev_number_valid=False;
-  summary_info.edit_time_valid=False;
-  summary_info.last_printed_valid=False;
-  summary_info.create_dtm_valid=False;
-  summary_info.last_save_dtm_valid=False;
-  summary_info.page_count_valid=False;
-  summary_info.word_count_valid=False;
-  summary_info.char_count_valid=False;
-  summary_info.thumbnail_valid=False;
-  summary_info.appname_valid=False;
-  summary_info.security_valid=False;
-  label=GetImageAttribute(image,"label");
-  if (label != (ImageAttribute *) NULL)
+  summary_info.title_valid=MagickFalse;
+  summary_info.subject_valid=MagickFalse;
+  summary_info.author_valid=MagickFalse;
+  summary_info.comments_valid=MagickFalse;
+  summary_info.keywords_valid=MagickFalse;
+  summary_info.OLEtemplate_valid=MagickFalse;
+  summary_info.last_author_valid=MagickFalse;
+  summary_info.rev_number_valid=MagickFalse;
+  summary_info.edit_time_valid=MagickFalse;
+  summary_info.last_printed_valid=MagickFalse;
+  summary_info.create_dtm_valid=MagickFalse;
+  summary_info.last_save_dtm_valid=MagickFalse;
+  summary_info.page_count_valid=MagickFalse;
+  summary_info.word_count_valid=MagickFalse;
+  summary_info.char_count_valid=MagickFalse;
+  summary_info.thumbnail_valid=MagickFalse;
+  summary_info.appname_valid=MagickFalse;
+  summary_info.security_valid=MagickFalse;
+  label=GetImageProperty(image,"label");
+  if (label != (const char *) NULL)
     {
+      size_t
+        length;
+
       /*
         Note image label.
       */
-      summary_info.title_valid=True;
-      summary_info.title.length=strlen(label->value);
-      summary_info.title.ptr=(unsigned char *)
-        AcquireMemory(strlen(label->value)+1);
-      if (summary_info.title.ptr != (unsigned char *) NULL)
-        (void) strncpy((char *) summary_info.title.ptr,label->value,
-          MaxTextExtent-1);
-      else
-        ThrowWriterException(DelegateWarning,"Unable to set image title",image);
+      summary_info.title_valid=MagickTrue;
+      length=strlen(label);
+      summary_info.title.length=length;
+      if (~length >= MaxTextExtent)
+        summary_info.title.ptr=(unsigned char *) AcquireQuantumMemory(
+          length+MaxTextExtent,sizeof(*summary_info.title.ptr));
+      if (summary_info.title.ptr == (unsigned char *) NULL)
+        ThrowWriterException(DelegateError,"UnableToSetImageTitle");
+      (void) CopyMagickString((char *) summary_info.title.ptr,label,
+        MaxTextExtent);
     }
-  comment=GetImageAttribute(image,"comment");
-  if (comment != (ImageAttribute *) NULL)
+  comment=GetImageProperty(image,"comment");
+  if (comment != (const char *) NULL)
     {
       /*
         Note image comment.
       */
-      summary_info.comments_valid=True;
-      summary_info.comments.length=strlen(comment->value);
-      summary_info.comments.ptr=(unsigned char *)
-        AcquireMemory(strlen(comment->value)+1);
-      if (summary_info.comments.ptr != (unsigned char *) NULL)
-        (void) strcpy((char *) summary_info.comments.ptr,comment->value);
-      else
-        ThrowWriterException(DelegateWarning,"Unable to set image comments",
-          image);
+      summary_info.comments_valid=MagickTrue;
+      summary_info.comments.ptr=(unsigned char *) AcquireString(comment);
+      summary_info.comments.length=strlen(comment);
     }
   fpx_status=FPX_SetSummaryInformation(flashpix,&summary_info);
   if (fpx_status != FPX_OK)
-    ThrowWriterException(DelegateWarning,"Unable to set summary info",image);
+    ThrowWriterException(DelegateError,"UnableToSetSummaryInfo");
   /*
     Allocate pixels.
   */
-  pixels=(unsigned char *)
-    AcquireMemory(colorspace.numberOfComponents*image->columns);
+  pixels=(unsigned char *) AcquireQuantumMemory(image->columns,
+    colorspace.numberOfComponents*sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     {
       (void) FPX_CloseImage(flashpix);
       FPX_ClearSystem();
-      ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",
-        image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     }
   /*
     Initialize FlashPix image description.
@@ -950,22 +992,30 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   /*
     Write image scanlines.
   */
+  GetQuantumInfo(image_info,&quantum_info);
   for (y=0; y < (long) image->rows; y++)
   {
-    if (!AcquireImagePixels(image,0,y,image->columns,1,&image->exception))
+    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    if (p == (const PixelPacket *) NULL)
       break;
     if (fpx_info.numberOfComponents == 1)
-      (void) PopImagePixels(image,GrayQuantum,pixels);
+      (void) ImportQuantumPixels(image,&quantum_info,GrayQuantum,pixels);
     else
-      if (!image->matte)
-        (void) PopImagePixels(image,RGBQuantum,pixels);
+      if (image->matte == MagickFalse)
+        (void) ImportQuantumPixels(image,&quantum_info,RGBQuantum,pixels);
       else
-        (void) PopImagePixels(image,RGBAQuantum,pixels);
+        (void) ImportQuantumPixels(image,&quantum_info,RGBAQuantum,pixels);
     fpx_status=FPX_WriteImageLine(flashpix,&fpx_info);
     if (fpx_status != FPX_OK)
       break;
-    if (QuantumTick(y,image->rows))
-      MagickMonitor(SaveImageText,y,image->rows);
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(SaveImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   if (image_info->view != (char *) NULL)
     {
@@ -987,7 +1037,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       FPXROI
         view_rect;
 
-      unsigned int
+      MagickBooleanType
         affine_valid,
         aspect_ratio_valid,
         color_twist_valid,
@@ -999,7 +1049,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
         Initialize default viewing parameters.
       */
       contrast=1.0;
-      contrast_valid=False;
+      contrast_valid=MagickFalse;
       color_twist.byy=1.0;
       color_twist.byc1=0.0;
       color_twist.byc2=0.0;
@@ -1016,16 +1066,16 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       color_twist.dummy5_zero=0.0;
       color_twist.dummy6_zero=0.0;
       color_twist.dummy7_one=1.0;
-      color_twist_valid=False;
+      color_twist_valid=MagickFalse;
       sharpen=0.0;
-      sharpen_valid=False;
+      sharpen_valid=MagickFalse;
       aspect_ratio=(double) image->columns/image->rows;
-      aspect_ratio_valid=False;
+      aspect_ratio_valid=MagickFalse;
       view_rect.left=(float) 0.1;
       view_rect.width=aspect_ratio-0.2;
       view_rect.top=(float) 0.1;
       view_rect.height=(float) 0.8; /* 1.0-0.2 */
-      view_rect_valid=False;
+      view_rect_valid=MagickFalse;
       affine.a11=1.0;
       affine.a12=0.0;
       affine.a13=0.0;
@@ -1042,7 +1092,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       affine.a42=0.0;
       affine.a43=0.0;
       affine.a44=1.0;
-      affine_valid=False;
+      affine_valid=MagickFalse;
       if (0)
         {
           /*
@@ -1051,81 +1101,48 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
           SetBrightness(0.5,&color_twist);
           SetSaturation(0.5,&color_twist);
           SetColorBalance(0.5,1.0,1.0,&color_twist);
-          color_twist_valid=True;
+          color_twist_valid=MagickTrue;
         }
       if (affine_valid)
         {
           fpx_status=FPX_SetImageAffineMatrix(flashpix,&affine);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateWarning,"Unable to set affine matrix",
-              image);
+            ThrowWriterException(DelegateError,"UnableToSetAffineMatrix");
         }
       if (aspect_ratio_valid)
         {
           fpx_status=FPX_SetImageResultAspectRatio(flashpix,&aspect_ratio);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateWarning,"Unable to set aspect ratio",
-              image);
+            ThrowWriterException(DelegateError,"UnableToSetAspectRatio");
         }
       if (color_twist_valid)
         {
           fpx_status=FPX_SetImageColorTwistMatrix(flashpix,&color_twist);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateWarning,
-              "Unable to set color color twist",image);
+            ThrowWriterException(DelegateError,"UnableToSetColorTwist");
         }
       if (contrast_valid)
         {
           fpx_status=FPX_SetImageContrastAdjustment(flashpix,&contrast);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateWarning,"Unable to set contrast",
-              image);
+            ThrowWriterException(DelegateError,"UnableToSetContrast");
         }
       if (sharpen_valid)
         {
           fpx_status=FPX_SetImageFilteringValue(flashpix,&sharpen);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateWarning,
-              "Unable to set filtering value",image);
+            ThrowWriterException(DelegateError,"UnableToSetFilteringValue");
         }
       if (view_rect_valid)
         {
           fpx_status=FPX_SetImageROI(flashpix,&view_rect);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateWarning,
-              "Unable to set region of interest",image);
+            ThrowWriterException(DelegateError,"UnableToSetRegionOfInterest");
         }
     }
   (void) FPX_CloseImage(flashpix);
   FPX_ClearSystem();
-  LiberateMemory((void **) &pixels);
-  if ((image->file == stdout) || image->pipet ||
-      (image->blob->data != (unsigned char *) NULL))
-    {
-      FILE
-        *file;
-
-      int
-        c;
-
-      /*
-        Copy temporary file to image blob->
-      */
-      file=fopen(filename,ReadBinaryType);
-      if (file == (FILE *) NULL)
-        ThrowWriterException(FileOpenWarning,"Unable to open file",image);
-      for (c=fgetc(file); c != EOF; c=fgetc(file))
-        (void) WriteBlobByte(image,c);
-      (void) fclose(file);
-      (void) remove(filename);
-      CloseBlob(image);
-    }
-  return(True);
-}
-#else
-static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
-{
-  ThrowBinaryException(MissingDelegateWarning,"FPX library is not available",
-    image->filename)
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+  return(MagickTrue);
 }
 #endif
