@@ -17,7 +17,7 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,7 +41,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/attribute.h"
+#include "magick/artifact.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/cache.h"
@@ -60,47 +60,14 @@
 #include "magick/monitor.h"
 #include "magick/option.h"
 #include "magick/profile.h"
+#include "magick/property.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 #include "magick/utility.h"
 
-/*
-  Define declarations.
-*/
-#define PopCharPixel(q,pixel) \
-{ \
-  *(q)++=(unsigned char) (pixel); \
-}
-#define PopLongPixel(q,pixel) \
-{ \
-  *(q)++=(unsigned char) ((pixel) >> 24); \
-  *(q)++=(unsigned char) ((pixel) >> 16); \
-  *(q)++=(unsigned char) ((pixel) >> 8); \
-  *(q)++=(unsigned char) (pixel); \
-}
-#define PopShortPixel(q,pixel) \
-{ \
-  *(q)++=(unsigned char) ((pixel) >> 8); \
-  *(q)++=(unsigned char) (pixel); \
-}
-#define PushCharPixel(pixel,p) \
-{ \
-  pixel=(unsigned long) (*(p)); \
-  (p)++; \
-}
-#define PushLongPixel(pixel,p) \
-{ \
-  pixel=(unsigned long) \
-    ((*(p) << 24) | (*((p)+1) << 16) | (*((p)+2) << 8) | *((p)+3)); \
-  (p)+=4; \
-}
-#define PushShortPixel(pixel,p) \
-{ \
-  pixel=(unsigned long) ((*(p) << 8) | *((p)+1)); \
-  (p)+=2; \
-}
-
 /*
   Forward declarations.
 */
@@ -132,7 +99,6 @@ static MagickBooleanType
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsMPC(const unsigned char *magick,const size_t length)
 {
@@ -170,7 +136,6 @@ static MagickBooleanType IsMPC(const unsigned char *magick,const size_t length)
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
@@ -180,6 +145,9 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     keyword[MaxTextExtent],
     *options;
 
+  const unsigned char
+    *p;
+
   GeometryInfo
     geometry_info;
 
@@ -201,12 +169,15 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickStatusType
     flags;
 
+  QuantumInfo
+    quantum_info;
+
+  QuantumState
+    quantum_state;
+
   register long
     i;
 
-  register unsigned char
-    *p;
-
   size_t
     length;
 
@@ -256,9 +227,11 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     profiles=(LinkedListInfo *) NULL;
     length=MaxTextExtent;
     options=AcquireString((char *) NULL);
-    quantum_depth=QuantumDepth;
+    quantum_depth=MAGICKCORE_QUANTUM_DEPTH;
     image->depth=8;
     image->compression=NoCompression;
+    GetQuantumInfo(image_info,&quantum_info);
+    InitializeQuantumState(&quantum_info,MSBEndian,&quantum_state);
     while ((isgraph(c) != MagickFalse) && (c != (int) ':'))
     {
       register char
@@ -283,8 +256,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 *p='\0';
                 length<<=1;
-                comment=(char *) ResizeMagickMemory(comment,
-                  (length+MaxTextExtent)*sizeof(*comment));
+                comment=(char *) ResizeQuantumMemory(comment,length+
+                  MaxTextExtent,sizeof(*comment));
                 if (comment == (char *) NULL)
                   break;
                 p=comment+strlen(comment);
@@ -294,8 +267,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (comment == (char *) NULL)
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           *p='\0';
-          (void) SetImageAttribute(image,"Comment",comment);
-          comment=(char *) RelinquishMagickMemory(comment);
+          (void) SetImageProperty(image,"comment",comment);
+          comment=DestroyString(comment);
           c=ReadBlobByte(image);
         }
       else
@@ -311,7 +284,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               if (c == (int) '=')
                 break;
-              if ((size_t) (p-keyword) < (MaxTextExtent/2))
+              if ((size_t) (p-keyword) < MaxTextExtent)
                 *p++=(char) c;
               c=ReadBlobByte(image);
             } while (c != EOF);
@@ -331,8 +304,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     {
                       *p='\0';
                       length<<=1;
-                      options=(char *) ResizeMagickMemory(options,
-                        (length+MaxTextExtent)*sizeof(*options));
+                      options=(char *) ResizeQuantumMemory(options,length+
+                        MaxTextExtent,sizeof(*options));
                       if (options == (char *) NULL)
                         break;
                       p=options+strlen(options);
@@ -380,7 +353,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                       exception);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,options);
+                (void) SetImageProperty(image,keyword,options);
                 break;
               }
               case 'c':
@@ -406,7 +379,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if (LocaleCompare(keyword,"compression") == 0)
                   {
                     image->compression=(CompressionType) ParseMagickOption(
-                      MagickCompressionOptions,MagickFalse,options);
+                      MagickCompressOptions,MagickFalse,options);
                     break;
                   }
                 if (LocaleCompare(keyword,"columns") == 0)
@@ -414,7 +387,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     image->columns=(unsigned long) atol(options);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,options);
+                (void) SetImageProperty(image,keyword,options);
                 break;
               }
               case 'd':
@@ -436,7 +409,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                       MagickDisposeOptions,MagickFalse,options);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,options);
+                (void) SetImageProperty(image,keyword,options);
                 break;
               }
               case 'e':
@@ -453,7 +426,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     image->error.mean_error_per_pixel=atof(options);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,options);
+                (void) SetImageProperty(image,keyword,options);
                 break;
               }
               case 'g':
@@ -474,7 +447,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                         image->chromaticity.green_primary.x;
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,options);
+                (void) SetImageProperty(image,keyword,options);
                 break;
               }
               case 'i':
@@ -490,7 +463,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     image->iterations=(unsigned long) atol(options);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,options);
+                (void) SetImageProperty(image,keyword,options);
                 break;
               }
               case 'm':
@@ -523,7 +496,25 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     (void) CloneString(&image->montage,options);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,options);
+                (void) SetImageProperty(image,keyword,options);
+                break;
+              }
+              case 'o':
+              case 'O':
+              {
+                if (LocaleCompare(keyword,"opaque") == 0)
+                  {
+                    image->matte=(MagickBooleanType) ParseMagickOption(
+                      MagickBooleanOptions,MagickFalse,options);
+                    break;
+                  }
+                if (LocaleCompare(keyword,"orientation") == 0)
+                  {
+                    image->orientation=(OrientationType) ParseMagickOption(
+                      MagickOrientationOptions,MagickFalse,options);
+                    break;
+                  }
+                (void) SetImageProperty(image,keyword,options);
                 break;
               }
               case 'p':
@@ -536,7 +527,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
                     geometry=GetPageGeometry(options);
                     (void) ParseAbsoluteGeometry(geometry,&image->page);
-                    geometry=(char *) RelinquishMagickMemory(geometry);
+                    geometry=DestroyString(geometry);
                     break;
                   }
                 if (LocaleNCompare(keyword,"profile-",8) == 0)
@@ -550,19 +541,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     profile=DestroyStringInfo(profile);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,options);
-                break;
-              }
-              case 'o':
-              case 'O':
-              {
-                if (LocaleCompare(keyword,"opaque") == 0)
-                  {
-                    image->matte=(MagickBooleanType) ParseMagickOption(
-                      MagickBooleanOptions,MagickFalse,options);
-                    break;
-                  }
-                (void) SetImageAttribute(image,keyword,options);
+                (void) SetImageProperty(image,keyword,options);
                 break;
               }
               case 'q':
@@ -578,7 +557,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     quantum_depth=(unsigned long) atol(options);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,options);
+                (void) SetImageProperty(image,keyword,options);
                 break;
               }
               case 'r':
@@ -612,7 +591,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     image->rows=(unsigned long) atol(options);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,options);
+                (void) SetImageProperty(image,keyword,options);
                 break;
               }
               case 's':
@@ -623,7 +602,27 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     image->scene=(unsigned long) atol(options);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,options);
+                (void) SetImageProperty(image,keyword,options);
+                break;
+              }
+              case 't':
+              case 'T':
+              {
+                if (LocaleCompare(keyword,"ticks-per-second") == 0)
+                  {
+                    image->ticks_per_second=(long) atol(options);
+                    break;
+                  }
+                if (LocaleCompare(keyword,"tile-offset") == 0)
+                  {
+                    char
+                      *geometry;
+
+                    geometry=GetPageGeometry(options);
+                    (void) ParseAbsoluteGeometry(geometry,&image->tile_offset);
+                    geometry=DestroyString(geometry);
+                  }
+                (void) SetImageProperty(image,keyword,options);
                 break;
               }
               case 'u':
@@ -635,7 +634,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                       MagickResolutionOptions,MagickFalse,options);
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,options);
+                (void) SetImageProperty(image,keyword,options);
                 break;
               }
               case 'w':
@@ -651,12 +650,12 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                         image->chromaticity.white_point.x;
                     break;
                   }
-                (void) SetImageAttribute(image,keyword,options);
+                (void) SetImageProperty(image,keyword,options);
                 break;
               }
               default:
               {
-                (void) SetImageAttribute(image,keyword,options);
+                (void) SetImageProperty(image,keyword,options);
                 break;
               }
             }
@@ -666,7 +665,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
       while (isspace((int) ((unsigned char) c)) != 0)
         c=ReadBlobByte(image);
     }
-    options=(char *) RelinquishMagickMemory(options);
+    options=DestroyString(options);
     (void) ReadBlobByte(image);
     /*
       Verify that required image information is defined.
@@ -676,7 +675,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (image->compression == UndefinedCompression) || (image->columns == 0) ||
         (image->rows == 0))
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-    if (quantum_depth != QuantumDepth)
+    if (quantum_depth != MAGICKCORE_QUANTUM_DEPTH)
       ThrowReaderException(CacheError,"InconsistentPersistentCacheDepth");
     if (image->montage != (char *) NULL)
       {
@@ -698,8 +697,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 Allocate more memory for the image directory.
               */
               length<<=1;
-              image->directory=(char *) ResizeMagickMemory(image->directory,
-                (length+MaxTextExtent)*sizeof(*image->directory));
+              image->directory=(char *) ResizeQuantumMemory(image->directory,
+                length+MaxTextExtent,sizeof(*image->directory));
               if (image->directory == (char *) NULL)
                 ThrowReaderException(CorruptImageError,"UnableToReadImageData");
               p=image->directory+strlen(image->directory);
@@ -713,6 +712,12 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         const char
           *name;
 
+        const StringInfo
+          *profile;
+
+        const unsigned char
+          *p;
+
         /*
           Read image profiles.
         */
@@ -722,7 +727,11 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         {
           profile=GetImageProfile(image,name);
           if (profile != (StringInfo *) NULL)
-            count=ReadBlob(image,profile->length,profile->datum);
+            {
+              p=GetStringInfoDatum(profile);
+              count=ReadBlob(image,GetStringInfoLength(profile),
+                (unsigned char *) p);
+            }
           name=(const char *) GetNextValueInLinkedList(profiles);
         }
         profiles=DestroyLinkedList(profiles,RelinquishMagickMemory);
@@ -743,15 +752,12 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             unsigned char
               *colormap;
 
-            unsigned long
-              pixel;
-
             /*
               Read image colormap from file.
             */
-            packet_size=(size_t) (3*depth/8);
-            colormap=(unsigned char *)
-              AcquireMagickMemory(packet_size*image->colors);
+            packet_size=(size_t) (3UL*depth/8UL);
+            colormap=(unsigned char *) AcquireQuantumMemory(image->colors,
+              packet_size*sizeof(*colormap));
             if (colormap == (unsigned char *) NULL)
               ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
             count=ReadBlob(image,packet_size*image->colors,colormap);
@@ -766,39 +772,48 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   "ImageDepthNotSupported");
               case 8:
               {
+                unsigned char
+                  pixel;
+
                 for (i=0; i < (long) image->colors; i++)
                 {
-                  PushCharPixel(pixel,p);
+                  pixel=PushCharPixel(&p);
                   image->colormap[i].red=ScaleCharToQuantum(pixel);
-                  PushCharPixel(pixel,p);
+                  pixel=PushCharPixel(&p);
                   image->colormap[i].green=ScaleCharToQuantum(pixel);
-                  PushCharPixel(pixel,p);
+                  pixel=PushCharPixel(&p);
                   image->colormap[i].blue=ScaleCharToQuantum(pixel);
                 }
                 break;
               }
               case 16:
               {
+                unsigned short
+                  pixel;
+
                 for (i=0; i < (long) image->colors; i++)
                 {
-                  PushShortPixel(pixel,p);
+                  pixel=PushShortPixel(&quantum_state,&p);
                   image->colormap[i].red=ScaleShortToQuantum(pixel);
-                  PushShortPixel(pixel,p);
+                  pixel=PushShortPixel(&quantum_state,&p);
                   image->colormap[i].green=ScaleShortToQuantum(pixel);
-                  PushShortPixel(pixel,p);
+                  pixel=PushShortPixel(&quantum_state,&p);
                   image->colormap[i].blue=ScaleShortToQuantum(pixel);
                 }
                 break;
               }
               case 32:
               {
+                unsigned long
+                  pixel;
+
                 for (i=0; i < (long) image->colors; i++)
                 {
-                  PushLongPixel(pixel,p);
+                  pixel=PushLongPixel(&quantum_state,&p);
                   image->colormap[i].red=ScaleLongToQuantum(pixel);
-                  PushLongPixel(pixel,p);
+                  pixel=PushLongPixel(&quantum_state,&p);
                   image->colormap[i].green=ScaleLongToQuantum(pixel);
-                  PushLongPixel(pixel,p);
+                  pixel=PushLongPixel(&quantum_state,&p);
                   image->colormap[i].blue=ScaleLongToQuantum(pixel);
                 }
                 break;
@@ -816,6 +831,11 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
+    if (SetImageExtent(image,0,0) == MagickFalse)
+      {
+        InheritException(exception,&image->exception);
+        return(DestroyImageList(image));
+      }
     /*
       Attach persistent pixel cache.
     */
@@ -865,8 +885,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RegisterMPCImage() adds attributes for the Cache image format to
-%  the list of supported formats.  The attributes include the image format
+%  RegisterMPCImage() adds properties for the Cache image format to
+%  the list of supported formats.  The properties include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
 %  whether the format supports native in-memory I/O, and a brief
@@ -874,25 +894,27 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterMPCImage method is:
 %
-%      RegisterMPCImage(void)
+%      unsigned long RegisterMPCImage(void)
 %
 */
-ModuleExport void RegisterMPCImage(void)
+ModuleExport unsigned long RegisterMPCImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("CACHE");
-  entry->description=AcquireString("Magick Persistent Cache image format");
-  entry->module=AcquireString("CACHE");
+  entry->description=ConstantString("Magick Persistent Cache image format");
+  entry->module=ConstantString("CACHE");
+  entry->stealth=MagickTrue;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("MPC");
-  entry->decoder=(DecoderHandler *) ReadMPCImage;
-  entry->encoder=(EncoderHandler *) WriteMPCImage;
-  entry->magick=(MagickHandler *) IsMPC;
-  entry->description=AcquireString("Magick Persistent Cache image format");
-  entry->module=AcquireString("MPC");
+  entry->decoder=(DecodeImageHandler *) ReadMPCImage;
+  entry->encoder=(EncodeImageHandler *) WriteMPCImage;
+  entry->magick=(IsImageFormatHandler *) IsMPC;
+  entry->description=ConstantString("Magick Persistent Cache image format");
+  entry->module=ConstantString("MPC");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -943,7 +965,6 @@ ModuleExport void UnregisterMPCImage(void)
 %
 %    o image: The image.
 %
-%
 */
 static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
 {
@@ -951,8 +972,9 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
     buffer[MaxTextExtent],
     cache_filename[MaxTextExtent];
 
-  const ImageAttribute
-    *attribute;
+  const char
+    *property,
+    *value;
 
   MagickBooleanType
     status;
@@ -961,6 +983,12 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
     offset,
     scene;
 
+  QuantumInfo
+    quantum_info;
+
+  QuantumState
+    quantum_state;
+
   register long
     i;
 
@@ -994,7 +1022,7 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
       image->storage_class=DirectClass;
     (void) WriteBlobString(image,"id=MagickCache\n");
     (void) FormatMagickString(buffer,MaxTextExtent,"quantum-depth=%d\n",
-      QuantumDepth);
+      MAGICKCORE_QUANTUM_DEPTH);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,
       "class=%s  colors=%lu  matte=%s\n",MagickOptionToMnemonic(
@@ -1021,7 +1049,7 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
       {
         (void) FormatMagickString(buffer,MaxTextExtent,
           "compression=%s  quality=%lu\n",
-          MagickOptionToMnemonic(MagickCompressionOptions,image->compression),
+          MagickOptionToMnemonic(MagickCompressOptions,image->compression),
           image->quality);
         (void) WriteBlobString(image,buffer);
       }
@@ -1050,15 +1078,24 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
             image->page.x,image->page.y);
           (void) WriteBlobString(image,buffer);
         }
-    if ((GetNextImageInList(image) != (Image *) NULL) || (GetPreviousImageInList(image) != (Image *) NULL))
+    if ((image->page.x != 0) || (image->page.y != 0))
+      {
+        (void) FormatMagickString(buffer,MaxTextExtent,"tile-offset=%+ld%+ld\n",
+          image->tile_offset.x,image->tile_offset.y);
+        (void) WriteBlobString(image,buffer);
+      }
+    if ((GetNextImageInList(image) != (Image *) NULL) ||
+        (GetPreviousImageInList(image) != (Image *) NULL))
       {
         if (image->scene == 0)
           (void) FormatMagickString(buffer,MaxTextExtent,
-            "iterations=%lu  delay=%lu\n",image->iterations,image->delay);
+            "iterations=%lu  delay=%lu  ticks-per-second=%lu\n",
+            image->iterations,image->delay,image->ticks_per_second);
         else
           (void) FormatMagickString(buffer,MaxTextExtent,
-            "scene=%lu  iterations=%lu  delay=%lu\n",image->scene,
-            image->iterations,image->delay);
+            "scene=%lu  iterations=%lu  delay=%lu  ticks-per-second=%lu\n",
+            image->scene,image->iterations,image->delay,
+            image->ticks_per_second);
         (void) WriteBlobString(image,buffer);
       }
     else
@@ -1081,6 +1118,12 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
               image->delay);
             (void) WriteBlobString(image,buffer);
           }
+        if (image->ticks_per_second != UndefinedTicksPerSecond)
+          {
+            (void) FormatMagickString(buffer,MaxTextExtent,
+              "ticks-per-second=%lu\n",image->ticks_per_second);
+            (void) WriteBlobString(image,buffer);
+          }
       }
     if (image->dispose != UndefinedDispose)
       {
@@ -1088,12 +1131,6 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
           MagickOptionToMnemonic(MagickDisposeOptions,image->dispose));
         (void) WriteBlobString(image,buffer);
       }
-    if (image->compose != UndefinedCompositeOp)
-      {
-        (void) FormatMagickString(buffer,MaxTextExtent,"compose=%s\n",
-          MagickOptionToMnemonic(MagickCompositeOptions,image->compose));
-        (void) WriteBlobString(image,buffer);
-      }
     if (image->rendering_intent != UndefinedIntent)
       {
         (void) FormatMagickString(buffer,MaxTextExtent,
@@ -1124,6 +1161,13 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
           image->chromaticity.white_point.x,image->chromaticity.white_point.y);
         (void) WriteBlobString(image,buffer);
       }
+    if (image->orientation != UndefinedOrientation)
+      {
+        (void) FormatMagickString(buffer,MaxTextExtent,
+          "orientation=%s\n",MagickOptionToMnemonic(MagickOrientationOptions,
+          image->orientation));
+        (void) WriteBlobString(image,buffer);
+      }
     if (image->profiles != (void *) NULL)
       {
         const char
@@ -1142,8 +1186,8 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
           profile=GetImageProfile(image,name);
           if (profile != (StringInfo *) NULL)
             {
-              (void) FormatMagickString(buffer,MaxTextExtent,
-                "profile-%s=%lu\n",name,(unsigned long) profile->length);
+              (void) FormatMagickString(buffer,MaxTextExtent,"profile-%s=%lu\n",
+                name,(unsigned long) GetStringInfoLength(profile));
               (void) WriteBlobString(image,buffer);
             }
           name=GetNextImageProfile(image);
@@ -1155,27 +1199,35 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
           image->montage);
         (void) WriteBlobString(image,buffer);
       }
-    ResetImageAttributeIterator(image);
-	  attribute=GetNextImageAttribute(image);
-		while (attribute != (const ImageAttribute *) NULL)
+    GetQuantumInfo(image_info,&quantum_info);
+    if (quantum_info.format == FloatingPointQuantumFormat)
+      {
+        (void) WriteBlobString(image,"quantum-format=floating-point\n");
+        quantum_info.scale=1.0/QuantumRange;
+      }
+    InitializeQuantumState(&quantum_info,MSBEndian,&quantum_state);
+    ResetImagePropertyIterator(image);
+    property=GetNextImageProperty(image);
+    while (property != (const char *) NULL)
     {
-      if (*attribute->key != '[')
+      (void) FormatMagickString(buffer,MaxTextExtent,"%s=",property);
+      (void) WriteBlobString(image,buffer);
+      value=GetImageProperty(image,property);
+      if (value != (const char *) NULL)
         {
-          (void) FormatMagickString(buffer,MaxTextExtent,"%s=",attribute->key);
-          (void) WriteBlobString(image,buffer);
-          for (i=0; i < (long) strlen(attribute->value); i++)
-            if (isspace((int) ((unsigned char) attribute->value[i])) != 0)
+          for (i=0; i < (long) strlen(value); i++)
+            if (isspace((int) ((unsigned char) value[i])) != 0)
               break;
-          if (i <= (long) strlen(attribute->value))
+          if (i <= (long) strlen(value))
             (void) WriteBlobByte(image,'{');
-          (void) WriteBlob(image,strlen(attribute->value),(unsigned char *)
-            attribute->value);
-          if (i <= (long) strlen(attribute->value))
+          (void) WriteBlob(image,strlen(value),(unsigned char *) value);
+          if (i <= (long) strlen(value))
             (void) WriteBlobByte(image,'}');
-          (void) WriteBlobByte(image,'\n');
         }
-  	  attribute=GetNextImageAttribute(image);
+      (void) WriteBlobByte(image,'\n');
+      property=GetNextImageProperty(image);
     }
+    ResetImageArtifactIterator(image);
     (void) WriteBlobString(image,"\f\n:\032");
     if (image->montage != (char *) NULL)
       {
@@ -1202,30 +1254,26 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
         while (name != (const char *) NULL)
         {
           profile=GetImageProfile(image,name);
-          (void) WriteBlob(image,profile->length,profile->datum);
+          (void) WriteBlob(image,GetStringInfoLength(profile),
+            GetStringInfoDatum(profile));
           name=GetNextImageProfile(image);
         }
       }
     if (image->storage_class == PseudoClass)
       {
-        register unsigned char
-          *q;
-
         size_t
           packet_size;
 
         unsigned char
-          *colormap;
-
-        unsigned long
-          pixel;
+          *colormap,
+          *q;
 
         /*
           Allocate colormap.
         */
-        packet_size=(size_t) (3*depth/8);
-        colormap=(unsigned char *)
-          AcquireMagickMemory(packet_size*image->colors*sizeof(*colormap));
+        packet_size=(size_t) (3UL*depth/8UL);
+        colormap=(unsigned char *) AcquireQuantumMemory(image->colors,
+          packet_size*sizeof(*colormap));
         if (colormap == (unsigned char *) NULL)
           return(MagickFalse);
         /*
@@ -1240,32 +1288,41 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
               ThrowWriterException(CorruptImageError,"ImageDepthNotSupported");
             case 32:
             {
+              unsigned long
+                pixel;
+
               pixel=ScaleQuantumToLong(image->colormap[i].red);
-              PopLongPixel(q,pixel);
+              PopLongPixel(&quantum_state,pixel,&q);
               pixel=ScaleQuantumToLong(image->colormap[i].green);
-              PopLongPixel(q,pixel);
+              PopLongPixel(&quantum_state,pixel,&q);
               pixel=ScaleQuantumToLong(image->colormap[i].blue);
-              PopLongPixel(q,pixel);
+              PopLongPixel(&quantum_state,pixel,&q);
             }
             case 16:
             {
+              unsigned short
+                pixel;
+
               pixel=ScaleQuantumToShort(image->colormap[i].red);
-              PopShortPixel(q,pixel);
+              PopShortPixel(&quantum_state,pixel,&q);
               pixel=ScaleQuantumToShort(image->colormap[i].green);
-              PopShortPixel(q,pixel);
+              PopShortPixel(&quantum_state,pixel,&q);
               pixel=ScaleQuantumToShort(image->colormap[i].blue);
-              PopShortPixel(q,pixel);
+              PopShortPixel(&quantum_state,pixel,&q);
               break;
             }
             case 8:
             {
-              pixel=(unsigned long) ScaleQuantumToChar(image->colormap[i].red);
-              PopCharPixel(q,pixel);
-              pixel=(unsigned long)
-                ScaleQuantumToChar(image->colormap[i].green);
-              PopCharPixel(q,pixel);
-              pixel=(unsigned long) ScaleQuantumToChar(image->colormap[i].blue);
-              PopCharPixel(q,pixel);
+              unsigned char
+                pixel;
+
+              pixel=(unsigned char) ScaleQuantumToChar(image->colormap[i].red);
+              PopCharPixel(pixel,&q);
+              pixel=(unsigned char) ScaleQuantumToChar(
+                image->colormap[i].green);
+              PopCharPixel(pixel,&q);
+              pixel=(unsigned char) ScaleQuantumToChar(image->colormap[i].blue);
+              PopCharPixel(pixel,&q);
               break;
             }
           }
