@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,8 +53,10 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 
 /*
   Typedef declarations.
@@ -192,8 +194,29 @@ static MagickBooleanType IsPCX(const unsigned char *magick,const size_t length)
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
+
+static inline long MagickAbsoluteValue(const long x)
+{
+  if (x < 0)
+    return(-x);
+  return(x);
+}
+
+static inline size_t MagickMax(const size_t x,const size_t y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   Image
@@ -211,6 +234,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     status;
 
   MagickOffsetType
+    offset,
     *page_table;
 
   PCXInfo
@@ -276,8 +300,8 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       magic=ReadBlobLSBLong(image);
       if (magic != 987654321)
         ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-      page_table=(MagickOffsetType *)
-         AcquireMagickMemory(1024*sizeof(*page_table));
+      page_table=(MagickOffsetType *) AcquireQuantumMemory(1024UL,
+        sizeof(*page_table));
       if (page_table == (MagickOffsetType *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       for (id=0; id < 1024; id++)
@@ -288,7 +312,11 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
     }
   if (page_table != (MagickOffsetType *) NULL)
-    (void) SeekBlob(image,(MagickOffsetType) page_table[0],SEEK_SET);
+    {
+      offset=SeekBlob(image,(MagickOffsetType) page_table[0],SEEK_SET);
+      if (offset < 0)
+        ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    }
   pcx_colormap=(unsigned char *) NULL;
   count=ReadBlob(image,1,&pcx_info.identifier);
   for (id=1; id < 1024; id++)
@@ -296,11 +324,11 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Verify PCX identifier.
     */
-    pcx_info.version=ReadBlobByte(image);
+    pcx_info.version=(unsigned char) ReadBlobByte(image);
     if ((count == 0) || (pcx_info.identifier != 0x0a))
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-    pcx_info.encoding=ReadBlobByte(image);
-    pcx_info.bits_per_pixel=ReadBlobByte(image);
+    pcx_info.encoding=(unsigned char) ReadBlobByte(image);
+    pcx_info.bits_per_pixel=(unsigned char) ReadBlobByte(image);
     pcx_info.left=ReadBlobLSBShort(image);
     pcx_info.top=ReadBlobLSBShort(image);
     pcx_info.right=ReadBlobLSBShort(image);
@@ -310,24 +338,30 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Read PCX raster colormap.
     */
-    image->columns=(pcx_info.right-pcx_info.left)+1;
-    image->rows=(pcx_info.bottom-pcx_info.top)+1;
-    image->depth=pcx_info.bits_per_pixel <= 8 ? 8 : QuantumDepth;
+    image->columns=(unsigned long) MagickAbsoluteValue((long) pcx_info.right-
+      pcx_info.left)+1UL;
+    image->rows=(unsigned long) MagickAbsoluteValue((long) pcx_info.bottom-
+      pcx_info.top)+1UL;
+    if ((image->columns == 0) || (image->rows == 0) ||
+        (pcx_info.bits_per_pixel == 0))
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    image->depth=pcx_info.bits_per_pixel <= 8 ? 8U : MAGICKCORE_QUANTUM_DEPTH;
     image->units=PixelsPerInchResolution;
-    image->x_resolution=pcx_info.horizontal_resolution;
-    image->y_resolution=pcx_info.vertical_resolution;
+    image->x_resolution=(double) pcx_info.horizontal_resolution;
+    image->y_resolution=(double) pcx_info.vertical_resolution;
     image->colors=16;
-    pcx_colormap=(unsigned char *)
-      AcquireMagickMemory(3*256*sizeof(*pcx_colormap));
+    pcx_colormap=(unsigned char *) AcquireQuantumMemory(256UL,
+      3*sizeof(*pcx_colormap));
     if (pcx_colormap == (unsigned char *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     count=ReadBlob(image,3*image->colors,pcx_colormap);
-    pcx_info.reserved=ReadBlobByte(image);
-    pcx_info.planes=ReadBlobByte(image);
+    pcx_info.reserved=(unsigned char) ReadBlobByte(image);
+    pcx_info.planes=(unsigned char) ReadBlobByte(image);
     if ((pcx_info.bits_per_pixel != 8) || (pcx_info.planes == 1))
       if ((pcx_info.version == 3) || (pcx_info.version == 5) ||
           ((pcx_info.bits_per_pixel*pcx_info.planes) == 1))
-        image->colors=1 << (pcx_info.bits_per_pixel*pcx_info.planes);
+        image->colors=(unsigned long) MagickMin(1UL << (1UL*
+          pcx_info.bits_per_pixel*pcx_info.planes),256UL);
     if (AllocateImageColormap(image,image->colors) == MagickFalse)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if ((pcx_info.bits_per_pixel >= 8) && (pcx_info.planes != 1))
@@ -349,12 +383,17 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Read image data.
     */
+    if (SetImageExtent(image,0,0) == MagickFalse)
+      {
+        InheritException(exception,&image->exception);
+        return(DestroyImageList(image));
+      }
     pcx_packets=(unsigned long) image->rows*pcx_info.bytes_per_line*
       pcx_info.planes;
-    pcx_pixels=(unsigned char *)
-      AcquireMagickMemory(pcx_packets*sizeof(*pcx_pixels));
-    scanline=(unsigned char *) AcquireMagickMemory(Max(image->columns,
-      pcx_info.bytes_per_line)*pcx_info.planes*sizeof(*scanline));
+    pcx_pixels=(unsigned char *) AcquireQuantumMemory(pcx_packets,
+      sizeof(*pcx_pixels));
+    scanline=(unsigned char *) AcquireQuantumMemory(MagickMax(image->columns,
+      pcx_info.bytes_per_line),MagickMax(8,pcx_info.planes)*sizeof(*scanline));
     if ((pcx_pixels == (unsigned char *) NULL) ||
         (scanline == (unsigned char *) NULL))
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -365,7 +404,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (pcx_info.encoding == 0)
       while (pcx_packets != 0)
       {
-        packet=ReadBlobByte(image);
+        packet=(unsigned char) ReadBlobByte(image);
         if (EOFBlob(image) != MagickFalse)
           break;
         *p++=packet;
@@ -374,7 +413,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     else
       while (pcx_packets != 0)
       {
-        packet=ReadBlobByte(image);
+        packet=(unsigned char) ReadBlobByte(image);
         if (EOFBlob(image) != MagickFalse)
           break;
         if ((packet & 0xc0) != 0xc0)
@@ -383,8 +422,8 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             pcx_packets--;
             continue;
           }
-        count=packet & 0x3f;
-        packet=ReadBlobByte(image);
+        count=(ssize_t) (packet & 0x3f);
+        packet=(unsigned char) ReadBlobByte(image);
         if (EOFBlob(image) != MagickFalse)
           break;
         for ( ; count != 0; count--)
@@ -396,7 +435,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
       }
     if (image->storage_class == DirectClass)
-      image->matte=(MagickBooleanType) (pcx_info.planes > 3);
+      image->matte=pcx_info.planes > 3 ? MagickTrue : MagickFalse;
     else
       if ((pcx_info.version == 5) ||
           ((pcx_info.bits_per_pixel*pcx_info.planes) == 1))
@@ -411,12 +450,12 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 Monochrome colormap.
               */
-              image->colormap[0].red=0;
-              image->colormap[0].green=0;
-              image->colormap[0].blue=0;
-              image->colormap[1].red=QuantumRange;
-              image->colormap[1].green=QuantumRange;
-              image->colormap[1].blue=QuantumRange;
+              image->colormap[0].red=(Quantum) 0;
+              image->colormap[0].green=(Quantum) 0;
+              image->colormap[0].blue=(Quantum) 0;
+              image->colormap[1].red=(Quantum) QuantumRange;
+              image->colormap[1].green=(Quantum) QuantumRange;
+              image->colormap[1].blue=(Quantum) QuantumRange;
             }
           else
             if (image->colors > 16)
@@ -424,7 +463,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 /*
                   256 color images have their color map at the end of the file.
                 */
-                pcx_info.colormap_signature=ReadBlobByte(image);
+                pcx_info.colormap_signature=(unsigned char) ReadBlobByte(image);
                 count=ReadBlob(image,3*image->colors,pcx_colormap);
                 p=pcx_colormap;
                 for (i=0; i < (long) image->colors; i++)
@@ -451,7 +490,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         for (i=0; i < pcx_info.planes; i++)
         {
           r=scanline+i;
-          for (x=0; x < pcx_info.bytes_per_line; x++)
+          for (x=0; x < (long) pcx_info.bytes_per_line; x++)
           {
             switch (i)
             {
@@ -488,7 +527,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             for (i=0; i < pcx_info.planes; i++)
             {
               r=scanline;
-              for (x=0; x < pcx_info.bytes_per_line; x++)
+              for (x=0; x < (long) pcx_info.bytes_per_line; x++)
               {
                  bits=(*p++);
                  for (mask=0x80; mask != 0; mask>>=1)
@@ -511,13 +550,13 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
               for (x=0; x < ((long) image->columns-7); x+=8)
               {
                 for (bit=7; bit >= 0; bit--)
-                  *r++=((*p) & (0x01 << bit) ? 0x01 : 0x00);
+                  *r++=(unsigned char) ((*p) & (0x01 << bit) ? 0x01 : 0x00);
                 p++;
               }
               if ((image->columns % 8) != 0)
                 {
                   for (bit=7; bit >= (long) (8-(image->columns % 8)); bit--)
-                    *r++=((*p) & (0x01 << bit) ? 0x01 : 0x00);
+                    *r++=(unsigned char) ((*p) & (0x01 << bit) ? 0x01 : 0x00);
                   p++;
                 }
               break;
@@ -535,7 +574,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
               if ((image->columns % 4) != 0)
                 {
                   for (i=3; i >= (long) (4-(image->columns % 4)); i--)
-                    *r++=(*p >> (i*2)) & 0x03;
+                    *r++=(unsigned char) ((*p >> (i*2)) & 0x03);
                   p++;
                 }
               break;
@@ -567,7 +606,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       for (x=0; x < (long) image->columns; x++)
       {
         if (image->storage_class == PseudoClass)
-          indexes[x]=(*r++);
+          indexes[x]=(IndexPacket) (*r++);
         else
           {
             q->red=ScaleCharToQuantum(*r++);
@@ -612,7 +651,9 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     if (page_table[id] == 0)
       break;
-    (void) SeekBlob(image,(MagickOffsetType) page_table[id],SEEK_SET);
+    offset=SeekBlob(image,(MagickOffsetType) page_table[id],SEEK_SET);
+    if (offset < 0)
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     count=ReadBlob(image,1,&pcx_info.identifier);
     if ((count != 0) && (pcx_info.identifier == 0x0a))
       {
@@ -661,31 +702,32 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterPCXImage method is:
 %
-%      RegisterPCXImage(void)
+%      unsigned long RegisterPCXImage(void)
 %
 */
-ModuleExport void RegisterPCXImage(void)
+ModuleExport unsigned long RegisterPCXImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("DCX");
-  entry->decoder=(DecoderHandler *) ReadPCXImage;
-  entry->encoder=(EncoderHandler *) WritePCXImage;
+  entry->decoder=(DecodeImageHandler *) ReadPCXImage;
+  entry->encoder=(EncodeImageHandler *) WritePCXImage;
   entry->seekable_stream=MagickTrue;
-  entry->magick=(MagickHandler *) IsDCX;
-  entry->description=AcquireString("ZSoft IBM PC multi-page Paintbrush");
-  entry->module=AcquireString("PCX");
+  entry->magick=(IsImageFormatHandler *) IsDCX;
+  entry->description=ConstantString("ZSoft IBM PC multi-page Paintbrush");
+  entry->module=ConstantString("PCX");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PCX");
-  entry->decoder=(DecoderHandler *) ReadPCXImage;
-  entry->encoder=(EncoderHandler *) WritePCXImage;
-  entry->magick=(MagickHandler *) IsPCX;
+  entry->decoder=(DecodeImageHandler *) ReadPCXImage;
+  entry->encoder=(EncodeImageHandler *) WritePCXImage;
+  entry->magick=(IsImageFormatHandler *) IsPCX;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("ZSoft IBM PC Paintbrush");
-  entry->module=AcquireString("PCX");
+  entry->description=ConstantString("ZSoft IBM PC Paintbrush");
+  entry->module=ConstantString("PCX");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -772,7 +814,7 @@ static MagickBooleanType PCXWritePixels(PCXInfo *pcx_info,
       if ((count > 1) || ((previous & 0xc0) == 0xc0))
         {
           count|=0xc0;
-          (void) WriteBlobByte(image,count);
+          (void) WriteBlobByte(image,(unsigned char) count);
         }
       (void) WriteBlobByte(image,previous);
       previous=packet;
@@ -781,7 +823,7 @@ static MagickBooleanType PCXWritePixels(PCXInfo *pcx_info,
     if ((count > 1) || ((previous & 0xc0) == 0xc0))
       {
         count|=0xc0;
-        (void) WriteBlobByte(image,count);
+        (void) WriteBlobByte(image,(unsigned char) count);
       }
     (void) WriteBlobByte(image,previous);
   }
@@ -797,6 +839,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
     status;
 
   MagickOffsetType
+    offset,
     *page_table,
     scene;
 
@@ -835,7 +878,8 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   page_table=(MagickOffsetType *) NULL;
   if ((LocaleCompare(image_info->magick,"DCX") == 0) ||
       ((GetNextImageInList(image) != (Image *) NULL) &&
@@ -845,8 +889,8 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
         Write the DCX page table.
       */
       (void) WriteBlobLSBLong(image,0x3ADE68B1L);
-      page_table=(MagickOffsetType *)
-        AcquireMagickMemory(1024*sizeof(*page_table));
+      page_table=(MagickOffsetType *) AcquireQuantumMemory(1024UL,
+        sizeof(*page_table));
       if (page_table == (MagickOffsetType *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       for (scene=0; scene < 1024; scene++)
@@ -892,7 +936,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
     }
     pcx_info.reserved=0;
     pcx_info.planes=1;
-    if (image->storage_class == DirectClass)
+    if ((image->storage_class == DirectClass) || (image->colors > 256))
       {
         pcx_info.planes=3;
         if (image->matte != MagickFalse)
@@ -918,13 +962,13 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
     /*
       Dump colormap to file.
     */
-    pcx_colormap=(unsigned char *)
-      AcquireMagickMemory(3*256*sizeof(*pcx_colormap));
+    pcx_colormap=(unsigned char *) AcquireQuantumMemory(256UL,
+      3*sizeof(*pcx_colormap));
     if (pcx_colormap == (unsigned char *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     (void) memset(pcx_colormap,0,3*256*sizeof(*pcx_colormap));
     q=pcx_colormap;
-    if (image->storage_class == PseudoClass)
+    if ((image->storage_class == PseudoClass) && (image->colors <= 256))
       for (i=0; i < (long) image->colors; i++)
       {
         *q++=ScaleQuantumToChar(image->colormap[i].red);
@@ -938,13 +982,13 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobLSBShort(image,pcx_info.palette_info);
     for (i=0; i < 58; i++)
       (void) WriteBlobByte(image,'\0');
-    length=(size_t) pcx_info.bytes_per_line*pcx_info.planes;
-    pcx_pixels=(unsigned char *)
-      AcquireMagickMemory(length*sizeof(*pcx_pixels));
+    length=(size_t) pcx_info.bytes_per_line;
+    pcx_pixels=(unsigned char *) AcquireQuantumMemory(length,
+      pcx_info.planes*sizeof(*pcx_pixels));
     if (pcx_pixels == (unsigned char *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     q=pcx_pixels;
-    if (image->storage_class == DirectClass)
+    if ((image->storage_class == DirectClass) || (image->colors > 256))
       {
         const PixelPacket
           *pixels;
@@ -966,7 +1010,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
             {
               case 0:
               {
-                for (x=0; x < pcx_info.bytes_per_line; x++)
+                for (x=0; x < (long) pcx_info.bytes_per_line; x++)
                 {
                   *q++=ScaleQuantumToChar(p->red);
                   p++;
@@ -975,7 +1019,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
               }
               case 1:
               {
-                for (x=0; x < pcx_info.bytes_per_line; x++)
+                for (x=0; x < (long) pcx_info.bytes_per_line; x++)
                 {
                   *q++=ScaleQuantumToChar(p->green);
                   p++;
@@ -984,7 +1028,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
               }
               case 2:
               {
-                for (x=0; x < pcx_info.bytes_per_line; x++)
+                for (x=0; x < (long) pcx_info.bytes_per_line; x++)
                 {
                   *q++=ScaleQuantumToChar(p->blue);
                   p++;
@@ -994,9 +1038,9 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
               case 3:
               default:
               {
-                for (x=(long) pcx_info.bytes_per_line; x > 0; x--)
+                for (x=(long) pcx_info.bytes_per_line; x != 0; x--)
                 {
-                  *q++=ScaleQuantumToChar(QuantumRange-p->opacity);
+                  *q++=ScaleQuantumToChar((Quantum) (QuantumRange-p->opacity));
                   p++;
                 }
                 break;
@@ -1025,7 +1069,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
           indexes=GetIndexes(image);
           q=pcx_pixels;
           for (x=0; x < (long) image->columns; x++)
-            *q++=indexes[x];
+            *q++=(unsigned char) indexes[x];
           if (PCXWritePixels(&pcx_info,pcx_pixels,image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
@@ -1040,19 +1084,22 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
         }
       else
         {
+          IndexPacket
+            polarity;
+
           register unsigned char
             bit,
-            byte,
-            polarity;
+            byte;
 
           /*
             Convert PseudoClass image to a PCX monochrome image.
           */
-          polarity=PixelIntensityToQuantum(&image->colormap[0]) <
-            (QuantumRange/2);
+          polarity=(IndexPacket) (PixelIntensityToQuantum(&image->colormap[0]) <
+            ((Quantum) QuantumRange/2) ? 1 : 0);
           if (image->colors == 2)
-            polarity=PixelIntensityToQuantum(&image->colormap[0]) <
-              PixelIntensityToQuantum(&image->colormap[1]);
+            polarity=(IndexPacket) (
+              PixelIntensityToQuantum(&image->colormap[0]) <
+              PixelIntensityToQuantum(&image->colormap[1]) ? 1 : 0);
           for (y=0; y < (long) image->rows; y++)
           {
             p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -1115,7 +1162,9 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
         Write the DCX page table.
       */
       page_table[scene+1]=0;
-      (void) SeekBlob(image,0L,SEEK_SET);
+      offset=SeekBlob(image,0L,SEEK_SET);
+      if (offset < 0)
+        ThrowWriterException(CorruptImageError,"ImproperImageHeader");
       (void) WriteBlobLSBLong(image,0x3ADE68B1L);
       for (i=0; i <= (long) scene; i++)
         (void) WriteBlobLSBLong(image,(unsigned long) page_table[i]);
@@ -1123,9 +1172,15 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
         RelinquishMagickMemory(page_table);
     }
   if (status == MagickFalse)
-    (void) ThrowMagickException(&image->exception,GetMagickModule(),
-      FileOpenError,"UnableToWriteFile","`%s': %s",image->filename,
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        FileOpenError,"UnableToWriteFile","`%s': %s",image->filename,message);
+      message=DestroyString(message);
+    }
   CloseBlob(image);
   return(MagickTrue);
 }
