@@ -10,39 +10,26 @@
 %                            R  R   LLLLL  EEEEE                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                          Read URT RLE Image Format.                         %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,8 +39,22 @@
 /*
   Include declarations.
 */
-#include "magick.h"
-#include "define.h"
+#include "magick/studio.h"
+#include "magick/property.h"
+#include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
+#include "magick/image.h"
+#include "magick/image-private.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/quantum-private.h"
+#include "magick/static.h"
+#include "magick/string_.h"
+#include "magick/module.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -66,17 +67,15 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsRLE returns True if the image format type, identified by the
+%  IsRLE() returns MagickTrue if the image format type, identified by the
 %  magick string, is RLE.
 %
 %  The format of the ReadRLEImage method is:
 %
-%      unsigned int IsRLE(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsRLE(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsRLE returns True if the image format type is RLE.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -84,13 +83,13 @@
 %
 %
 */
-static unsigned int IsRLE(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsRLE(const unsigned char *magick,const size_t length)
 {
   if (length < 2)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\122\314",2) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -104,7 +103,7 @@ static unsigned int IsRLE(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadRLEImage reads a run-length encoded Utah Raster Toolkit
+%  ReadRLEImage() reads a run-length encoded Utah Raster Toolkit
 %  image file and returns it.  It allocates the memory necessary for the new
 %  Image structure and returns a pointer to the new image.
 %
@@ -114,11 +113,7 @@ static unsigned int IsRLE(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadRLEImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -147,6 +142,12 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  MagickStatusType
+    flags;
+
+  MagickSizeType
+    number_pixels;
+
   register IndexPacket
     *indexes;
 
@@ -162,7 +163,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register unsigned char
     *p;
 
-  size_t
+  ssize_t
     count;
 
   unsigned char
@@ -172,14 +173,10 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
     plane,
     *rle_pixels;
 
-  unsigned int
-    bits_per_pixel,
-    flags,
-    map_length;
-
   unsigned long
+    bits_per_pixel,
+    map_length,
     number_colormaps,
-    number_pixels,
     number_planes;
 
   /*
@@ -187,18 +184,24 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenWarning,"Unable to open file",image);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
+  if (status == MagickFalse)
+    {
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Determine if this is a RLE file.
   */
-  count=ReadBlob(image,2,(char *) magick);
+  count=ReadBlob(image,2,(unsigned char *) magick);
   if ((count == 0) || (memcmp(magick,"\122\314",2) != 0))
-    ThrowReaderException(CorruptImageWarning,"Not a RLE image file",image);
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   do
   {
     /*
@@ -208,16 +211,15 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) ReadBlobLSBShort(image);
     image->columns=ReadBlobLSBShort(image);
     image->rows=ReadBlobLSBShort(image);
-    flags=ReadBlobByte(image);
-    image->matte=flags & 0x04;
-    number_planes=ReadBlobByte(image);
-    bits_per_pixel=ReadBlobByte(image);
-    number_colormaps=ReadBlobByte(image);
-    map_length=1 << ReadBlobByte(image);
+    flags=(MagickStatusType) ReadBlobByte(image);
+    image->matte=flags & 0x04 ? MagickTrue : MagickFalse;
+    number_planes=1UL*ReadBlobByte(image);
+    bits_per_pixel=1UL*ReadBlobByte(image);
+    number_colormaps=1UL*ReadBlobByte(image);
+    map_length=1UL << ReadBlobByte(image);
     if ((number_planes == 0) || (number_planes == 2) || (bits_per_pixel != 8) ||
         (image->columns == 0))
-      ThrowReaderException(CorruptImageWarning,"Unsupported RLE image file",
-        image);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     if (flags & 0x02)
       {
         /*
@@ -234,7 +236,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         p=background_color;
         for (i=0; i < (long) number_planes; i++)
-          *p++=ReadBlobByte(image);
+          *p++=(unsigned char) ReadBlobByte(image);
       }
     if ((number_planes & 0x01) == 0)
       (void) ReadBlobByte(image);
@@ -244,51 +246,60 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Read image colormaps.
         */
-        colormap=(unsigned char *) AcquireMemory(number_colormaps*map_length);
+        colormap=(unsigned char *) AcquireQuantumMemory(number_colormaps,
+          map_length*sizeof(*colormap));
         if (colormap == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-            image);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         p=colormap;
         for (i=0; i < (long) number_colormaps; i++)
           for (x=0; x < (long) map_length; x++)
-            *p++=XDownscale(ReadBlobLSBShort(image));
+            *p++=(unsigned char) ScaleShortToQuantum(ReadBlobLSBShort(image));
       }
-    if (flags & 0x08)
+    if ((flags & 0x08) != 0)
       {
         char
           *comment;
 
-        unsigned int
+        unsigned long
           length;
 
         /*
           Read image comment.
         */
         length=ReadBlobLSBShort(image);
-        comment=(char *) AcquireMemory(length);
-        if (comment == (char *) NULL)
-          ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-            image);
-        (void) ReadBlob(image,length-1,comment);
-        comment[length-1]='\0';
-        (void) SetImageAttribute(image,"comment",comment);
-        LiberateMemory((void **) &comment);
-        if ((length & 0x01) == 0)
-          (void) ReadBlobByte(image);
+        if (length != 0)
+          {
+            comment=(char *) AcquireQuantumMemory(length,sizeof(*comment));
+            if (comment == (char *) NULL)
+              ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+            count=ReadBlob(image,length-1,(unsigned char *) comment);
+            comment[length-1]='\0';
+            (void) SetImageProperty(image,"comment",comment);
+            comment=DestroyString(comment);
+            if ((length & 0x01) == 0)
+              (void) ReadBlobByte(image);
+          }
       }
-    if (image_info->ping && (image_info->subrange != 0))
-      if (image->scene >= (image_info->subimage+image_info->subrange-1))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
+      if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
+    if (SetImageExtent(image,0,0) == MagickFalse)
+      {
+        InheritException(exception,&image->exception);
+        return(DestroyImageList(image));
+      }
     /*
       Allocate RLE pixels.
     */
-    if (image->matte)
+    if (image->matte != MagickFalse)
       number_planes++;
-    number_pixels=image->columns*image->rows;
-    rle_pixels=(unsigned char *) AcquireMemory(number_pixels*number_planes);
+    number_pixels=(MagickSizeType) image->columns*image->rows;
+    if ((number_pixels*number_planes) != (size_t) (number_pixels*number_planes))
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    rle_pixels=(unsigned char *) AcquireQuantumMemory(image->columns,
+      image->rows*number_planes*sizeof(*rle_pixels));
     if (rle_pixels == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if ((flags & 0x01) && !(flags & 0x02))
       {
         long
@@ -300,7 +311,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         p=rle_pixels;
         for (i=0; i < (long) number_pixels; i++)
         {
-          if (!image->matte)
+          if (image->matte == MagickFalse)
             for (j=0; j < (long) number_planes; j++)
               *p++=background_color[j];
           else
@@ -326,7 +337,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         {
           operand=ReadBlobByte(image);
           if (opcode & 0x40)
-            operand=ReadBlobLSBShort(image);
+            operand=(int) ReadBlobLSBShort(image);
           x=0;
           y+=operand;
           break;
@@ -344,7 +355,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         {
           operand=ReadBlobByte(image);
           if (opcode & 0x40)
-            operand=ReadBlobLSBShort(image);
+            operand=(int) ReadBlobLSBShort(image);
           x+=operand;
           break;
         }
@@ -352,13 +363,13 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         {
           operand=ReadBlobByte(image);
           if (opcode & 0x40)
-            operand=ReadBlobLSBShort(image);
+            operand=(int) ReadBlobLSBShort(image);
           p=rle_pixels+((image->rows-y-1)*image->columns*number_planes)+
             x*number_planes+plane;
           operand++;
           for (i=0; i < (long) operand; i++)
           {
-            pixel=ReadBlobByte(image);
+            pixel=(unsigned char) ReadBlobByte(image);
             if ((y < (long) image->rows) && ((x+i) < (long) image->columns))
               *p=pixel;
             p+=number_planes;
@@ -372,8 +383,8 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         {
           operand=ReadBlobByte(image);
           if (opcode & 0x40)
-            operand=ReadBlobLSBShort(image);
-          pixel=ReadBlobByte(image);
+            operand=(int) ReadBlobLSBShort(image);
+          pixel=(unsigned char) ReadBlobByte(image);
           (void) ReadBlobByte(image);
           operand++;
           p=rle_pixels+((image->rows-y-1)*image->columns*number_planes)+
@@ -394,13 +405,13 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
     } while (((opcode & 0x3f) != EOFOp) && (opcode != EOF));
     if (number_colormaps != 0)
       {
-        unsigned int
+        MagickStatusType
           mask;
 
         /*
           Apply colormap affineation to image.
         */
-        mask=(map_length-1);
+        mask=(MagickStatusType) (map_length-1);
         p=rle_pixels;
         if (number_colormaps == 1)
           for (i=0; i < (long) number_pixels; i++)
@@ -433,18 +444,24 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           for (x=0; x < (long) image->columns; x++)
           {
-            q->red=Upscale(*p++);
-            q->green=Upscale(*p++);
-            q->blue=Upscale(*p++);
-            if (image->matte)
-              q->opacity=(Quantum) (MaxRGB-Upscale(*p++));
+            q->red=ScaleCharToQuantum(*p++);
+            q->green=ScaleCharToQuantum(*p++);
+            q->blue=ScaleCharToQuantum(*p++);
+            if (image->matte != MagickFalse)
+              q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(*p++));
             q++;
           }
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
       }
     else
@@ -454,9 +471,8 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         if (number_colormaps == 0)
           map_length=256;
-        if (!AllocateImageColormap(image,map_length))
-          ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-            image);
+        if (AllocateImageColormap(image,map_length) == MagickFalse)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         p=colormap;
         if (number_colormaps == 1)
           for (i=0; i < (long) image->colors; i++)
@@ -464,21 +480,21 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
             /*
               Pseudocolor.
             */
-            image->colormap[i].red=Upscale(i);
-            image->colormap[i].green=Upscale(i);
-            image->colormap[i].blue=Upscale(i);
+            image->colormap[i].red=ScaleCharToQuantum((unsigned char) i);
+            image->colormap[i].green=ScaleCharToQuantum((unsigned char) i);
+            image->colormap[i].blue=ScaleCharToQuantum((unsigned char) i);
           }
         else
           if (number_colormaps > 1)
             for (i=0; i < (long) image->colors; i++)
             {
-              image->colormap[i].red=Upscale(*p);
-              image->colormap[i].green=Upscale(*(p+map_length));
-              image->colormap[i].blue=Upscale(*(p+map_length*2));
+              image->colormap[i].red=ScaleCharToQuantum(*p);
+              image->colormap[i].green=ScaleCharToQuantum(*(p+map_length));
+              image->colormap[i].blue=ScaleCharToQuantum(*(p+map_length*2));
               p++;
             }
         p=rle_pixels;
-        if (!image->matte)
+        if (image->matte == MagickFalse)
           {
             /*
               Convert raster image to PseudoClass pixel packets.
@@ -490,14 +506,20 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               indexes=GetIndexes(image);
               for (x=0; x < (long) image->columns; x++)
-                indexes[x]=(*p++);
-              if (!SyncImagePixels(image))
+                indexes[x]=(IndexPacket) (*p++);
+              if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows))
-                  MagickMonitor(LoadImageText,y,image->rows);
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(LoadImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
             }
-            SyncImage(image);
+            (void) SyncImage(image);
           }
         else
           {
@@ -514,53 +536,67 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 q->red=image->colormap[*p++].red;
                 q->green=image->colormap[*p++].green;
                 q->blue=image->colormap[*p++].blue;
-                q->opacity=(Quantum) (MaxRGB-Upscale(*p++));
+                q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(*p++));
                 q++;
               }
-              if (!SyncImagePixels(image))
+              if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows))
-                  MagickMonitor(LoadImageText,y,image->rows);
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(LoadImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
             }
-            LiberateMemory((void **) &image->colormap);
-            image->colormap=(PixelPacket *) NULL;
+            image->colormap=(PixelPacket *)
+              RelinquishMagickMemory(image->colormap);
             image->storage_class=DirectClass;
             image->colors=0;
           }
       }
     if (number_colormaps != 0)
-      LiberateMemory((void **) &colormap);
-    LiberateMemory((void **) &rle_pixels);
-    if (EOFBlob(image))
-      ThrowReaderException(CorruptImageWarning,"Unexpected end-of-file",image);
+      colormap=(unsigned char *) RelinquishMagickMemory(colormap);
+    rle_pixels=(unsigned char *) RelinquishMagickMemory(rle_pixels);
+    if (EOFBlob(image) != MagickFalse)
+      {
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
-    if (image_info->subrange != 0)
-      if (image->scene >= (image_info->subimage+image_info->subrange-1))
+    if (image_info->number_scenes != 0)
+      if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     (void) ReadBlobByte(image);
-    count=ReadBlob(image,2,(char *) magick);
+    count=ReadBlob(image,2,(unsigned char *) magick);
     if ((count != 0) && (memcmp(magick,"\122\314",2) == 0))
       {
         /*
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while ((count != 0) && (memcmp(magick,"\122\314",2) == 0));
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -574,7 +610,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterRLEImage adds attributes for the RLE image format to
+%  RegisterRLEImage() adds attributes for the RLE image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -583,21 +619,22 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterRLEImage method is:
 %
-%      RegisterRLEImage(void)
+%      unsigned long RegisterRLEImage(void)
 %
 */
-ModuleExport void RegisterRLEImage(void)
+ModuleExport unsigned long RegisterRLEImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("RLE");
-  entry->decoder=ReadRLEImage;
-  entry->magick=IsRLE;
-  entry->adjoin=False;
-  entry->description=AcquireString("Utah Run length encoded image");
-  entry->module=AcquireString("RLE");
+  entry->decoder=(DecodeImageHandler *) ReadRLEImage;
+  entry->magick=(IsImageFormatHandler *) IsRLE;
+  entry->adjoin=MagickFalse;
+  entry->description=ConstantString("Utah Run length encoded image");
+  entry->module=ConstantString("RLE");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -611,7 +648,7 @@ ModuleExport void RegisterRLEImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterRLEImage removes format registrations made by the
+%  UnregisterRLEImage() removes format registrations made by the
 %  RLE module from the list of supported formats.
 %
 %  The format of the UnregisterRLEImage method is:
