@@ -10,39 +10,26 @@
 %                       V    IIIII   CCCC  A   A  R  R                        %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                    Read/Write VICAR Rasterfile Format.                      %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,13 +39,29 @@
 /*
   Include declarations.
 */
-#include "magick.h"
-#include "define.h"
+#include "magick/studio.h"
+#include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
+#include "magick/image.h"
+#include "magick/image-private.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/quantum-private.h"
+#include "magick/quantum-private.h"
+#include "magick/static.h"
+#include "magick/string_.h"
+#include "magick/module.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteVICARImage(const ImageInfo *,Image *);
 
 /*
@@ -72,17 +75,16 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsVICAR returns True if the image format type, identified by the
+%  IsVICAR() returns MagickTrue if the image format type, identified by the
 %  magick string, is VICAR.
 %
 %  The format of the IsVICAR method is:
 %
-%      unsigned int IsVICAR(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsVICAR(const unsigned char *magick,
+%        const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsVICAR returns True if the image format type is VICAR.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -90,15 +92,18 @@ static unsigned int
 %
 %
 */
-static unsigned int IsVICAR(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsVICAR(const unsigned char *magick,
+  const size_t length)
 {
-  if (length < 7)
-    return(False);
+  if (length < 14)
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick,"LBLSIZE",7) == 0)
-    return(True);
+    return(MagickTrue);
   if (LocaleNCompare((char *) magick,"NJPL1I",6) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  if (LocaleNCompare((char *) magick,"PDS_VERSION_ID",14) == 0)
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -112,7 +117,7 @@ static unsigned int IsVICAR(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadVICARImage reads a VICAR image file and returns it.  It
+%  ReadVICARImage() reads a VICAR image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -127,7 +132,7 @@ static unsigned int IsVICAR(const unsigned char *magick,const size_t length)
 %      reading.  A null image is returned if there is a memory shortage or if
 %      the image cannot be read.
 %
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -144,33 +149,47 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
     *image;
 
   int
-    c,
-    y;
+    c;
 
   long
-    count;
+    y;
 
-  unsigned char
-    *scanline;
-
-  unsigned int
+  MagickBooleanType
     status,
     value_expected;
 
-  unsigned long
+  QuantumInfo
+    quantum_info;
+
+  register PixelPacket
+    *q;
+
+  ssize_t
+    count;
+
+  ssize_t
     length;
 
+  unsigned char
+    *scanline;
+
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenWarning,"Unable to open file",image);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
+  if (status == MagickFalse)
+    {
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Decode image header.
   */
@@ -178,7 +197,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   count=1;
   if (c == EOF)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   length=0;
@@ -186,7 +205,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   image->rows=0;
   while (isgraph(c) && ((image->columns == 0) || (image->rows == 0)))
   {
-    if (!isalnum(c))
+    if (isalnum(c) == MagickFalse)
       {
         c=ReadBlobByte(image);
         count++;
@@ -202,26 +221,26 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
         p=keyword;
         do
         {
-          if ((p-keyword) < (MaxTextExtent-1))
+          if ((size_t) (p-keyword) < MaxTextExtent)
             *p++=c;
           c=ReadBlobByte(image);
           count++;
         } while (isalnum(c) || (c == '_'));
         *p='\0';
-        value_expected=False;
-        while (isspace(c) || (c == '='))
+        value_expected=MagickFalse;
+        while ((isspace((int) ((unsigned char) c)) != 0) || (c == '='))
         {
           if (c == '=')
-            value_expected=True;
+            value_expected=MagickTrue;
           c=ReadBlobByte(image);
           count++;
         }
-        if (value_expected == False)
+        if (value_expected == MagickFalse)
           continue;
         p=value;
         while (isalnum(c))
         {
-          if ((p-value) < (MaxTextExtent-1))
+          if ((size_t) (p-value) < MaxTextExtent)
             *p++=c;
           c=ReadBlobByte(image);
           count++;
@@ -231,61 +250,76 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
           Assign a value to the specified keyword.
         */
         if (LocaleCompare(keyword,"Label_RECORDS") == 0)
-          length=atol(value);
+          length=(ssize_t) atol(value);
         if (LocaleCompare(keyword,"LBLSIZE") == 0)
-          length=atol(value);
+          length=(ssize_t) atol(value);
         if (LocaleCompare(keyword,"RECORD_BYTES") == 0)
-          image->columns= atol(value);
+          image->columns=1UL*atol(value);
         if (LocaleCompare(keyword,"NS") == 0)
-          image->columns= atol(value);
+          image->columns=1UL*atol(value);
         if (LocaleCompare(keyword,"LINES") == 0)
-          image->rows= atol(value);
+          image->rows=1UL*atol(value);
         if (LocaleCompare(keyword,"NL") == 0)
-          image->rows= atol(value);
+          image->rows=1UL*atol(value);
       }
-    while (isspace(c))
+    while (isspace((int) ((unsigned char) c)) != 0)
     {
       c=ReadBlobByte(image);
       count++;
     }
   }
-  while (count < (long) length)
+  while (count < (ssize_t) length)
   {
     c=ReadBlobByte(image);
     count++;
   }
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(CorruptImageWarning,"image size is zero",image);
+    ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
   image->depth=8;
-  if (!AllocateImageColormap(image,256))
-    ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",image);
-  if (image_info->ping)
+  if (AllocateImageColormap(image,256) == MagickFalse)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
-      return(image);
+      return(GetFirstImageInList(image));
     }
   /*
     Read VICAR pixels.
   */
-  scanline=(unsigned char *) AcquireMemory(image->columns);
+  if (SetImageExtent(image,0,0) == MagickFalse)
+    {
+      InheritException(exception,&image->exception);
+      return(DestroyImageList(image));
+    }
+  GetQuantumInfo(image_info,&quantum_info);
+  scanline=(unsigned char *) AcquireQuantumMemory(image->columns,
+    sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
-    ThrowReaderException(CorruptImageWarning,"Unable to read image data",image);
+    ThrowReaderException(CorruptImageError,"UnableToReadImageData");
   for (y=0; y < (long) image->rows; y++)
   {
-    if (!SetImagePixels(image,0,y,image->columns,1))
+    q=SetImagePixels(image,0,y,image->columns,1);
+    if (q == (PixelPacket *) NULL)
       break;
-    (void) ReadBlob(image,image->columns,scanline);
-    (void) PushImagePixels(image,GrayQuantum,scanline);
-    if (!SyncImagePixels(image))
+    count=ReadBlob(image,image->columns,scanline);
+    (void) ExportQuantumPixels(image,&quantum_info,GrayQuantum,scanline);
+    if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
-  LiberateMemory((void **) &scanline);
-  if (EOFBlob(image))
-    ThrowReaderException(CorruptImageWarning,"Unexpected end-of-file",image);
+  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -299,7 +333,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterVICARImage adds attributes for the VICAR image format to
+%  RegisterVICARImage() adds attributes for the VICAR image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -308,22 +342,23 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterVICARImage method is:
 %
-%      RegisterVICARImage(void)
+%      unsigned long RegisterVICARImage(void)
 %
 */
-ModuleExport void RegisterVICARImage(void)
+ModuleExport unsigned long RegisterVICARImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("VICAR");
-  entry->decoder=ReadVICARImage;
-  entry->encoder=WriteVICARImage;
-  entry->magick=IsVICAR;
-  entry->adjoin=False;
-  entry->description=AcquireString("VICAR rasterfile format");
-  entry->module=AcquireString("VICAR");
+  entry->decoder=(DecodeImageHandler *) ReadVICARImage;
+  entry->encoder=(EncodeImageHandler *) WriteVICARImage;
+  entry->magick=(IsImageFormatHandler *) IsVICAR;
+  entry->adjoin=MagickFalse;
+  entry->description=ConstantString("VICAR rasterfile format");
+  entry->module=ConstantString("VICAR");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -337,7 +372,7 @@ ModuleExport void RegisterVICARImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterVICARImage removes format registrations made by the
+%  UnregisterVICARImage() removes format registrations made by the
 %  VICAR module from the list of supported formats.
 %
 %  The format of the UnregisterVICARImage method is:
@@ -361,32 +396,30 @@ ModuleExport void UnregisterVICARImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteVICARImage writes an image in the VICAR rasterfile format.
+%  WriteVICARImage() writes an image in the VICAR rasterfile format.
 %  Vicar files contain a text header, followed by one or more planes of binary
 %  grayscale image data.  Vicar files are designed to allow many planes to be
 %  stacked together to form image cubes.  This method only writes a single
 %  grayscale plane.
 %
-%  Method WriteVICARImage was written contributed by
+%  WriteVICARImage was written contributed by
 %  gorelick@esther.la.asu.edu.
 %
 %  The format of the WriteVICARImage method is:
 %
-%      unsigned int WriteVICARImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteVICARImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
+%    o image_info: The image info.
 %
-%    o image_info: Specifies a pointer to an ImageInfo structure.
-%
-%    o image:  A pointer to a Image structure.
+%    o image:  The image.
 %
 %
 */
-static unsigned int WriteVICARImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
+  Image *image)
 {
   char
     header[MaxTextExtent];
@@ -394,12 +427,18 @@ static unsigned int WriteVICARImage(const ImageInfo *image_info,Image *image)
   int
     y;
 
+  MagickBooleanType
+    status;
+
+  QuantumInfo
+    quantum_info;
+
+  register const PixelPacket
+    *p;
+
   unsigned char
     *scanline;
 
-  unsigned int
-    status;
-
   /*
     Open output image file.
   */
@@ -407,40 +446,53 @@ static unsigned int WriteVICARImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenWarning,"Unable to open file",image);
-  (void) TransformRGBImage(image,RGBColorspace);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  if (status == MagickFalse)
+    return(status);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   /*
     Write header.
   */
-  (void) memset(header,' ',MaxTextExtent);
-  FormatString(header,"LBLSIZE=%u FORMAT='BYTE' TYPE='IMAGE' BUFSIZE=20000 "
-    "DIM=2 EOL=0 RECSIZE=%u ORG='BSQ' NL=%u NS=%u NB=1 N1=0 N2=0 N3=0 N4=0 "
-    "NBB=0 NLB=0 TASK='ImageMagick'",MaxTextExtent,image->columns,image->rows,
-    image->columns);
-  (void) WriteBlob(image,MaxTextExtent,header);
+  (void) ResetMagickMemory(header,' ',MaxTextExtent);
+  (void) FormatMagickString(header,MaxTextExtent,
+    "LBLSIZE=%lu FORMAT='BYTE' TYPE='IMAGE' BUFSIZE=20000 DIM=2 EOL=0 "
+    "RECSIZE=%lu ORG='BSQ' NL=%lu NS=%lu NB=1 N1=0 N2=0 N3=0 N4=0 NBB=0 "
+    "NLB=0 TASK='ImageMagick'",(unsigned long) MaxTextExtent,image->columns,
+    image->rows,image->columns);
+  (void) WriteBlob(image,MaxTextExtent,(unsigned char *) header);
   /*
     Allocate memory for scanline.
   */
-  scanline=(unsigned char *) AcquireMemory(image->columns);
+  scanline=(unsigned char *) AcquireQuantumMemory(image->columns,
+    sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Write VICAR scanline.
   */
+  GetQuantumInfo(image_info,&quantum_info);
   image->depth=8;
   for (y=0; y < (long) image->rows; y++)
   {
-    if (!AcquireImagePixels(image,0,y,image->columns,1,&image->exception))
+    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    if (p == (const PixelPacket *) NULL)
       break;
-    (void) PopImagePixels(image,GrayQuantum,scanline);
+    (void) ImportQuantumPixels(image,&quantum_info,GrayQuantum,scanline);
     (void) WriteBlob(image,image->columns,scanline);
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows))
-        MagickMonitor(SaveImageText,y,image->rows);
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
+        {
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
+          if (status == MagickFalse)
+            break;
+        }
   }
-  LiberateMemory((void **) &scanline);
+  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
