@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -54,8 +54,10 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/resize.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 #include "magick/utility.h"
 
 /*
@@ -90,7 +92,6 @@ static MagickBooleanType
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
@@ -184,16 +185,22 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           return((Image *) NULL);
         }
       for (i=0; i < image->offset; i++)
-        (void) ReadBlobByte(image);
+        if (ReadBlobByte(image) == EOF)
+          {
+            ThrowFileException(exception,CorruptImageError,
+              "UnexpectedEndOfFile",image->filename);
+            break;
+          }
     }
   /*
     Allocate memory for a scanline.
   */
   if (interlace == NoInterlace)
-    scanline=(unsigned char *)
-      AcquireMagickMemory((size_t) (2*image->columns+2));
+    scanline=(unsigned char *) AcquireQuantumMemory((size_t) 2UL*
+      image->columns+2UL,sizeof(*scanline));
   else
-    scanline=(unsigned char *) AcquireMagickMemory((size_t) image->columns);
+    scanline=(unsigned char *) AcquireQuantumMemory((size_t) image->columns,
+      sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   do
@@ -208,6 +215,11 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
+    if (SetImageExtent(image,0,0) == MagickFalse)
+      {
+        InheritException(exception,&image->exception);
+        return(DestroyImageList(image));
+      }
     if (interlace == PartitionInterlace)
       {
         AppendImageFormat("Y",image->filename);
@@ -237,11 +249,11 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           for (x=0; x < (long) image->columns; x+=2)
           {
-            chroma_pixels->red=0;
+            chroma_pixels->red=(Quantum) 0;
             chroma_pixels->green=ScaleCharToQuantum(*p++);
             q->red=ScaleCharToQuantum(*p++);
-            q->green=0;
-            q->blue=0;
+            q->green=(Quantum) 0;
+            q->blue=(Quantum) 0;
             q++;
             q->green=0;
             q->blue=0;
@@ -303,9 +315,9 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           for (x=0; x < (long) chroma_image->columns; x++)
           {
-            q->red=0;
+            q->red=(Quantum) 0;
             q->green=ScaleCharToQuantum(*p++);
-            q->blue=0;
+            q->blue=(Quantum) 0;
             q++;
           }
           if (SyncImagePixels(chroma_image) == MagickFalse)
@@ -366,7 +378,8 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     resize_image=DestroyImage(resize_image);
     image->colorspace=YCbCrColorspace;
-    (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     if (interlace == PartitionInterlace)
       (void) CopyMagickString(image->filename,image_info->filename,
         MaxTextExtent);
@@ -432,22 +445,23 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterYUVImage method is:
 %
-%      RegisterYUVImage(void)
+%      unsigned long RegisterYUVImage(void)
 %
 */
-ModuleExport void RegisterYUVImage(void)
+ModuleExport unsigned long RegisterYUVImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("YUV");
-  entry->decoder=(DecoderHandler *) ReadYUVImage;
-  entry->encoder=(EncoderHandler *) WriteYUVImage;
+  entry->decoder=(DecodeImageHandler *) ReadYUVImage;
+  entry->encoder=(EncodeImageHandler *) WriteYUVImage;
   entry->adjoin=MagickFalse;
   entry->raw=MagickTrue;
-  entry->description=AcquireString("CCIR 601 4:1:1 or 4:2:2");
-  entry->module=AcquireString("YUV");
+  entry->description=ConstantString("CCIR 601 4:1:1 or 4:2:2");
+  entry->module=ConstantString("YUV");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -588,14 +602,16 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
       Sample image to an even width and height, if necessary.
     */
     image->depth=8;
-    (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     width=image->columns+(image->columns & (horizontal_factor-1));
     height=image->rows+(image->rows & (vertical_factor-1));
     yuv_image=ResizeImage(image,width,height,TriangleFilter,1.0,
       &image->exception);
     if (yuv_image == (Image *) NULL)
       ThrowWriterException(ResourceLimitError,image->exception.reason);
-    (void) SetImageColorspace(yuv_image,YCbCrColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(yuv_image,YCbCrColorspace);
     /*
       Downsample image.
     */
@@ -603,7 +619,8 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
       height/vertical_factor,TriangleFilter,1.0,&image->exception);
     if (chroma_image == (Image *) NULL)
       ThrowWriterException(ResourceLimitError,image->exception.reason);
-    (void) SetImageColorspace(chroma_image,YCbCrColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(chroma_image,YCbCrColorspace);
     if (interlace == NoInterlace)
       {
         /*
