@@ -17,7 +17,7 @@
 %                                September 2002                               %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -55,6 +55,13 @@
 #include "magick/token.h"
 #include "magick/utility.h"
 #include "magick/version.h"
+#include "magick/xml-tree.h"
+#if defined(MAGICKCORE_HAVE_PTHREAD)
+#include <pthread.h>
+#endif
+#if defined(__WINDOWS__)
+#include <windows.h>
+#endif
 
 /*
   Define declarations.
@@ -64,6 +71,18 @@
 /*
   Typedef declarations.
 */
+typedef enum
+{
+  UndefinedHandler = 0x0000,
+  NoHandler = 0x0000,
+  ConsoleHandler = 0x0001,
+  StdoutHandler = 0x0002,
+  StderrHandler = 0x0004,
+  FileHandler = 0x0008,
+  DebugHandler = 0x0010,
+  EventHandler = 0x0020
+} LogHandlerType;
+
 typedef struct _EventInfo
 {
   char
@@ -75,12 +94,47 @@ typedef struct _EventInfo
 
 typedef struct _HandlerInfo
 {
-  char
+  const char
     *name;
 
   LogHandlerType
     handler;
 } HandlerInfo;
+
+struct _LogInfo
+{
+  LogEventType
+    event_mask;
+
+  LogHandlerType
+    handler_mask;
+
+  char
+    *path,
+    *name,
+    *filename,
+    *format;
+
+  unsigned long
+    generations,
+    limit;
+
+  FILE
+    *file;
+
+  unsigned long
+    generation;
+
+  MagickBooleanType
+    append,
+    stealth;
+
+  TimerInfo
+    timer;
+
+  unsigned long
+    signature;
+};
 
 /*
   Declare log map.
@@ -89,28 +143,31 @@ static const HandlerInfo
   LogHandlers[] =
   {
     { "console", ConsoleHandler },
+    { "debug", DebugHandler },
+    { "event", EventHandler },
     { "file", FileHandler },
     { "none", NoHandler },
-    { "stdout", StdoutHandler },
     { "stderr", StderrHandler },
+    { "stdout", StdoutHandler },
     { (char *) NULL, UndefinedHandler }
   };
 
 static const char
-  *LogMap = (char *)
+  *LogMap = (const char *)
     "<?xml version=\"1.0\"?>"
     "<logmap>"
     "  <log events=\"None\" />"
     "  <log output=\"console\" />"
     "  <log filename=\"Magick-%d.log\" />"
-    "  <log generations=\"3\" />"
-    "  <log limit=\"2000\" />"
     "  <log format=\"%t %r %u %v %d %c[%p]: %m/%f/%l/%d\n  %e\" />"
     "</logmap>";
 
 /*
   Static declarations.
 */
+static char
+  log_name[MaxTextExtent] = "Magick";
+
 static LinkedListInfo
   *log_list = (LinkedListInfo *) NULL;
 
@@ -135,6 +192,48 @@ static MagickBooleanType
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   C l o s e M a g i c k L o g                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  CloseMagickLog() closes the Magick log.
+%
+%  The format of the CloseMagickLog method is:
+%
+%      CloseMagickLog(void)
+%
+*/
+MagickExport void CloseMagickLog(void)
+{
+  ExceptionInfo
+    *exception;
+
+  LogInfo
+    *log_info;
+
+  if (IsEventLogging() == MagickFalse)
+    return;
+  exception=AcquireExceptionInfo();
+  log_info=(LogInfo *) GetLogInfo("*",exception);
+  exception=DestroyExceptionInfo(exception);
+  AcquireSemaphoreInfo(&log_semaphore);
+  if (log_info->file != (FILE *) NULL)
+    {
+      if (log_info->append == MagickFalse)
+        (void) fprintf(log_info->file,"</log>\n");
+      (void) fclose(log_info->file);
+      log_info->file=(FILE *) NULL;
+    }
+  RelinquishSemaphoreInfo(log_semaphore);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   D e s t r o y L o g L i s t                                               %
 %                                                                             %
 %                                                                             %
@@ -147,7 +246,6 @@ static MagickBooleanType
 %
 %      DestroyLogList(void)
 %
-%
 */
 
 static void *DestroyLogElement(void *log_info)
@@ -157,17 +255,18 @@ static void *DestroyLogElement(void *log_info)
 
   p=(LogInfo *) log_info;
   if (p->file != (FILE *) NULL)
-    if (LocaleCompare(p->format,"xml") == 0)
-      {
+    {
+      if (p->append == MagickFalse)
         (void) fprintf(p->file,"</log>\n");
-        (void) fclose(p->file);
-      }
+      (void) fclose(p->file);
+      p->file=(FILE *) NULL;
+    }
   if (p->filename != (char *) NULL)
-    p->filename=(char *) RelinquishMagickMemory(p->filename);
+    p->filename=DestroyString(p->filename);
   if (p->path != (char *) NULL)
-    p->path=(char *) RelinquishMagickMemory(p->path);
+    p->path=DestroyString(p->path);
   if (p->format != (char *) NULL)
-    p->format=(char *) RelinquishMagickMemory(p->format);
+    p->format=DestroyString(p->format);
   p=(LogInfo *) RelinquishMagickMemory(p);
   return((void *) NULL);
 }
@@ -206,7 +305,6 @@ MagickExport void DestroyLogList(void)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport const LogInfo *GetLogInfo(const char *name,
   ExceptionInfo *exception)
@@ -312,8 +410,8 @@ MagickExport const LogInfo **GetLogInfoList(const char *pattern,
   p=GetLogInfo("*",exception);
   if (p == (const LogInfo *) NULL)
     return((const LogInfo **) NULL);
-  preferences=(const LogInfo **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(log_list)+1)*sizeof(*preferences));
+  preferences=(const LogInfo **) AcquireQuantumMemory((size_t)
+    GetNumberOfElementsInLinkedList(log_list)+1UL,sizeof(*preferences));
   if (preferences == (const LogInfo **) NULL)
     return((const LogInfo **) NULL);
   /*
@@ -325,7 +423,7 @@ MagickExport const LogInfo **GetLogInfoList(const char *pattern,
   for (i=0; p != (const LogInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
-        (GlobExpression(p->name,pattern) != MagickFalse))
+        (GlobExpression(p->name,pattern,MagickFalse) != MagickFalse))
       preferences[i++]=p;
     p=(const LogInfo *) GetNextValueInLinkedList(log_list);
   }
@@ -370,12 +468,12 @@ extern "C" {
 
 static int LogCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     **p,
     **q;
 
-  p=(char **) x;
-  q=(char **) y;
+  p=(const char **) x;
+  q=(const char **) y;
   return(LocaleCompare(*p,*q));
 }
 
@@ -405,8 +503,8 @@ MagickExport char **GetLogList(const char *pattern,
   p=GetLogInfo("*",exception);
   if (p == (const LogInfo *) NULL)
     return((char **) NULL);
-  preferences=(char **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(log_list)+1)*sizeof(*preferences));
+  preferences=(char **) AcquireQuantumMemory((size_t)
+    GetNumberOfElementsInLinkedList(log_list)+1UL,sizeof(*preferences));
   if (preferences == (char **) NULL)
     return((char **) NULL);
   /*
@@ -418,8 +516,8 @@ MagickExport char **GetLogList(const char *pattern,
   for (i=0; p != (const LogInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
-        (GlobExpression(p->name,pattern) != MagickFalse))
-      preferences[i++]=ConstantString(AcquireString(p->name));
+        (GlobExpression(p->name,pattern,MagickFalse) != MagickFalse))
+      preferences[i++]=ConstantString(p->name);
     p=(const LogInfo *) GetNextValueInLinkedList(log_list);
   }
   RelinquishSemaphoreInfo(log_semaphore);
@@ -434,6 +532,29 @@ MagickExport char **GetLogList(const char *pattern,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t L o g N a m e                                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetLogName() returns the current log name.
+%
+%  The format of the GetLogName method is:
+%
+%      const char *GetLogName(void)
+%
+*/
+MagickExport const char *GetLogName(void)
+{
+  return(log_name);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   I n i t i a l i z e L o g L i s t                                         %
 %                                                                             %
 %                                                                             %
@@ -485,7 +606,6 @@ static MagickBooleanType InitializeLogList(ExceptionInfo *exception)
 %
 %      MagickBooleanType IsEventLogging(void)
 %
-%
 */
 MagickExport MagickBooleanType IsEventLogging(void)
 {
@@ -493,15 +613,15 @@ MagickExport MagickBooleanType IsEventLogging(void)
     *log_info;
 
   ExceptionInfo
-    exception;
+    *exception;
 
   if ((log_list == (LinkedListInfo *) NULL) ||
       (IsLinkedListEmpty(log_list) != MagickFalse))
     return(MagickFalse);
-  GetExceptionInfo(&exception);
-  log_info=GetLogInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
-  return((MagickBooleanType) (log_info->event_mask != NoEvents));
+  exception=AcquireExceptionInfo();
+  log_info=GetLogInfo("*",exception);
+  exception=DestroyExceptionInfo(exception);
+  return(log_info->event_mask != NoEvents ? MagickTrue : MagickFalse);
 }
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -526,10 +646,14 @@ MagickExport MagickBooleanType IsEventLogging(void)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport MagickBooleanType ListLogInfo(FILE *file,ExceptionInfo *exception)
 {
+#define MegabytesToBytes(value) ((MagickSizeType) (value)*1024*1024)
+
+  char
+    limit[MaxTextExtent];
+
   const char
     *path;
 
@@ -561,7 +685,7 @@ MagickExport MagickBooleanType ListLogInfo(FILE *file,ExceptionInfo *exception)
       {
         if (log_info[i]->path != (char *) NULL)
           (void) fprintf(file,"\nPath: %s\n\n",log_info[i]->path);
-        (void) fprintf(file,"Filename         Generations  Limit  Format\n");
+        (void) fprintf(file,"Filename       Generations     Limit  Format\n");
         (void) fprintf(file,"-------------------------------------------------"
           "------------------------------\n");
       }
@@ -572,8 +696,9 @@ MagickExport MagickBooleanType ListLogInfo(FILE *file,ExceptionInfo *exception)
         for (j=(long) strlen(log_info[i]->filename); j <= 16; j++)
           (void) fprintf(file," ");
       }
-    (void) fprintf(file,"%11lu",log_info[i]->generations);
-    (void) fprintf(file,"%7lu  ",log_info[i]->limit);
+    (void) fprintf(file,"%9lu  ",log_info[i]->generations);
+    (void) FormatMagickSize(MegabytesToBytes(log_info[i]->limit),limit);
+    (void) fprintf(file,"%8s  ",limit);
     if (log_info[i]->format != (char *) NULL)
       (void) fprintf(file,"%s",log_info[i]->format);
     (void) fprintf(file,"\n");
@@ -615,12 +740,22 @@ MagickExport MagickBooleanType ListLogInfo(FILE *file,ExceptionInfo *exception)
 %
 %    o format: The output format.
 %
-%
 */
 
-static char *TranslateEvent(const LogEventType type,const char *module,
-  const char *function,const unsigned long line,const char *domain,
-  const char *event)
+static unsigned long GetLogThreadId(void)
+{
+#if defined(MAGICKCORE_HAVE_PTHREAD)
+  return((unsigned long) pthread_self());
+#endif
+#if defined(__WINDOWS__)
+  return((unsigned long) GetCurrentThreadId());
+#endif
+  return((unsigned long) getpid());
+}
+
+static char *TranslateEvent(const LogEventType magick_unused(type),
+  const char *module,const char *function,const unsigned long line,
+  const char *domain,const char *event)
 {
   char
     *text;
@@ -629,38 +764,34 @@ static char *TranslateEvent(const LogEventType type,const char *module,
     elapsed_time,
     user_time;
 
+  ExceptionInfo
+    *exception;
+
   LogInfo
     *log_info;
 
-  ExceptionInfo
-    exception;
-
-  register const char
-    *p;
-
   register char
     *q;
 
-  size_t
-    length;
+  register const char
+    *p;
 
-  struct tm
-    *time_meridian;
+  size_t
+    extent;
 
   time_t
     seconds;
 
-  GetExceptionInfo(&exception);
-  log_info=(LogInfo *) GetLogInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  exception=AcquireExceptionInfo();
+  log_info=(LogInfo *) GetLogInfo("*",exception);
+  exception=DestroyExceptionInfo(exception);
   seconds=time((time_t *) NULL);
-  time_meridian=localtime(&seconds);
   elapsed_time=GetElapsedTime(&log_info->timer);
   user_time=GetUserTime(&log_info->timer);
-  text=AcquireString((char *) NULL);
-  length=MaxTextExtent;
+  text=AcquireString(event);
   if (log_info->format == (char *) NULL)
     return(text);
+  extent=strlen(event)+MaxTextExtent;
   if (LocaleCompare(log_info->format,"xml") == 0)
     {
       char
@@ -669,24 +800,22 @@ static char *TranslateEvent(const LogEventType type,const char *module,
       /*
         Translate event in "XML" format.
       */
-      (void) FormatMagickString(timestamp,MaxTextExtent,
-        "%04d%02d%02d%02d%02d%02d",time_meridian->tm_year+1900,
-        time_meridian->tm_mon+1,time_meridian->tm_mday,time_meridian->tm_hour,
-        time_meridian->tm_min,time_meridian->tm_sec);
-      (void) FormatMagickString(text,length,
+      (void) FormatMagickTime(seconds,extent,timestamp);
+      (void) FormatMagickString(text,extent,
         "<entry>\n"
         "  <timestamp>%s</timestamp>\n"
         "  <elapsed-time>%ld:%02ld</elapsed-time>\n"
         "  <user-time>%0.3f</user-time>\n"
-        "  <pid>%ld</pid>\n"
+        "  <process-id>%ld</process-id>\n"
+        "  <thread-id>%lu</thread-id>\n"
         "  <module>%s</module>\n"
         "  <function>%s</function>\n"
         "  <line>%lu</line>\n"
         "  <domain>%s</domain>\n"
         "  <event>%s</event>\n"
         "</entry>",timestamp,(long) (elapsed_time/60.0),
-        (long) ceil(fmod(elapsed_time,60.0)),user_time,(long) getpid(),module,
-        function,line,domain,event);
+        (long) ceil(fmod(elapsed_time,60.0)),user_time,(long) getpid(),
+        GetLogThreadId(),module,function,line,domain,event);
       return(text);
     }
   /*
@@ -696,10 +825,10 @@ static char *TranslateEvent(const LogEventType type,const char *module,
   for (p=log_info->format; *p != '\0'; p++)
   {
     *q='\0';
-    if ((size_t) (q-text+MaxTextExtent) >= length)
+    if ((size_t) (q-text+MaxTextExtent) >= extent)
       {
-        length<<=1;
-        text=(char *) ResizeMagickMemory(text,(length+MaxTextExtent)*
+        extent+=MaxTextExtent;
+        text=(char *) ResizeQuantumMemory(text,extent+MaxTextExtent,
           sizeof(*text));
         if (text == (char *) NULL)
           return((char *) NULL);
@@ -712,8 +841,10 @@ static char *TranslateEvent(const LogEventType type,const char *module,
         %d   domain
         %e   event
         %f   function
+        %g   generation
         %l   line
         %m   module
+        %n   log name
         %p   process id
         %r   real CPU time
         %t   wall clock time
@@ -745,33 +876,39 @@ static char *TranslateEvent(const LogEventType type,const char *module,
     {
       case 'c':
       {
-        (void) CopyMagickString(q,GetClientName(),MaxTextExtent);
-        q+=strlen(GetClientName());
+        q+=CopyMagickString(q,GetClientName(),extent);
         break;
       }
       case 'd':
       {
-        (void) CopyMagickString(q,domain,MaxTextExtent);
-        q+=strlen(domain);
+        q+=CopyMagickString(q,domain,extent);
         break;
       }
       case 'e':
       {
-        (void) CopyMagickString(q,event,MaxTextExtent);
-        q+=strlen(event);
+        q+=CopyMagickString(q,event,extent);
         break;
       }
       case 'f':
       {
-        (void) CopyMagickString(q,function,MaxTextExtent);
-        q+=strlen(function);
+        q+=CopyMagickString(q,function,extent);
+        break;
+      }
+      case 'g':
+      {
+        if (log_info->generations == 0)
+          {
+            (void) CopyMagickString(q,"0",extent);
+            q++;
+            break;
+          }
+        q+=FormatMagickString(q,extent,"%lu",log_info->generation %
+          log_info->generations);
         break;
       }
       case 'l':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",line);
-        while (*q != '\0')
-          q++;
+        q+=FormatMagickString(q,extent,"%lu",line);
         break;
       }
       case 'm':
@@ -785,48 +922,48 @@ static char *TranslateEvent(const LogEventType type,const char *module,
               p++;
               break;
             }
-        (void) CopyMagickString(q,p,MaxTextExtent);
-        q+=strlen(p);
+        q+=CopyMagickString(q,p,extent);
+        break;
+      }
+      case 'n':
+      {
+        q+=CopyMagickString(q,GetLogName(),extent);
         break;
       }
       case 'p':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%ld",(long) getpid());
-        while (*q != '\0')
-          q++;
+        q+=FormatMagickString(q,extent,"%ld",(long) getpid());
         break;
       }
       case 'r':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%ld:%02ld",
-          (long) (elapsed_time/60.0),(long) ceil(fmod(elapsed_time,60.0)));
-        while (*q != '\0')
-          q++;
+        q+=FormatMagickString(q,extent,"%ld:%02ld",(long)
+          (elapsed_time/60.0),(long) ceil(fmod(elapsed_time,60.0)));
         break;
       }
       case 't':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%02d:%02d:%02d",
-          time_meridian->tm_hour,time_meridian->tm_min,time_meridian->tm_sec);
-        while (*q != '\0')
-          q++;
+        q+=FormatMagickTime(seconds,extent,q);
         break;
       }
       case 'u':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%0.3fu",user_time);
-        while (*q != '\0')
-          q++;
+        q+=FormatMagickString(q,extent,"%0.3fu",user_time);
         break;
       }
       case 'v':
       {
-        (void) CopyMagickString(q,MagickLibVersionText,MaxTextExtent);
-        q+=strlen(MagickLibVersionText);
+        q+=CopyMagickString(q,MagickLibVersionText,extent);
+        break;
+      }
+      case '%':
+      {
+        *q++=(*p);
         break;
       }
       default:
       {
+        *q++='%';
         *q++=(*p);
         break;
       }
@@ -836,6 +973,107 @@ static char *TranslateEvent(const LogEventType type,const char *module,
   return(text);
 }
 
+static char *TranslateFilename(const LogInfo *log_info)
+{
+  char
+    *filename;
+
+  register char
+    *q;
+
+  register const char
+    *p;
+
+  size_t
+    extent;
+
+  /*
+    Translate event in "human readable" format.
+  */
+  assert(log_info != (LogInfo *) NULL);
+  assert(log_info->filename != (char *) NULL);
+  filename=AcquireString((char *) NULL);
+  extent=MaxTextExtent;
+  q=filename;
+  for (p=log_info->filename; *p != '\0'; p++)
+  {
+    *q='\0';
+    if ((size_t) (q-filename+MaxTextExtent) >= extent)
+      {
+        extent+=MaxTextExtent;
+        filename=(char *) ResizeQuantumMemory(filename,extent+MaxTextExtent,
+          sizeof(*filename));
+        if (filename == (char *) NULL)
+          return((char *) NULL);
+        q=filename+strlen(filename);
+      }
+    /*
+      The format of the filename is defined by embedding special format
+      characters:
+
+        %c   client name
+        %n   log name
+        %p   process id
+        %v   version
+        %%   percent sign
+    */
+    if (*p != '%')
+      {
+        *q++=(*p);
+        continue;
+      }
+    p++;
+    switch (*p)
+    {
+      case 'c':
+      {
+        q+=CopyMagickString(q,GetClientName(),extent);
+        break;
+      }
+      case 'g':
+      {
+        if (log_info->generations == 0)
+          {
+            (void) CopyMagickString(q,"0",extent);
+            q++;
+            break;
+          }
+        q+=FormatMagickString(q,extent,"%lu",log_info->generation %
+          log_info->generations);
+        break;
+      }
+      case 'n':
+      {
+        q+=CopyMagickString(q,GetLogName(),extent);
+        break;
+      }
+      case 'p':
+      {
+        q+=FormatMagickString(q,extent,"%ld",(long) getpid());
+        break;
+      }
+      case 'v':
+      {
+        q+=CopyMagickString(q,MagickLibVersionText,extent);
+        break;
+      }
+      case '%':
+      {
+        *q++=(*p);
+        break;
+      }
+      default:
+      {
+        *q++='%';
+        *q++=(*p);
+        break;
+      }
+    }
+  }
+  *q='\0';
+  return(filename);
+}
+
 MagickBooleanType LogMagickEventList(const LogEventType type,const char *module,
   const char *function,const unsigned long line,const char *format,
   va_list operands)
@@ -848,7 +1086,7 @@ MagickBooleanType LogMagickEventList(const LogEventType type,const char *module,
     *domain;
 
   ExceptionInfo
-    exception;
+    *exception;
 
   int
     n;
@@ -858,9 +1096,9 @@ MagickBooleanType LogMagickEventList(const LogEventType type,const char *module,
 
   if (IsEventLogging() == MagickFalse)
     return(MagickFalse);
-  GetExceptionInfo(&exception);
-  log_info=(LogInfo *) GetLogInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  exception=AcquireExceptionInfo();
+  log_info=(LogInfo *) GetLogInfo("*",exception);
+  exception=DestroyExceptionInfo(exception);
   AcquireSemaphoreInfo(&log_semaphore);
   if ((log_info->event_mask & type) == 0)
     {
@@ -868,12 +1106,12 @@ MagickBooleanType LogMagickEventList(const LogEventType type,const char *module,
       return(MagickTrue);
     }
   domain=MagickOptionToMnemonic(MagickLogEventOptions,type);
-#if defined(HAVE_VSNPRINTF)
+#if defined(MAGICKCORE_HAVE_VSNPRINTF)
   n=vsnprintf(event,MaxTextExtent,format,operands);
 #else
   n=vsprintf(event,format,operands);
 #endif
-  if  (n < 0)
+  if (n < 0)
     event[MaxTextExtent-1]='\0';
   text=TranslateEvent(type,module,function,line,domain,event);
   if (text == (char *) NULL)
@@ -884,12 +1122,66 @@ MagickBooleanType LogMagickEventList(const LogEventType type,const char *module,
     }
   if ((log_info->handler_mask & ConsoleHandler) != 0)
     {
-      (void) fprintf(stdout,"%s\n",text);
-      (void) fflush(stdout);
+      (void) fprintf(stderr,"%s\n",text);
+      (void) fflush(stderr);
+    }
+  if ((log_info->handler_mask & DebugHandler) != 0)
+    {
 #if defined(__WINDOWS__)
       OutputDebugString(text);
 #endif
     }
+  if ((log_info->handler_mask & EventHandler) != 0)
+    {
+#if defined(__WINDOWS__)
+      (void) NTReportEvent(text,MagickFalse);
+#endif
+    }
+  if ((log_info->handler_mask & FileHandler) != 0)
+    {
+      struct stat
+        file_info;
+
+      file_info.st_size=0;
+      if (log_info->file != (FILE *) NULL)
+        (void) fstat(fileno(log_info->file),&file_info);
+      if (file_info.st_size > (off_t) (1024*1024*log_info->limit))
+        {
+          (void) fprintf(log_info->file,"</log>\n");
+          (void) fclose(log_info->file);
+          log_info->file=(FILE *) NULL;
+        }
+      if (log_info->file == (FILE *) NULL)
+        {
+          char
+            *filename;
+
+          filename=TranslateFilename(log_info);
+          if (filename == (char *) NULL)
+            {
+              (void) ContinueTimer((TimerInfo *) &log_info->timer);
+              RelinquishSemaphoreInfo(log_semaphore);
+              return(MagickFalse);
+            }
+          log_info->append=IsAccessible(filename);
+          log_info->file=MagickOpenStream(filename,"ab");
+          filename=(char  *) RelinquishMagickMemory(filename);
+          if (log_info->file == (FILE *) NULL)
+            {
+              RelinquishSemaphoreInfo(log_semaphore);
+              return(MagickFalse);
+            }
+          log_info->generation++;
+          if (log_info->append == MagickFalse)
+            {
+              (void) fprintf(log_info->file,"<?xml version=\"1.0\" "
+                "encoding=\"UTF-8\" standalone=\"yes\"?>\n");
+              (void) fprintf(log_info->file,"<log>\n");
+            }
+        }
+      (void) fprintf(log_info->file,"%s\n",text);
+      (void) fflush(log_info->file);
+    }
   if ((log_info->handler_mask & StdoutHandler) != 0)
     {
       (void) fprintf(stdout,"%s\n",text);
@@ -900,38 +1192,8 @@ MagickBooleanType LogMagickEventList(const LogEventType type,const char *module,
       (void) fprintf(stderr,"%s\n",text);
       (void) fflush(stderr);
     }
-  if ((log_info->handler_mask & FileHandler) != 0)
-    {
-      log_info->count++;
-      if (log_info->count == log_info->limit)
-        {
-          (void) fprintf(log_info->file,"</log>\n");
-          (void) fclose(log_info->file);
-          log_info->file=(FILE *) NULL;
-        }
-      if (log_info->file == (FILE *) NULL)
-        {
-          char
-            filename[MaxTextExtent];
-
-          (void) FormatMagickString(filename,MaxTextExtent,log_info->filename,
-            log_info->generation % log_info->generations);
-          log_info->file=fopen(filename,"w");
-          if (log_info->file == (FILE *) NULL)
-            {
-              RelinquishSemaphoreInfo(log_semaphore);
-              return(MagickFalse);
-            }
-          log_info->generation++;
-          (void) fprintf(log_info->file,
-            "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n");
-          (void) fprintf(log_info->file,"<log>\n");
-        }
-      (void) fprintf(log_info->file,"%s\n",text);
-      (void) fflush(log_info->file);
-    }
   text=(char  *) RelinquishMagickMemory(text);
-  (void)  ContinueTimer((TimerInfo *) &log_info->timer);
+  (void) ContinueTimer((TimerInfo *) &log_info->timer);
   RelinquishSemaphoreInfo(log_semaphore);
   return(MagickTrue);
 }
@@ -944,7 +1206,7 @@ MagickBooleanType LogMagickEvent(const LogEventType type,const char *module,
 
   MagickBooleanType
     status;
-   
+
   va_start(operands,format);
   status=LogMagickEventList(type,module,function,line,format,operands);
   va_end(operands);
@@ -980,16 +1242,17 @@ MagickBooleanType LogMagickEvent(const LogEventType type,const char *module,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 static MagickBooleanType LoadLogList(const char *xml,const char *filename,
   const unsigned long depth,ExceptionInfo *exception)
 {
   char
     keyword[MaxTextExtent],
-    *q,
     *token;
 
+  const char
+    *q;
+
   LogInfo
     *log_info = (LogInfo *) NULL;
 
@@ -1022,6 +1285,15 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+      {
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
@@ -1059,9 +1331,12 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
                   (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path);
-                  status|=LoadLogList(xml,path,depth+1,exception);
-                  xml=(char *) RelinquishMagickMemory(xml);
+                  xml=FileToString(path,~0,exception);
+                  if (xml != (char *) NULL)
+                    {
+                      status|=LoadLogList(xml,path,depth+1,exception);
+                      xml=DestroyString(xml);
+                    }
                 }
             }
         }
@@ -1074,10 +1349,9 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
         */
         log_info=(LogInfo *) AcquireMagickMemory(sizeof(*log_info));
         if (log_info == (LogInfo *) NULL)
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",filename);
+          ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
         (void) ResetMagickMemory(log_info,0,sizeof(*log_info));
-        log_info->path=ConstantString(AcquireString(filename));
+        log_info->path=ConstantString(filename);
         GetTimerInfo((TimerInfo *) &log_info->timer);
         log_info->signature=MagickSignature;
         continue;
@@ -1092,7 +1366,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
             ResourceLimitError,"MemoryAllocationFailed","`%s'",filename);
         log_info=(LogInfo *) NULL;
       }
-    GetMagickToken(q,(char **) NULL,token);
+    GetMagickToken(q,(const char **) NULL,token);
     if (*token != '=')
       continue;
     GetMagickToken(q,&q,token);
@@ -1118,7 +1392,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
             if (log_info->filename != (char *) NULL)
               log_info->filename=(char *)
                 RelinquishMagickMemory(log_info->filename);
-            log_info->filename=ConstantString(AcquireString(token));
+            log_info->filename=ConstantString(token);
             break;
           }
         if (LocaleCompare((char *) keyword,"format") == 0)
@@ -1126,7 +1400,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
             if (log_info->format != (char *) NULL)
               log_info->format=(char *)
                 RelinquishMagickMemory(log_info->format);
-            log_info->format=ConstantString(AcquireString(token));
+            log_info->format=ConstantString(token);
             break;
           }
         break;
@@ -1136,6 +1410,11 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"generations") == 0)
           {
+            if (LocaleCompare(token,"unlimited") == 0)
+              {
+                log_info->generations=(~0UL);
+                break;
+              }
             log_info->generations=(unsigned long) atol(token);
             break;
           }
@@ -1146,6 +1425,11 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"limit") == 0)
           {
+            if (LocaleCompare(token,"unlimited") == 0)
+              {
+                log_info->limit=(~0UL);
+                break;
+              }
             log_info->limit=(unsigned long) atol(token);
             break;
           }
@@ -1166,7 +1450,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
         break;
     }
   }
-  token=(char *) RelinquishMagickMemory(token);
+  token=DestroyString(token);
   if (log_list == (LinkedListInfo *) NULL)
     return(MagickFalse);
   return(status != 0 ? MagickTrue : MagickFalse);
@@ -1197,17 +1481,13 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 static MagickBooleanType LoadLogLists(const char *filename,
   ExceptionInfo *exception)
 {
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
   return(LoadLogList(LogMap,"built-in",0,exception));
 #else
-  char
-    path[MaxTextExtent];
-
   const StringInfo
     *option;
 
@@ -1218,23 +1498,18 @@ static MagickBooleanType LoadLogLists(const char *filename,
     status;
 
   status=MagickFalse;
-  *path='\0';
   options=GetConfigureOptions(filename,exception);
   option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
-    (void) CopyMagickString(path,option->path,MaxTextExtent);
-    status|=LoadLogList((const char *) option->datum,option->path,0,exception);
+    status|=LoadLogList((const char *) GetStringInfoDatum(option),
+      GetStringInfoPath(option),0,exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
   options=DestroyConfigureOptions(options);
-  if ((log_list == (LinkedListInfo *) NULL) || 
+  if ((log_list == (LinkedListInfo *) NULL) ||
       (IsLinkedListEmpty(log_list) != MagickFalse))
-    {
-      (void) ThrowMagickException(exception,GetMagickModule(),ConfigureWarning,
-        "UnableToOpenConfigureFile","`%s'",path);
-      status|=LoadLogList(LogMap,"built-in",0,exception);
-    }
+    status|=LoadLogList(LogMap,"built-in",0,exception);
   return(status != 0 ? MagickTrue : MagickFalse);
 #endif
 }
@@ -1261,7 +1536,6 @@ static MagickBooleanType LoadLogLists(const char *filename,
 %
 %    o handlers: one or more handlers separated by commas.
 %
-%
 */
 static LogHandlerType ParseLogHandlers(const char *handlers)
 {
@@ -1321,12 +1595,11 @@ static LogHandlerType ParseLogHandlers(const char *handlers)
 %
 %    o events: log these events.
 %
-%
 */
 MagickExport LogEventType SetLogEventMask(const char *events)
 {
   ExceptionInfo
-    exception;
+    *exception;
 
   LogInfo
     *log_info;
@@ -1334,9 +1607,9 @@ MagickExport LogEventType SetLogEventMask(const char *events)
   long
     option;
 
-  GetExceptionInfo(&exception);
-  log_info=(LogInfo *) GetLogInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  exception=AcquireExceptionInfo();
+  log_info=(LogInfo *) GetLogInfo("*",exception);
+  exception=DestroyExceptionInfo(exception);
   option=ParseMagickOption(MagickLogEventOptions,MagickTrue,events);
   AcquireSemaphoreInfo(&log_semaphore);
   log_info=(LogInfo *) GetValueFromLinkedList(log_list,0);
@@ -1356,7 +1629,6 @@ MagickExport LogEventType SetLogEventMask(const char *events)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  SetLogFormat() sets the format for the "human readable" log record.
@@ -1369,22 +1641,52 @@ MagickExport LogEventType SetLogEventMask(const char *events)
 %
 %    o format: The log record format.
 %
-%
 */
 MagickExport void SetLogFormat(const char *format)
 {
   ExceptionInfo
-    exception;
+    *exception;
 
   LogInfo
     *log_info;
 
-  GetExceptionInfo(&exception);
-  log_info=(LogInfo *) GetLogInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  exception=AcquireExceptionInfo();
+  log_info=(LogInfo *) GetLogInfo("*",exception);
+  exception=DestroyExceptionInfo(exception);
   AcquireSemaphoreInfo(&log_semaphore);
   if (log_info->format != (char *) NULL)
-    log_info->format=(char *) RelinquishMagickMemory(log_info->format);
-  log_info->format=ConstantString(AcquireString(format));
+    log_info->format=DestroyString(log_info->format);
+  log_info->format=ConstantString(format);
   RelinquishSemaphoreInfo(log_semaphore);
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   S e t L o g N a m e                                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetLogName() sets the log name and returns it.
+%
+%  The format of the SetLogName method is:
+%
+%      const char *SetLogName(const char *name)
+%
+%  A description of each parameter follows:
+%
+%    o log_name: SetLogName() returns the current client name.
+%
+%    o name: Specifies the new client name.
+%
+*/
+MagickExport const char *SetLogName(const char *name)
+{
+  if ((name != (char *) NULL) && (*name != '\0'))
+    (void) CopyMagickString(log_name,name,MaxTextExtent);
+  return(log_name);
+}
