@@ -17,7 +17,7 @@
 %                                 July 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,8 +43,8 @@
 %  sizes.  It is based on the paper, "Fast Memory Allocation using Lazy Fits."
 %  written by Yoo C. Chung.
 %
-%  By default, ANSI memory methods are called (e.g. malloc()).  Use the
-%  custom memory allocator by defining UseEmbeddableMagick.  The custom memory
+%  By default, ANSI memory methods are called (e.g. malloc).  Use the
+%  custom memory allocator by defining MAGICKCORE_EMBEDDABLE_SUPPORT.  The custom memory
 %  allocator has the advantage of using memory-mapped heap allocation which
 %  permits the virtual memory to be returned to the system in all cases, unlike
 %  the ANSI memory methods that typically only memory-map large allocations.
@@ -123,7 +123,7 @@ typedef struct _MemoryInfo
 /*
   Global declarations.
 */
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
 static MemoryInfo
   memory_info;
 
@@ -140,7 +140,7 @@ static MagickBooleanType
   ExpandHeap(size_t);
 #endif
 
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -247,7 +247,7 @@ static void *AcquireBlock(size_t size)
   /*
     Find free block.
   */
-  size=(size_t) (size+sizeof(size_t)+6*sizeof(size_t)-1) & -(4*sizeof(size_t));
+  size=(size_t) (size+sizeof(size_t)+6*sizeof(size_t)-1) & -(4U*sizeof(size_t));
   i=AllocationPolicy(size);
   block=memory_info.blocks[i];
   while ((block != (void *) NULL) && (SizeOfBlock(block) < size))
@@ -317,7 +317,7 @@ MagickExport void *AcquireMagickMemory(const size_t size)
   register void
     *memory;
 
-#if !defined(UseEmbeddableMagick)
+#if !defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
   memory=malloc(size == 0 ? 1UL : size);
 #else
   if (free_segments == (DataSegmentInfo *) NULL)
@@ -329,7 +329,7 @@ MagickExport void *AcquireMagickMemory(const size_t size)
             i;
 
           assert(2*sizeof(size_t) > (size_t) (~SizeMask));
-          (void) ResetMagickMemory(&memory_info,0,sizeof(MemoryInfo));
+          (void) ResetMagickMemory(&memory_info,0,sizeof(memory_info));
           memory_info.allocation=SegmentSize;
           memory_info.blocks[MaxBlocks]=(void *) (-1);
           for (i=0; i < MaxSegments; i++)
@@ -361,6 +361,45 @@ MagickExport void *AcquireMagickMemory(const size_t size)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   A c q u i r e Q u a n t u m M e m o r y                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AcquireQuantumMemory() returns a pointer to a block of memory at least
+%  count * quantum bytes suitably aligned for any use.
+%
+%  The format of the AcquireQuantumMemory method is:
+%
+%      void *AcquireQuantumMemory(const size_t count,const size_t quantum)
+%
+%  A description of each parameter follows:
+%
+%    o count: The number of quantum elements to allocate.
+%
+%    o quantum: The number of bytes in each quantum.
+%
+*/
+MagickExport void *AcquireQuantumMemory(const size_t count,const size_t quantum)
+{
+  size_t
+    size;
+
+  size=count*quantum;
+  if ((count == 0) || (quantum != (size/count)))
+    {
+      errno=ENOMEM;
+      return((void *) NULL);
+    }
+  return(AcquireMagickMemory(size));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   C o p y M a g i c k M e m o r y                                           %
 %                                                                             %
 %                                                                             %
@@ -384,7 +423,6 @@ MagickExport void *AcquireMagickMemory(const size_t size)
 %
 %    o size: The size of the memory in bytes to allocate.
 %
-%
 */
 MagickExport void *CopyMagickMemory(void *destination,const void *source,
   const size_t size)
@@ -426,7 +464,7 @@ MagickExport void *CopyMagickMemory(void *destination,const void *source,
 */
 MagickExport void DestroyMagickMemory(void)
 {
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
   register long
     i;
 
@@ -439,12 +477,12 @@ MagickExport void DestroyMagickMemory(void)
       (void) UnmapBlob(memory_info.segments[i]->allocation,
         memory_info.segments[i]->length);
   free_segments=(DataSegmentInfo *) NULL;
-  (void) ResetMagickMemory(&memory_info,0,sizeof(MemoryInfo));
+  (void) ResetMagickMemory(&memory_info,0,sizeof(memory_info));
   memory_semaphore=DestroySemaphoreInfo(memory_semaphore);
 #endif
 }
 
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -461,7 +499,7 @@ MagickExport void DestroyMagickMemory(void)
 %
 %  The format of the ExpandHeap method is:
 %
-%      MagickBooleanType ExpandHeap(const size_t size)
+%      MagickBooleanType ExpandHeap(size_t size)
 %
 %  A description of each parameter follows:
 %
@@ -547,7 +585,7 @@ MagickExport void *RelinquishMagickMemory(void *memory)
 {
   if (memory == (void *) NULL)
     return((void *) NULL);
-#if !defined(UseEmbeddableMagick)
+#if !defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
   free(memory);
 #else
   assert((SizeOfBlock(memory) % (4*sizeof(size_t))) == 0);
@@ -648,7 +686,7 @@ MagickExport void *ResetMagickMemory(void *memory,int byte,const size_t size)
 %
 */
 
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
 static inline void *ResizeBlock(void *block,size_t size)
 {
   register void
@@ -675,8 +713,10 @@ MagickExport void *ResizeMagickMemory(void *memory,const size_t size)
 
   if (memory == (void *) NULL)
     return(AcquireMagickMemory(size));
-#if !defined(UseEmbeddableMagick)
+#if !defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
   block=realloc(memory,size == 0 ? 1UL : size);
+  if (block == (void *) NULL)
+    memory=RelinquishMagickMemory(memory);
 #else
   AcquireSemaphoreInfo(&memory_semaphore);
   block=ResizeBlock(memory,size == 0 ? 1UL : size);
@@ -686,15 +726,57 @@ MagickExport void *ResizeMagickMemory(void *memory,const size_t size)
         {
           RelinquishSemaphoreInfo(memory_semaphore);
           memory=RelinquishMagickMemory(memory);
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",strerror(errno));
+          ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
         }
       block=ResizeBlock(memory,size == 0 ? 1UL : size);
       assert(block != (void *) NULL);
     }
   RelinquishSemaphoreInfo(memory_semaphore);
+  memory=RelinquishMagickMemory(memory);
 #endif
-  if (block == (void *) NULL)
-    memory=RelinquishMagickMemory(memory);
   return(block);
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   R e s i z e Q u a n t u m M e m o r y                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ResizeQuantumMemory() changes the size of the memory and returns a pointer
+%  to the (possibly moved) block.  The contents will be unchanged up to the
+%  lesser of the new and old sizes.
+%
+%  The format of the ResizeQuantumMemory method is:
+%
+%      void *ResizeQuantumMemory(void *memory,const size_t count,
+%        const size_t quantum)
+%
+%  A description of each parameter follows:
+%
+%    o memory: A pointer to a memory allocation.
+%
+%    o count: The number of quantum elements to allocate.
+%
+%    o quantum: The number of bytes in each quantum.
+%
+*/
+MagickExport void *ResizeQuantumMemory(void *memory,const size_t count,
+  const size_t quantum)
+{
+  size_t
+    size;
+
+  size=count*quantum;
+  if ((count == 0) || (quantum != (size/count)))
+    {
+      errno=ENOMEM;
+      return((void *) NULL);
+    }
+  return(ResizeMagickMemory(memory,size));
+}
