@@ -18,7 +18,7 @@
 %                                December 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -50,8 +50,10 @@
 #include "magick/draw.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
+#include "magick/image-private.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/splay-tree.h"
@@ -88,7 +90,6 @@
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport void *CropImageToHBITMAP(Image *image,
   const RectangleInfo *geometry,ExceptionInfo *exception)
@@ -185,8 +186,8 @@ MagickExport void *CropImageToHBITMAP(Image *image,
     if (p == (const PixelPacket *) NULL)
       break;
 
-#if QuantumDepth == 8
-      /* Form of PixelPacket is identical to RGBQUAD when QuantumDepth==8 */
+#if MAGICKCORE_QUANTUM_DEPTH == 8
+      /* Form of PixelPacket is identical to RGBQUAD when MAGICKCORE_QUANTUM_DEPTH==8 */
       CopyMagickMemory((void*)q,(const void*)p,page.width*sizeof(PixelPacket));
       q += page.width;
 
@@ -222,13 +223,10 @@ MagickExport void *CropImageToHBITMAP(Image *image,
       GlobalFree((HGLOBAL) bitmap_bitsH);
       return((void *) NULL);
     }
-
   bitmap.bmBits = bitmap_bits;
   bitmapH = CreateBitmapIndirect( &bitmap );
-
   GlobalUnlock((HGLOBAL) bitmap_bitsH);
-
-  return (void *)bitmapH;
+  return((void *) bitmapH);
 }
 
 /*
@@ -253,14 +251,18 @@ MagickExport void *CropImageToHBITMAP(Image *image,
 %
 %    o magick: Specifies the image format.
 %
-%
 */
 MagickExport MagickBooleanType NTIsMagickConflict(const char *magick)
 {
+  MagickBooleanType
+    status;
+
   assert(magick != (char *) NULL);
   if (strlen(magick) > 1)
     return(MagickFalse);
-  return((GetLogicalDrives() & (1 << ((toupper((int) (*magick)))-'A'))) != 0);
+  status=(GetLogicalDrives() & (1 << ((toupper((int) (*magick)))-'A'))) != 0 ?
+    MagickTrue : MagickFalse;
+  return(status);
 }
 
 /*
@@ -383,13 +385,12 @@ MagickExport MagickBooleanType NTLoadTypeLists(SplayTreeInfo *type_list,
           continue;
         *pos='\0'; /* Remove (TrueType) from string */
 
-        type_info=(TypeInfo *) AcquireMagickMemory(sizeof(TypeInfo));
+        type_info=(TypeInfo *) AcquireMagickMemory(sizeof(*type_info));
         if (type_info == (TypeInfo *) NULL)
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "UnableToAllocateTypeInfo",value_name);
+          ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
         (void) ResetMagickMemory(type_info,0,sizeof(TypeInfo));
 
-        type_info->path=ConstantString(AcquireString("Windows Fonts"));
+        type_info->path=ConstantString("Windows Fonts");
         type_info->signature=MagickSignature;
 
         /* Name */
@@ -397,13 +398,13 @@ MagickExport MagickBooleanType NTLoadTypeLists(SplayTreeInfo *type_list,
         for(pos = buffer; *pos != 0 ; pos++)
           if (*pos == ' ')
             *pos = '-';
-        type_info->name=ConstantString(AcquireString(buffer));
+        type_info->name=ConstantString(buffer);
 
         /* Fullname */
-        type_info->description=ConstantString(AcquireString(value_name));
+        type_info->description=ConstantString(value_name);
 
         /* Format */
-        type_info->format=ConstantString(AcquireString("truetype"));
+        type_info->format=ConstantString("truetype");
 
         /* Glyphs */
         if (strchr(value_data,'\\') != (char *) NULL)
@@ -415,7 +416,7 @@ MagickExport MagickBooleanType NTLoadTypeLists(SplayTreeInfo *type_list,
           }
 
         LocaleLower(buffer);
-        type_info->glyphs=ConstantString(AcquireString(buffer));
+        type_info->glyphs=ConstantString(buffer);
 
         type_info->stretch=NormalStretch;
         type_info->style=NormalStyle;
@@ -426,13 +427,13 @@ MagickExport MagickBooleanType NTLoadTypeLists(SplayTreeInfo *type_list,
              (LocaleCompare(type_info->name, "Wingdings") == 0 ) ||
              (LocaleCompare(type_info->name, "Wingdings-2") == 0 ) ||
              (LocaleCompare(type_info->name, "Wingdings-3") == 0 ) )
-          type_info->encoding=ConstantString(AcquireString("AppleRoman"));
+          type_info->encoding=ConstantString("AppleRoman");
 
         family_extent=value_name;
 
         for (q=value_name; *q != '\0'; )
           {
-            GetMagickToken(q,&q,token);
+            GetMagickToken(q,(const char **) &q,token);
             if (*token == '\0')
               break;
 
@@ -543,11 +544,12 @@ MagickExport MagickBooleanType NTLoadTypeLists(SplayTreeInfo *type_list,
           }
 
         (void) CopyMagickString(buffer,value_name,family_extent-value_name+1);
-        type_info->family=ConstantString(AcquireString(buffer));
+        StripString(buffer);
+        type_info->family=ConstantString(buffer);
 
         list_entries++;
-        status=AddValueToSplayTree(type_list,
-          ConstantString(AcquireString(type_info->name)),type_info);
+        status=AddValueToSplayTree(type_list,ConstantString(type_info->name),
+          type_info);
         if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
             ResourceLimitError,"MemoryAllocationFailed","`%s'",type_info->name);
@@ -568,95 +570,102 @@ MagickExport MagickBooleanType NTLoadTypeLists(SplayTreeInfo *type_list,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ImageToHBITMAP() creates a Windows HBITMAP from an IM Image
+%  ImageToHBITMAP() creates a Windows HBITMAP from an image.
 %
 %  The format of the ImageToHBITMAP method is:
 %
-%      HBITMAP ImageToHBITMAP(Image* image)
+%      HBITMAP ImageToHBITMAP(Image *image)
 %
 %  A description of each parameter follows:
 %
 %    o image: the image to convert.
 %
-%
 */
-MagickExport void *ImageToHBITMAP(Image* image)
+MagickExport void *ImageToHBITMAP(Image *image)
 {
-  unsigned long
-    row;
-
-  long
-    memSize;
-
-  const PixelPacket
-    *pPixels;
-
   BITMAP
     bitmap;
 
-  HBITMAP
-    bitmapH;
-
   HANDLE
     bitmap_bitsH;
 
+  HBITMAP
+    bitmapH;
+
+  long
+    y;
+
+  register long
+    x;
+
+  register const PixelPacket
+    *p;
+
+  register RGBQUAD
+    *q;
+
   RGBQUAD
-    *pDestPixel,
     *bitmap_bits;
 
-  bitmap.bmType         = 0;
-  bitmap.bmWidth        = image->columns;
-  bitmap.bmHeight       = image->rows;
-  bitmap.bmWidthBytes   = bitmap.bmWidth * 4;
-  bitmap.bmPlanes       = 1;
-  bitmap.bmBitsPixel    = 32;
-  bitmap.bmBits         = NULL;
-
-  memSize = bitmap.bmWidthBytes * bitmap.bmHeight;
-  bitmap_bitsH = (HANDLE) GlobalAlloc (GMEM_MOVEABLE | GMEM_DDESHARE, memSize);
+  size_t
+    length;
 
+  (void) ResetMagickMemory(&bitmap,0,sizeof(bitmap));
+  bitmap.bmType=0;
+  bitmap.bmWidth=image->columns;
+  bitmap.bmHeight=image->rows;
+  bitmap.bmWidthBytes=4*bitmap.bmWidth;
+  bitmap.bmPlanes=1;
+  bitmap.bmBitsPixel=32;
+  bitmap.bmBits=NULL;
+  length=bitmap.bmWidthBytes*bitmap.bmHeight;
+  bitmap_bitsH=(HANDLE) GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE,length);
   if (bitmap_bitsH == NULL)
-    return(NULL);
-  bitmap_bits=(RGBQUAD *) GlobalLock((HGLOBAL) bitmap_bitsH);
-  pDestPixel=bitmap_bits;
-  if ( bitmap.bmBits == NULL )
-    bitmap.bmBits=bitmap_bits;
-  SetImageColorspace(image,RGBColorspace);
-  for (row = 0 ; row < image->rows ; row++ )
-  {
-      pPixels = AcquireImagePixels(image,0,row,image->columns,1,
-                                   &image->exception);
-#if QuantumDepth == 8
-      /* Form of PixelPacket is identical to RGBQUAD when QuantumDepth==8 */
-      CopyMagickMemory((void*)pDestPixel,(const void*)pPixels,
-             sizeof(PixelPacket)*image->columns);
-      pDestPixel += image->columns;
+    {
+      char
+        *message;
 
-#else  /* 16 or 32 bit Quantum */
-      {
-        unsigned long nPixelCount;
-
-        /* Transfer pixels, scaling to Quantum */
-        for( nPixelCount = image->columns; nPixelCount ; nPixelCount-- )
-          {
-            pDestPixel->rgbRed = ScaleQuantumToChar(pPixels->red);
-            pDestPixel->rgbGreen = ScaleQuantumToChar(pPixels->green);
-            pDestPixel->rgbBlue = ScaleQuantumToChar(pPixels->blue);
-            pDestPixel->rgbReserved = 0;
-            ++pDestPixel;
-            ++pPixels;
-          }
-      }
-#endif
+      message=GetExceptionMessage(errno);
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",message);
+      message=DestroyString(message);
+      return(NULL);
     }
+  bitmap_bits=(RGBQUAD *) GlobalLock((HGLOBAL) bitmap_bitsH);
+  q=bitmap_bits;
+  if (bitmap.bmBits == NULL)
+    bitmap.bmBits=bitmap_bits;
+  (void) SetImageColorspace(image,RGBColorspace);
+  for (y=0; y < (long) image->rows; y++)
+  {
+    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    if (p == (const PixelPacket *) NULL)
+      break;
+    for (x=0; x < (long) image->columns; x++)
+    {
+      q->rgbRed=ScaleQuantumToChar(p->red);
+      q->rgbGreen=ScaleQuantumToChar(p->green);
+      q->rgbBlue=ScaleQuantumToChar(p->blue);
+      q->rgbReserved=0;
+      p++;
+      q++;
+    }
+  }
+  bitmap.bmBits=bitmap_bits;
+  bitmapH=CreateBitmapIndirect(&bitmap);
+  if (bitmapH == NULL)
+    {
+      char
+        *message;
 
-  bitmap.bmBits = bitmap_bits;
-  bitmapH = CreateBitmapIndirect( &bitmap );
-
+      message=GetExceptionMessage(errno);
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",message);
+      message=DestroyString(message);
+    }
   GlobalUnlock((HGLOBAL) bitmap_bitsH);
   GlobalFree((HGLOBAL) bitmap_bitsH);
-
-  return (void *)bitmapH;
+  return((void *) bitmapH);
 }
 
 #endif
