@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,18 +13,49 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick image resize methods.
+  MagickCore image resize methods.
 */
-#ifndef _MAGICK_RESIZE_H
-#define _MAGICK_RESIZE_H
+#ifndef _MAGICKCORE_RESIZE_H
+#define _MAGICKCORE_RESIZE_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
+typedef enum
+{
+  UndefinedFilter,
+  PointFilter,
+  BoxFilter,
+  TriangleFilter,
+  HermiteFilter,
+  HanningFilter,
+  HammingFilter,
+  BlackmanFilter,
+  GaussianFilter,
+  QuadraticFilter,
+  CubicFilter,
+  CatromFilter,
+  MitchellFilter,
+  LanczosFilter,
+  BesselFilter,
+  SincFilter,
+  KaiserFilter,
+  WelshFilter,
+  ParzenFilter,
+  LagrangeFilter,
+  BohmanFilter,
+  BartlettFilter,
+  SentinelFilter  /* a count of all the filters, not a real filter */
+} FilterTypes;
+
 extern MagickExport Image
+  *AdaptiveResizeImage(const Image *,const unsigned long,const unsigned long,
+    ExceptionInfo *),
   *MagnifyImage(const Image *,ExceptionInfo *),
   *MinifyImage(const Image *,ExceptionInfo *),
+  *ResampleImage(const Image *,const double,const double,const FilterTypes,
+    const double,ExceptionInfo *),
   *ResizeImage(const Image *,const unsigned long,const unsigned long,
     const FilterTypes,const double,ExceptionInfo *),
   *SampleImage(const Image *,const unsigned long,const unsigned long,
