@@ -11,37 +11,24 @@
 %                                                                             %
 %             Methods to Compute a Message Digest for an Image                %
 %                                                                             %
-%                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                              December 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,31 +39,16 @@
 /*
   Include declarations.
 */
-#include "magick.h"
-#include "define.h"
-
-/*
-  Define declarations.
-*/
-#define SignatureSize  64
-#define Trunc32(x)  ((x) & 0xffffffffUL)
-
-/*
-  Typedef declarations.
-*/
-typedef struct _SignatureInfo
-{
-  unsigned long
-    digest[8],
-    low_order,
-    high_order;
-
-  long
-    offset;
-
-  unsigned char
-    message[SignatureSize];
-} SignatureInfo;
+#include "magick/studio.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
+#include "magick/property.h"
+#include "magick/image.h"
+#include "magick/memory_.h"
+#include "magick/quantum.h"
+#include "magick/quantum-private.h"
+#include "magick/signature.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
@@ -95,7 +67,7 @@ static void
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method FinalizeSignature finalizes the SHA message digest computation.
+%  FinalizeSignature() finalizes the SHA message digest computation.
 %
 %  The format of the FinalizeSignature method is:
 %
@@ -105,9 +77,8 @@ static void
 %
 %    o signature_info: The address of a structure of type SignatureInfo.
 %
-%
 */
-static void FinalizeSignature(SignatureInfo *signature_info)
+MagickExport void FinalizeSignature(SignatureInfo *signature_info)
 {
   long
     count;
@@ -117,19 +88,23 @@ static void FinalizeSignature(SignatureInfo *signature_info)
     low_order;
 
   /*
-    Finalize computing the SHA digest.
+    Add padding and return the message digest.
   */
+  assert(signature_info != (SignatureInfo *) NULL);
+  assert(signature_info->signature == MagickSignature);
   low_order=signature_info->low_order;
   high_order=signature_info->high_order;
   count=(long) ((low_order >> 3) & 0x3f);
-  signature_info->message[count++]=0x80;
-  if (count <= (SignatureSize-8))
-    (void) memset(signature_info->message+count,0,SignatureSize-8-count);
+  signature_info->message[count++]=(unsigned char) 0x80;
+  if (count <= (MagickSignatureSize-8))
+    (void) ResetMagickMemory(signature_info->message+count,0,(size_t)
+      (MagickSignatureSize-8-count));
   else
     {
-      (void) memset(signature_info->message+count,0,SignatureSize-count);
+      (void) ResetMagickMemory(signature_info->message+count,0,(size_t)
+        (MagickSignatureSize-count));
       TransformSignature(signature_info);
-      (void) memset(signature_info->message,0,SignatureSize-8);
+      (void) ResetMagickMemory(signature_info->message,0,MagickSignatureSize-8);
     }
   signature_info->message[56]=(unsigned char) (high_order >> 24);
   signature_info->message[57]=(unsigned char) (high_order >> 16);
@@ -153,7 +128,7 @@ static void FinalizeSignature(SignatureInfo *signature_info)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method GetSignatureInfo initializes the SHA message digest structure.
+%  GetSignatureInfo() initializes the SHA message digest structure.
 %
 %  The format of the GetSignatureInfo method is:
 %
@@ -163,11 +138,14 @@ static void FinalizeSignature(SignatureInfo *signature_info)
 %
 %    o signature_info: The address of a structure of type SignatureInfo.
 %
-%
 */
-static void GetSignatureInfo(SignatureInfo *signature_info)
+MagickExport void GetSignatureInfo(SignatureInfo *signature_info)
 {
-  (void) memset(signature_info,0,sizeof(SignatureInfo));
+  unsigned long
+    lsb_first;
+
+  assert(signature_info != (SignatureInfo *) NULL);
+  (void) ResetMagickMemory(signature_info,0,sizeof(*signature_info));
   signature_info->digest[0]=0x6a09e667UL;
   signature_info->digest[1]=0xbb67ae85UL;
   signature_info->digest[2]=0x3c6ef372UL;
@@ -176,6 +154,142 @@ static void GetSignatureInfo(SignatureInfo *signature_info)
   signature_info->digest[5]=0x9b05688cUL;
   signature_info->digest[6]=0x1f83d9abUL;
   signature_info->digest[7]=0x5be0cd19UL;
+  lsb_first=1;
+  signature_info->lsb_first=
+    (*(char *) &lsb_first) == 1 ? MagickTrue : MagickFalse;
+  signature_info->signature=MagickSignature;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   S i g n a t u r e I m a g e                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SignatureImage() computes a message digest from an image pixel stream with
+%  an implementation of the NIST SHA-256 Message Digest algorithm.  This
+%  signature uniquely identifies the image and is convenient for determining
+%  if an image has been modified or whether two images are identical.
+%
+%  The format of the SignatureImage method is:
+%
+%      MagickBooleanType SignatureImage(Image *image)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+*/
+MagickExport MagickBooleanType SignatureImage(Image *image)
+{
+  char
+    signature[MaxTextExtent];
+
+  const IndexPacket
+    *indexes;
+
+  long
+    y;
+
+  register const PixelPacket
+    *p;
+
+  register long
+    x;
+
+  register unsigned char
+    *q;
+
+  SignatureInfo
+    signature_info;
+
+  size_t
+    length;
+
+  unsigned char
+    *message;
+
+  unsigned long
+    quantum;
+
+  /*
+    Allocate memory for digital signature.
+  */
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  length=(size_t) image->columns;
+  message=(unsigned char *) AcquireQuantumMemory(length,20UL*sizeof(*message));
+  if (message == (unsigned char *) NULL)
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+      image->filename);
+  /*
+    Compute image digital signature.
+  */
+  GetSignatureInfo(&signature_info);
+  for (y=0; y < (long) image->rows; y++)
+  {
+    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    if (p == (const PixelPacket *) NULL)
+      break;
+    indexes=AcquireIndexes(image);
+    q=message;
+    for (x=0; x < (long) image->columns; x++)
+    {
+      quantum=ScaleQuantumToLong(p->red);
+      *q++=(unsigned char) (quantum >> 24);
+      *q++=(unsigned char) (quantum >> 16);
+      *q++=(unsigned char) (quantum >> 8);
+      *q++=(unsigned char) quantum;
+      quantum=ScaleQuantumToLong(p->green);
+      *q++=(unsigned char) (quantum >> 24);
+      *q++=(unsigned char) (quantum >> 16);
+      *q++=(unsigned char) (quantum >> 8);
+      *q++=(unsigned char) quantum;
+      quantum=ScaleQuantumToLong(p->blue);
+      *q++=(unsigned char) (quantum >> 24);
+      *q++=(unsigned char) (quantum >> 16);
+      *q++=(unsigned char) (quantum >> 8);
+      *q++=(unsigned char) quantum;
+      if (image->colorspace == CMYKColorspace)
+        {
+          quantum=ScaleQuantumToLong(indexes[x]);
+          *q++=(unsigned char) (quantum >> 24);
+          *q++=(unsigned char) (quantum >> 16);
+          *q++=(unsigned char) (quantum >> 8);
+          *q++=(unsigned char) quantum;
+        }
+      if (image->matte != MagickFalse)
+        {
+          quantum=ScaleQuantumToLong(p->opacity);
+          *q++=(unsigned char) (quantum >> 24);
+          *q++=(unsigned char) (quantum >> 16);
+          *q++=(unsigned char) (quantum >> 8);
+          *q++=(unsigned char) quantum;
+        }
+      p++;
+    }
+    UpdateSignature(&signature_info,message,(size_t) (q-message));
+  }
+  FinalizeSignature(&signature_info);
+  message=(unsigned char *) RelinquishMagickMemory(message);
+  /*
+    Convert digital signature to a 64 character hex string.
+  */
+  (void) FormatMagickString(signature,MaxTextExtent,
+    "%08lx%08lx%08lx%08lx%08lx%08lx%08lx%08lx",signature_info.digest[0],
+    signature_info.digest[1],signature_info.digest[2],signature_info.digest[3],
+    signature_info.digest[4],signature_info.digest[5],signature_info.digest[6],
+    signature_info.digest[7]);
+  (void) DeleteImageProperty(image,"signature");
+  (void) SetImageProperty(image,"signature",signature);
+  return(MagickTrue);
 }
 
 /*
@@ -189,7 +303,7 @@ static void GetSignatureInfo(SignatureInfo *signature_info)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method TransformSignature transforms the SHA message digest.
+%  TransformSignature() transforms the SHA message digest.
 %
 %  The format of the TransformSignature method is:
 %
@@ -199,17 +313,43 @@ static void GetSignatureInfo(SignatureInfo *signature_info)
 %
 %    o signature_info: The address of a structure of type SignatureInfo.
 %
-%
 */
+
+static inline unsigned long Ch(unsigned long x,unsigned long y,unsigned long z)
+{
+  return((x & y) ^ (~x & z));
+}
+
+static inline unsigned long Maj(unsigned long x,unsigned long y,unsigned long z)
+{
+  return((x & y) ^ (x & z) ^ (y & z));
+}
+
+static inline unsigned long Trunc32(unsigned long x)
+{
+  return((unsigned long) (x & 0xffffffffUL));
+}
+
+static unsigned long RotateRight(unsigned long x,unsigned long n)
+{
+  return(Trunc32((x >> n) | (x << (32-n))));
+}
+
 static void TransformSignature(SignatureInfo *signature_info)
 {
-#define Ch(x,y,z)  (((x) & (y))^(~(x) & (z)))
-#define Maj(x,y,z)  (((x) & (y))^((x) & (z))^((y) & (z)))
-#define Rot32(x,n)  Trunc32(((x >> n) | (x << (32-n))))
-#define Sigma0(x)  (Rot32(x,7)^Rot32(x,18)^Trunc32(x >> 3))
-#define Sigma1(x)  (Rot32(x,17)^Rot32(x,19)^Trunc32(x >> 10))
-#define Suma0(x)  (Rot32(x,2)^Rot32(x,13)^Rot32(x,22))
-#define Suma1(x)  (Rot32(x,6)^Rot32(x,11)^Rot32(x,25))
+#define Sigma0(x)  (RotateRight(x,7) ^ RotateRight(x,18) ^ Trunc32((x) >> 3))
+#define Sigma1(x)  (RotateRight(x,17) ^ RotateRight(x,19) ^ Trunc32((x) >> 10))
+#define Suma0(x)  (RotateRight(x,2) ^ RotateRight(x,13) ^ RotateRight(x,22))
+#define Suma1(x)  (RotateRight(x,6) ^ RotateRight(x,11) ^ RotateRight(x,25))
+
+  long
+    j;
+
+  register long
+    i;
+
+  register unsigned char
+    *p;
 
   static unsigned long
     K[64] =
@@ -229,16 +369,6 @@ static void TransformSignature(SignatureInfo *signature_info)
       0x90befffaUL, 0xa4506cebUL, 0xbef9a3f7UL, 0xc67178f2UL
     };  /* 32-bit fractional part of the cube root of the first 64 primes */
 
-  int
-    j,
-    shift;
-
-  register long
-    i;
-
-  register unsigned char
-    *p;
-
   unsigned long
     A,
     B,
@@ -248,7 +378,7 @@ static void TransformSignature(SignatureInfo *signature_info)
     F,
     G,
     H,
-    lsb_first,
+    shift,
     T,
     T1,
     T2,
@@ -256,32 +386,16 @@ static void TransformSignature(SignatureInfo *signature_info)
 
   shift=32;
   p=signature_info->message;
-  lsb_first=1;
-  if (*(char *) &lsb_first)
+  if (signature_info->lsb_first == MagickFalse)
     {
-      if (sizeof(lsb_first) > 4)
-        for (i=0; i < 16; i+=2)
-        {
-          T=(*((unsigned long *) p));
-          p+=8;
-          W[i]=((T << 24) & 0xff000000) | ((T << 8) & 0x00ff0000) |
-            ((T >> 8) & 0x0000ff00) | ((T >> 24) & 0x000000ff);
-          T>>=shift;
-          W[i+1]=((T << 24) & 0xff000000) | ((T << 8) & 0x00ff0000) |
-            ((T >> 8) & 0x0000ff00) | ((T >> 24) & 0x000000ff);
-        }
-      else
+      if (sizeof(unsigned long) <= 4)
         for (i=0; i < 16; i++)
         {
           T=(*((unsigned long *) p));
           p+=4;
-          W[i]=((T << 24) & 0xff000000) | ((T << 8) & 0x00ff0000) |
-            ((T >> 8) & 0x0000ff00) | ((T >> 24) & 0x000000ff);
+          W[i]=Trunc32(T);
         }
-    }
-  else
-    {
-      if (sizeof(lsb_first) > 4)
+      else
         for (i=0; i < 16; i+=2)
         {
           T=(*((unsigned long *) p));
@@ -289,14 +403,30 @@ static void TransformSignature(SignatureInfo *signature_info)
           W[i]=Trunc32(T >> shift);
           W[i+1]=Trunc32(T);
         }
-      else
-        for (i=0; i < 16; i++)
-        {
-          T=(*((unsigned long *) p));
-          p+=4;
-          W[i]=Trunc32(T);
-        }
     }
+  else
+    if (sizeof(unsigned long) <= 4)
+      for (i=0; i < 16; i++)
+      {
+        T=(*((unsigned long *) p));
+        p+=4;
+        W[i]=((T << 24) & 0xff000000) | ((T << 8) & 0x00ff0000) |
+          ((T >> 8) & 0x0000ff00) | ((T >> 24) & 0x000000ff);
+      }
+    else
+      for (i=0; i < 16; i+=2)
+      {
+        T=(*((unsigned long *) p));
+        p+=8;
+        W[i]=((T << 24) & 0xff000000) | ((T << 8) & 0x00ff0000) |
+          ((T >> 8) & 0x0000ff00) | ((T >> 24) & 0x000000ff);
+        T>>=shift;
+        W[i+1]=((T << 24) & 0xff000000) | ((T << 8) & 0x00ff0000) |
+          ((T >> 8) & 0x0000ff00) | ((T >> 24) & 0x000000ff);
+      }
+  /*
+    Copy digest to registers.
+  */
   A=signature_info->digest[0];
   B=signature_info->digest[1];
   C=signature_info->digest[2];
@@ -320,6 +450,9 @@ static void TransformSignature(SignatureInfo *signature_info)
     B=A;
     A=Trunc32(T1+T2);
   }
+  /*
+    Add registers back to digest.
+  */
   signature_info->digest[0]=Trunc32(signature_info->digest[0]+A);
   signature_info->digest[1]=Trunc32(signature_info->digest[1]+B);
   signature_info->digest[2]=Trunc32(signature_info->digest[2]+C);
@@ -341,12 +474,12 @@ static void TransformSignature(SignatureInfo *signature_info)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UpdateSignature updates the SHA message digest.
+%  UpdateSignature() updates the SHA message digest.
 %
 %  The format of the UpdateSignature method is:
 %
-%      UpdateSignature(SignatureInfo *signature_info,unsigned char *message,
-%        size_t length)
+%      UpdateSignature(SignatureInfo *signature_info,
+%        const unsigned char *message,const size_t length)
 %
 %  A description of each parameter follows:
 %
@@ -356,164 +489,50 @@ static void TransformSignature(SignatureInfo *signature_info)
 %
 %    o length: The length of the message.
 %
-%
 */
-static void UpdateSignature(SignatureInfo *signature_info,
-  unsigned char *message,size_t length)
+MagickExport void UpdateSignature(SignatureInfo *signature_info,
+  const unsigned char *message,const size_t length)
 {
   register long
     i;
 
   unsigned long
-    count;
+    count,
+    n;
 
   /*
     Update the SHA digest.
   */
-  count=Trunc32(signature_info->low_order+((unsigned char) length << 3));
-  if (length < signature_info->low_order)
+  assert(signature_info != (SignatureInfo *) NULL);
+  assert(signature_info->signature == MagickSignature);
+  n=(unsigned long) length;
+  count=Trunc32(signature_info->low_order+(n << 3));
+  if (count < signature_info->low_order)
     signature_info->high_order++;
   signature_info->low_order=count;
-  signature_info->high_order+=(unsigned char) (length >> 29);
-  if (signature_info->offset)
+  signature_info->high_order+=(n >> 29);
+  if (signature_info->offset != 0)
     {
-      i=SignatureSize-signature_info->offset;
-      if (i > (long) length)
-        i=(long) length;
-      (void) memcpy(signature_info->message+signature_info->offset,
-        message,i);
-      length-=i;
+      i=MagickSignatureSize-signature_info->offset;
+      if (i > (long) n)
+        i=(long) n;
+      (void) CopyMagickMemory(signature_info->message+signature_info->offset,
+        message,(size_t) i);
+      n-=i;
       message+=i;
       signature_info->offset+=i;
-      if (signature_info->offset != SignatureSize)
+      if (signature_info->offset != MagickSignatureSize)
         return;
       TransformSignature(signature_info);
     }
-  while (length >= SignatureSize)
+  while (n >= MagickSignatureSize)
   {
-    (void) memcpy(signature_info->message,message,SignatureSize);
-    message+=SignatureSize;
-    length-=SignatureSize;
+    (void) CopyMagickMemory(signature_info->message,message,
+      MagickSignatureSize);
+    message+=MagickSignatureSize;
+    n-=MagickSignatureSize;
     TransformSignature(signature_info);
   }
-  (void) memcpy(signature_info->message,message,length);
-  signature_info->offset=(long) length;
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   S i g n a t u r e I m a g e                                               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  SignatureImage() computes a message digest from an image pixel stream with
-%  an implementation of the NIST SHA-256 Message Digest algorithm.  This
-%  signature uniquely identifies the image and is convenient for determining
-%  if an image has been modified or whether two images are identical.
-%
-%  The format of the SignatureImage method is:
-%
-%      unsigned int SignatureImage(Image *image)
-%
-%  A description of each parameter follows:
-%
-%    o image: The image.
-%
-%
-*/
-MagickExport unsigned int SignatureImage(Image *image)
-{
-  char
-    signature[MaxTextExtent];
-
-  IndexPacket
-    *indexes;
-
-  long
-    y;
-
-  register long
-    x;
-
-  register const PixelPacket
-    *p;
-
-  register unsigned char
-    *q;
-
-  SignatureInfo
-    signature_info;
-
-  unsigned char
-    *message;
-
-  /*
-    Allocate memory for digital signature.
-  */
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  message=(unsigned char *) AcquireMemory(10*image->columns);
-  if (message == (unsigned char *) NULL)
-    ThrowBinaryException(ResourceLimitWarning,"Unable to compute signature",
-      "Memory allocation failed");
-  /*
-    Compute image digital signature.
-  */
-  GetSignatureInfo(&signature_info);
-  for (y=0; y < (long) image->rows; y++)
-  {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-    if (p == (const PixelPacket *) NULL)
-      break;
-    indexes=GetIndexes(image);
-    q=message;
-    for (x=0; x < (long) image->columns; x++)
-    {
-      *q++=XUpscale(p->red) >> 8;
-      *q++=XUpscale(p->red);
-      *q++=XUpscale(p->green) >> 8;
-      *q++=XUpscale(p->green);
-      *q++=XUpscale(p->blue) >> 8;
-      *q++=XUpscale(p->blue);
-      if (!image->matte)
-        {
-          if (image->colorspace == CMYKColorspace)
-            {
-              *q++=XUpscale(p->opacity) >> 8;
-              *q++=XUpscale(p->opacity);
-            }
-          *q++=XUpscale(OpaqueOpacity) >> 8;
-          *q++=XUpscale(OpaqueOpacity) & 0xff;
-        }
-      else
-        {
-          *q++=XUpscale(p->opacity) >> 8;
-          *q++=XUpscale(p->opacity);
-          if (image->colorspace == CMYKColorspace)
-            {
-              *q++=XUpscale(indexes[x]) >> 8;
-              *q++=XUpscale(indexes[x]);
-            }
-        }
-      p++;
-    }
-    UpdateSignature(&signature_info,message,q-message);
-  }
-  FinalizeSignature(&signature_info);
-  LiberateMemory((void **) &message);
-  /*
-    Convert digital signature to a 64 character hex string.
-  */
-  FormatString(signature,"%08lx%08lx%08lx%08lx%08lx%08lx%08lx%08lx",
-    signature_info.digest[0],signature_info.digest[1],signature_info.digest[2],
-    signature_info.digest[3],signature_info.digest[4],signature_info.digest[5],
-    signature_info.digest[6],signature_info.digest[7]);
-  while (SetImageAttribute(image,"signature",(char *) NULL) != False);
-  (void) SetImageAttribute(image,"signature",signature);
-  return(True);
+  (void) CopyMagickMemory(signature_info->message,message,(size_t) n);
+  signature_info->offset=(long) n;
 }
