@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,11 +13,11 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  Magick Wand Image Methods.
+  MagickWand image Methods.
 */
 
-#ifndef _WAND_MAGICK_IMAGE_H
-#define _WAND_MAGICK_IMAGE_H
+#ifndef _MAGICKWAND_MAGICK_IMAGE_H
+#define _MAGICKWAND_MAGICK_IMAGE_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
@@ -27,7 +27,6 @@ extern WandExport ChannelStatistics
   *MagickGetImageChannelStatistics(MagickWand *);
 
 extern WandExport char
-  *MagickGetImageAttribute(MagickWand *,const char *),
   *MagickGetImageFilename(MagickWand *),
   *MagickGetImageFormat(MagickWand *),
   *MagickGetImageSignature(MagickWand *),
@@ -50,6 +49,7 @@ extern WandExport double
   MagickGetImageTotalInkDensity(MagickWand *);
 
 extern WandExport Image
+  *MagickDestroyImage(Image *),
   *GetImageFromMagickWand(MagickWand *);
 
 extern WandExport ImageType
@@ -58,14 +58,23 @@ extern WandExport ImageType
 extern WandExport InterlaceType
   MagickGetImageInterlaceScheme(MagickWand *);
 
-extern WandExport long
-  MagickGetImageIndex(MagickWand *);
+extern WandExport InterpolatePixelMethod
+  MagickGetImageInterpolateMethod(MagickWand *);
 
 extern WandExport MagickBooleanType
+  MagickAdaptiveBlurImage(MagickWand *,const double,const double),
+  MagickAdaptiveBlurImageChannel(MagickWand *,const ChannelType,const double,
+    const double),
+  MagickAdaptiveResizeImage(MagickWand *,const unsigned long,
+    const unsigned long),
+  MagickAdaptiveSharpenImage(MagickWand *,const double,const double),
+  MagickAdaptiveSharpenImageChannel(MagickWand *,const ChannelType,const double,
+    const double),
   MagickAdaptiveThresholdImage(MagickWand *,const unsigned long,
     const unsigned long,const long),
   MagickAddImage(MagickWand *,const MagickWand *),
   MagickAddNoiseImage(MagickWand *,const NoiseType),
+  MagickAddNoiseImageChannel(MagickWand *,const ChannelType,const NoiseType),
   MagickAffineTransformImage(MagickWand *,const DrawingWand *),
   MagickAnnotateImage(MagickWand *,const DrawingWand *,const double,
     const double,const double,const char *),
@@ -80,16 +89,20 @@ extern WandExport MagickBooleanType
   MagickChopImage(MagickWand *,const unsigned long,const unsigned long,
     const long,const long),
   MagickClipImage(MagickWand *),
-  MagickClipPathImage(MagickWand *,const char *,const MagickBooleanType),
-  MagickColorFloodfillImage(MagickWand *,const PixelWand *,const double,
-    const PixelWand *,const long,const long),
+  MagickClipImagePath(MagickWand *,const char *,const MagickBooleanType),
+  MagickClutImage(MagickWand *,const MagickWand *),
+  MagickClutImageChannel(MagickWand *,const ChannelType,const MagickWand *),
   MagickColorizeImage(MagickWand *,const PixelWand *,const PixelWand *),
   MagickCommentImage(MagickWand *,const char *),
   MagickCompositeImage(MagickWand *,const MagickWand *,const CompositeOperator,
     const long,const long),
+  MagickCompositeImageChannel(MagickWand *,const ChannelType,const MagickWand *,    const CompositeOperator,const long,const long),
   MagickConstituteImage(MagickWand *,const unsigned long,const unsigned long,
     const char *,const StorageType,const void *),
   MagickContrastImage(MagickWand *,const MagickBooleanType),
+  MagickContrastStretchImage(MagickWand *,const double,const double),
+  MagickContrastStretchImageChannel(MagickWand *,const ChannelType,const double,
+    const double),
   MagickConvolveImage(MagickWand *,const unsigned long,const double *),
   MagickConvolveImageChannel(MagickWand *,const ChannelType,const unsigned long,
     const double *),
@@ -99,14 +112,19 @@ extern WandExport MagickBooleanType
   MagickDespeckleImage(MagickWand *),
   MagickDisplayImage(MagickWand *,const char *),
   MagickDisplayImages(MagickWand *,const char *),
+  MagickDistortImage(MagickWand *,const DistortImageMethod,const unsigned long,
+    const double *,const MagickBooleanType),
   MagickDrawImage(MagickWand *,const DrawingWand *),
   MagickEdgeImage(MagickWand *,const double),
   MagickEmbossImage(MagickWand *,const double,const double),
   MagickEnhanceImage(MagickWand *),
   MagickEqualizeImage(MagickWand *),
+  MagickEqualizeImageChannel(MagickWand *,const ChannelType),
   MagickEvaluateImage(MagickWand *,const MagickEvaluateOperator,const double),
   MagickEvaluateImageChannel(MagickWand *,const ChannelType,
     const MagickEvaluateOperator,const double),
+  MagickExtentImage(MagickWand *,const unsigned long,const unsigned long,
+    const long,const long),
   MagickFlipImage(MagickWand *),
   MagickFlopImage(MagickWand *),
   MagickFrameImage(MagickWand *,const PixelWand *,const unsigned long,
@@ -123,18 +141,19 @@ extern WandExport MagickBooleanType
     const ChannelType, const MetricType,double *),
   MagickGetImageDistortion(MagickWand *,const MagickWand *,const MetricType,
     double *),
-  MagickGetImageChannelExtrema(MagickWand *,const ChannelType,unsigned long *,
-    unsigned long *),
   MagickGetImageChannelMean(MagickWand *,const ChannelType,double *,double *),
+  MagickGetImageChannelRange(MagickWand *,const ChannelType,double *,double *),
   MagickGetImageColormapColor(MagickWand *,const unsigned long,PixelWand *),
-  MagickGetImageExtrema(MagickWand *,unsigned long *,unsigned long *),
   MagickGetImageGreenPrimary(MagickWand *,double *,double *),
+  MagickGetImageMatte(MagickWand *),
   MagickGetImageMatteColor(MagickWand *,PixelWand *),
+  MagickGetImageLength(MagickWand *,MagickSizeType *),
   MagickGetImagePage(MagickWand *,unsigned long *,unsigned long *,long *,
     long *),
   MagickGetImagePixelColor(MagickWand *,const long,const long,PixelWand *),
   MagickGetImagePixels(MagickWand *,const long,const long,const unsigned long,
     const unsigned long,const char *,const StorageType,void *),
+  MagickGetImageRange(MagickWand *,double *,double *),
   MagickGetImageRedPrimary(MagickWand *,double *,double *),
   MagickGetImageResolution(MagickWand *,double *,double *),
   MagickGetImageWhitePoint(MagickWand *,double *,double *),
@@ -145,10 +164,9 @@ extern WandExport MagickBooleanType
   MagickLevelImage(MagickWand *,const double,const double,const double),
   MagickLevelImageChannel(MagickWand *,const ChannelType,const double,
     const double,const double),
+  MagickLinearStretchImage(MagickWand *,const double,const double),
   MagickMagnifyImage(MagickWand *),
   MagickMapImage(MagickWand *,const MagickWand *,const MagickBooleanType),
-  MagickMatteFloodfillImage(MagickWand *,const Quantum,const double,
-    const PixelWand *,const long,const long),
   MagickMedianFilterImage(MagickWand *,const double),
   MagickMinifyImage(MagickWand *),
   MagickModulateImage(MagickWand *,const double,const double,const double),
@@ -160,27 +178,41 @@ extern WandExport MagickBooleanType
     const PixelWand *),
   MagickNextImage(MagickWand *),
   MagickNormalizeImage(MagickWand *),
+  MagickNormalizeImageChannel(MagickWand *,const ChannelType),
   MagickOilPaintImage(MagickWand *,const double),
+  MagickOrderedPosterizeImage(MagickWand *,const char *),
+  MagickOrderedPosterizeImageChannel(MagickWand *,const ChannelType,
+    const char *),
+  MagickPaintFloodfillImage(MagickWand *,const ChannelType,const PixelWand *,
+    const double,const PixelWand *,const long,const long),
   MagickPaintOpaqueImage(MagickWand *,const PixelWand *,const PixelWand *,
     const double),
-  MagickPaintTransparentImage(MagickWand *,const PixelWand *,const Quantum,
+  MagickPaintOpaqueImageChannel(MagickWand *,const ChannelType,
+    const PixelWand *,const PixelWand *,const double),
+  MagickPaintTransparentImage(MagickWand *,const PixelWand *,const double,
     const double),
   MagickPingImage(MagickWand *,const char *),
+  MagickPingImageBlob(MagickWand *,const void *,const size_t),
+  MagickPingImageFile(MagickWand *,FILE *),
+  MagickPolaroidImage(MagickWand *,const DrawingWand *,const double),
   MagickPosterizeImage(MagickWand *,const unsigned long,
     const MagickBooleanType),
   MagickPreviousImage(MagickWand *),
-  MagickProfileImage(MagickWand *,const char *,const void *,
-    const unsigned long),
   MagickQuantizeImage(MagickWand *,const unsigned long,const ColorspaceType,
     const unsigned long,const MagickBooleanType,const MagickBooleanType),
   MagickQuantizeImages(MagickWand *,const unsigned long,const ColorspaceType,
     const unsigned long,const MagickBooleanType,const MagickBooleanType),
   MagickRadialBlurImage(MagickWand *,const double),
+  MagickRadialBlurImageChannel(MagickWand *,const ChannelType,const double),
   MagickRaiseImage(MagickWand *,const unsigned long,const unsigned long,
     const long,const long,const MagickBooleanType),
+  MagickRandomThresholdImage(MagickWand *,const double,const double),
+  MagickRandomThresholdImageChannel(MagickWand *,const ChannelType,const double,
+    const double),
   MagickReadImage(MagickWand *,const char *),
-  MagickReadImageBlob(MagickWand *,const void *,const size_t length),
+  MagickReadImageBlob(MagickWand *,const void *,const size_t),
   MagickReadImageFile(MagickWand *,FILE *),
+  MagickRecolorImage(MagickWand *,const unsigned long,const double *),
   MagickReduceNoiseImage(MagickWand *,const double),
   MagickRemoveImage(MagickWand *),
   MagickResampleImage(MagickWand *,const double,const double,const FilterTypes,
@@ -191,16 +223,19 @@ extern WandExport MagickBooleanType
   MagickRotateImage(MagickWand *,const PixelWand *,const double),
   MagickSampleImage(MagickWand *,const unsigned long,const unsigned long),
   MagickScaleImage(MagickWand *,const unsigned long,const unsigned long),
+  MagickSegmentImage(MagickWand *,const ColorspaceType,const MagickBooleanType,
+    const double,const double),
   MagickSeparateImageChannel(MagickWand *,const ChannelType),
   MagickSepiaToneImage(MagickWand *,const double),
   MagickSetImage(MagickWand *,const MagickWand *),
-  MagickSetImageAttribute(MagickWand *,const char *,const char *),
+  MagickSetImageAlphaChannel(MagickWand *,const AlphaChannelType),
   MagickSetImageBackgroundColor(MagickWand *,const PixelWand *),
   MagickSetImageBias(MagickWand *,const double),
   MagickSetImageBluePrimary(MagickWand *,const double,const double),
   MagickSetImageBorderColor(MagickWand *,const PixelWand *),
   MagickSetImageChannelDepth(MagickWand *,const ChannelType,
     const unsigned long),
+  MagickSetImageClipMask(MagickWand *,const MagickWand *),
   MagickSetImageColormapColor(MagickWand *,const unsigned long,
     const PixelWand *),
   MagickSetImageCompose(MagickWand *,const CompositeOperator),
@@ -212,26 +247,32 @@ extern WandExport MagickBooleanType
   MagickSetImageCompressionQuality(MagickWand *,const unsigned long),
   MagickSetImageGreenPrimary(MagickWand *,const double,const double),
   MagickSetImageGamma(MagickWand *,const double),
+  MagickSetImageExtent(MagickWand *,const unsigned long,const unsigned long),
   MagickSetImageFilename(MagickWand *,const char *),
   MagickSetImageFormat(MagickWand *,const char *),
-  MagickSetImageIndex(MagickWand *,const long),
   MagickSetImageInterlaceScheme(MagickWand *,const InterlaceType),
+  MagickSetImageInterpolateMethod(MagickWand *,
+    const InterpolatePixelMethod),
   MagickSetImageIterations(MagickWand *,const unsigned long),
+  MagickSetImageMatte(MagickWand *,const MagickBooleanType),
   MagickSetImageMatteColor(MagickWand *,const PixelWand *),
+  MagickSetImageOpacity(MagickWand *,const double),
+  MagickSetImageOrientation(MagickWand *,const OrientationType),
   MagickSetImagePage(MagickWand *,const unsigned long,const unsigned long,
     const long,const long),
+  MagickResetImagePage(MagickWand *,const char *),
   MagickSetImagePixels(MagickWand *,const long,const long,const unsigned long,
     const unsigned long,const char *,const StorageType,const void *),
   MagickSetImageRedPrimary(MagickWand *,const double,const double),
   MagickSetImageRenderingIntent(MagickWand *,const RenderingIntent),
   MagickSetImageResolution(MagickWand *,const double,const double),
   MagickSetImageScene(MagickWand *,const unsigned long),
+  MagickSetImageTicksPerSecond(MagickWand *,const long),
   MagickSetImageType(MagickWand *,const ImageType),
   MagickSetImageUnits(MagickWand *,const ResolutionType),
-  MagickSetImageVirtualPixelMethod(MagickWand *,const VirtualPixelMethod),
   MagickSetImageWhitePoint(MagickWand *,const double,const double),
-  MagickSetImageProfile(MagickWand *,const char *,const void *,
-    const unsigned long),
+  MagickShadeImage(MagickWand *,const MagickBooleanType,const double,const 
+    double),
   MagickShadowImage(MagickWand *,const double,const double,const long,
     const long),
   MagickSharpenImage(MagickWand *,const double,const double),
@@ -243,6 +284,7 @@ extern WandExport MagickBooleanType
     const double,const double),
   MagickSigmoidalContrastImageChannel(MagickWand *,const ChannelType,
     const MagickBooleanType,const double,const double),
+  MagickSketchImage(MagickWand *,const double,const double,const double),
   MagickSolarizeImage(MagickWand *,const double),
   MagickSpliceImage(MagickWand *,const unsigned long,const unsigned long,
     const long,const long),
@@ -250,13 +292,19 @@ extern WandExport MagickBooleanType
   MagickStripImage(MagickWand *),
   MagickSwirlImage(MagickWand *,const double),
   MagickTintImage(MagickWand *,const PixelWand *,const PixelWand *),
+  MagickTransposeImage(MagickWand *),
+  MagickTransverseImage(MagickWand *),
   MagickThresholdImage(MagickWand *,const double),
   MagickThresholdImageChannel(MagickWand *,const ChannelType,const double),
+  MagickThumbnailImage(MagickWand *,const unsigned long,const unsigned long),
   MagickTrimImage(MagickWand *,const double),
+  MagickUniqueImageColors(MagickWand *),
   MagickUnsharpMaskImage(MagickWand *,const double,const double,const double,
     const double),
   MagickUnsharpMaskImageChannel(MagickWand *,const ChannelType,const double,
     const double,const double,const double),
+  MagickVignetteImage(MagickWand *,const double,const double,
+    const long,const long),
   MagickWaveImage(MagickWand *,const double,const double),
   MagickWhiteThresholdImage(MagickWand *,const PixelWand *),
   MagickWriteImage(MagickWand *,const char *),
@@ -268,9 +316,6 @@ WandExport MagickProgressMonitor
   MagickSetImageProgressMonitor(MagickWand *,const MagickProgressMonitor,
     void *);
 
-extern WandExport MagickSizeType
-  MagickGetImageSize(MagickWand *);
-
 extern WandExport MagickWand
   *MagickAppendImages(MagickWand *,const MagickBooleanType),
   *MagickAverageImages(MagickWand *),
@@ -280,17 +325,19 @@ extern WandExport MagickWand
     const MetricType,double *),
   *MagickCompareImages(MagickWand *,const MagickWand *,const MetricType,
     double *),
+  *MagickCompareImageLayers(MagickWand *,const ImageLayerMethod),
   *MagickDeconstructImages(MagickWand *),
-  *MagickFlattenImages(MagickWand *),
   *MagickFxImage(MagickWand *,const char *),
   *MagickFxImageChannel(MagickWand *,const ChannelType,const char *),
   *MagickGetImage(MagickWand *),
+  *MagickGetImageClipMask(MagickWand *),
   *MagickGetImageRegion(MagickWand *,const unsigned long,const unsigned long,
     const long,const long),
+  *MagickMergeImageLayers(MagickWand *,const ImageLayerMethod),
   *MagickMorphImages(MagickWand *,const unsigned long),
-  *MagickMosaicImages(MagickWand *),
   *MagickMontageImage(MagickWand *,const DrawingWand *,const char *,
     const char *,const MontageMode,const char *),
+  *MagickOptimizeImageLayers(MagickWand *),
   *MagickPreviewImages(MagickWand *wand,const PreviewType),
   *MagickSteganoImage(MagickWand *,const MagickWand *,const long),
   *MagickStereoImage(MagickWand *,const MagickWand *),
@@ -298,6 +345,9 @@ extern WandExport MagickWand
   *MagickTransformImage(MagickWand *,const char *,const char *),
   *NewMagickWandFromImage(const Image *);
 
+extern WandExport OrientationType
+  MagickGetImageOrientation(MagickWand *);
+
 extern WandExport PixelWand
   **MagickGetImageHistogram(MagickWand *,unsigned long *);
 
@@ -309,9 +359,7 @@ extern WandExport ResolutionType
 
 extern WandExport unsigned char
   *MagickGetImageBlob(MagickWand *,size_t *),
-  *MagickGetImagesBlob(MagickWand *,size_t *),
-  *MagickGetImageProfile(MagickWand *,const char *,unsigned long *),
-  *MagickRemoveImageProfile(MagickWand *,const char *,unsigned long *);
+  *MagickGetImagesBlob(MagickWand *,size_t *);
 
 extern WandExport unsigned long
   MagickGetImageColors(MagickWand *),
@@ -322,32 +370,13 @@ extern WandExport unsigned long
   MagickGetImageHeight(MagickWand *),
   MagickGetImageIterations(MagickWand *),
   MagickGetImageScene(MagickWand *),
+  MagickGetImageTicksPerSecond(MagickWand *),
   MagickGetImageWidth(MagickWand *),
   MagickGetNumberImages(MagickWand *);
 
 extern WandExport VirtualPixelMethod
   MagickGetImageVirtualPixelMethod(MagickWand *);
 
-/*
-  Deprecated methods.
-*/
-extern WandExport char
-  *MagickDescribeImage(MagickWand *);
-
-extern WandExport MagickBooleanType
-  MagickOpaqueImage(MagickWand *,const PixelWand *,const PixelWand *,
-    const double),
-  MagickSetImageOption(MagickWand *,const char *,const char *,const char *),
-  MagickTransparentImage(MagickWand *,const PixelWand *,const Quantum,
-    const double);
-
-extern WandExport MagickWand
-  *MagickRegionOfInterestImage(MagickWand *,const unsigned long,
-	  const unsigned long,const long,const long);
-
-extern WandExport unsigned char
-  *MagickWriteImageBlob(MagickWand *,size_t *);
-
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
