@@ -23,7 +23,7 @@
 %                                 August 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -47,21 +47,25 @@
   Include declarations.
 */
 #include "wand/studio.h"
-#include "wand/magick-wand.h"
+#include "wand/MagickWand.h"
 #include "wand/magick-wand-private.h"
 #include "wand/wand.h"
-#include "magick/ImageMagick.h"
 
 /*
   Define declarations.
 */
 #define ThrowWandException(severity,tag,context) \
 { \
-  (void) ThrowMagickException(&wand->exception,GetMagickModule(),severity, \
+  (void) ThrowMagickException(wand->exception,GetMagickModule(),severity, \
     tag,"`%s'",context); \
   return(MagickFalse); \
 }
-#define MagickWandId  "MagickWand"
+
+/*
+  Global declarations.
+*/
+static unsigned long
+  id = 0;
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -89,16 +93,15 @@
 WandExport void ClearMagickWand(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   wand->quantize_info=DestroyQuantizeInfo(wand->quantize_info);
   wand->image_info=DestroyImageInfo(wand->image_info);
   wand->images=DestroyImageList(wand->images);
-  DestroyExceptionInfo(&wand->exception);
   wand->quantize_info=CloneQuantizeInfo((QuantizeInfo *) NULL);
-  wand->image_info=CloneImageInfo((ImageInfo *) NULL);
-  GetExceptionInfo(&wand->exception);
+  wand->image_info=AcquireImageInfo();
+  ClearMagickException(wand->exception);
   wand->debug=IsEventLogging();
 }
 
@@ -123,7 +126,6 @@ WandExport void ClearMagickWand(MagickWand *wand)
 %
 %    o wand: The magick wand.
 %
-%
 */
 WandExport MagickWand *CloneMagickWand(const MagickWand *wand)
 {
@@ -131,26 +133,33 @@ WandExport MagickWand *CloneMagickWand(const MagickWand *wand)
     *clone_wand;
 
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   clone_wand=(MagickWand *) AcquireMagickMemory(sizeof(*clone_wand));
   if (clone_wand == (MagickWand *) NULL)
-    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+        message);
+      message=DestroyString(message);
+    }
   (void) ResetMagickMemory(clone_wand,0,sizeof(*clone_wand));
   clone_wand->id=AcquireWandId();
-  (void) FormatMagickString(clone_wand->name,MaxTextExtent,"MagickWand-%lu",
-    clone_wand->id);
-  GetExceptionInfo(&clone_wand->exception);
-  InheritException(&clone_wand->exception,&wand->exception);
+  (void) FormatMagickString(clone_wand->name,MaxTextExtent,"%s-%lu",
+    MagickWandId,clone_wand->id);
+  clone_wand->exception=AcquireExceptionInfo();
+  InheritException(clone_wand->exception,wand->exception);
   clone_wand->image_info=CloneImageInfo(wand->image_info);
   clone_wand->quantize_info=CloneQuantizeInfo(wand->quantize_info);
-  clone_wand->images=CloneImageList(wand->images,&clone_wand->exception);
+  clone_wand->images=CloneImageList(wand->images,clone_wand->exception);
   clone_wand->debug=IsEventLogging();
   if (clone_wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",clone_wand->name);
-  clone_wand->signature=MagickSignature;
+  clone_wand->signature=WandSignature;
   return(clone_wand);
 }
 
@@ -180,15 +189,15 @@ WandExport MagickWand *CloneMagickWand(const MagickWand *wand)
 WandExport MagickWand *DestroyMagickWand(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   wand->quantize_info=DestroyQuantizeInfo(wand->quantize_info);
   wand->image_info=DestroyImageInfo(wand->image_info);
   wand->images=DestroyImageList(wand->images);
-  DestroyExceptionInfo(&wand->exception);
+  wand->exception=DestroyExceptionInfo(wand->exception);
   RelinquishWandId(wand->id);
-  wand->signature=(~MagickSignature);
+  wand->signature=(~WandSignature);
   wand=(MagickWand *) RelinquishMagickMemory(wand);
   return(wand);
 }
@@ -219,7 +228,7 @@ WandExport MagickBooleanType IsMagickWand(const MagickWand *wand)
 {
   if (wand == (const MagickWand *) NULL)
     return(MagickFalse);
-  if (wand->signature != MagickSignature)
+  if (wand->signature != WandSignature)
     return(MagickFalse);
   if (LocaleNCompare(wand->name,MagickWandId,strlen(MagickWandId)) != 0)
     return(MagickFalse);
@@ -231,6 +240,167 @@ WandExport MagickBooleanType IsMagickWand(const MagickWand *wand)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M a g i c k C l e a r E x c e p t i o n                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickClearException() clears any exceptions associated with the wand.
+%
+%  The format of the MagickClearException method is:
+%
+%      MagickBooleanType MagickClearException(MagickWand *wand)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+*/
+WandExport MagickBooleanType MagickClearException(MagickWand *wand)
+{
+  assert(wand != (MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  ClearMagickException(wand->exception);
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a g i c k G e t E x c e p t i o n                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickGetException() returns the severity, reason, and description of any
+%  error that occurs when using other methods in this API.
+%
+%  The format of the MagickGetException method is:
+%
+%      char *MagickGetException(const MagickWand *wand,ExceptionType *severity)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+%    o severity: The severity of the error is returned here.
+%
+*/
+WandExport char *MagickGetException(const MagickWand *wand,
+  ExceptionType *severity)
+{
+  char
+    *description;
+
+  assert(wand != (const MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  assert(severity != (ExceptionType *) NULL);
+  *severity=wand->exception->severity;
+  description=(char *) AcquireQuantumMemory(2UL*MaxTextExtent,
+    sizeof(*description));
+  if (description == (char *) NULL)
+    {
+      (void) ThrowMagickException(wand->exception,GetMagickModule(),WandError,
+        "MemoryAllocationFailed","`%s'",wand->name);
+      return((char *) NULL);
+    }
+  *description='\0';
+  if (wand->exception->reason != (char *) NULL)
+    (void) CopyMagickString(description,GetLocaleExceptionMessage(
+      wand->exception->severity,wand->exception->reason),MaxTextExtent);
+  if (wand->exception->description != (char *) NULL)
+    {
+      (void) ConcatenateMagickString(description," (",MaxTextExtent);
+      (void) ConcatenateMagickString(description,GetLocaleExceptionMessage(
+        wand->exception->severity,wand->exception->description),MaxTextExtent);
+      (void) ConcatenateMagickString(description,")",MaxTextExtent);
+    }
+  return(description);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a g i c k G e t  E x c e p t i o n T y p e                              %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickGetExceptionType() returns the exception type associated with the
+%  wand.  If no exception has occurred, UndefinedExceptionType is returned.
+%
+%  The format of the MagickGetExceptionType method is:
+%
+%      ExceptionType MagickGetExceptionType(const MagickWand *wand)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+*/
+WandExport ExceptionType MagickGetExceptionType(const MagickWand *wand)
+{
+  assert(wand != (MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  return(wand->exception->severity);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a g i c k G e t I t e r a t o r I n d e x                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickGetIteratorIndex() returns the position of the iterator in the image
+%  list.
+%
+%  The format of the MagickGetIteratorIndex method is:
+%
+%      long MagickGetIteratorIndex(MagickWand *wand)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+*/
+WandExport long MagickGetIteratorIndex(MagickWand *wand)
+{
+  assert(wand != (MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  if (wand->images == (Image *) NULL)
+    {
+      (void) ThrowMagickException(wand->exception,GetMagickModule(),WandError,
+        "ContainsNoIterators","`%s'",wand->name);
+      return(-1);
+    }
+  return(GetImageIndexInList(wand->images));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   M a g i c k Q u e r y C o n f i g u r e O p t i o n                       %
 %                                                                             %
 %                                                                             %
@@ -258,14 +428,14 @@ WandExport char *MagickQueryConfigureOption(const char *option)
     **configure_info;
 
   ExceptionInfo
-    exception;
+    *exception;
 
   unsigned long
     number_options;
 
-  GetExceptionInfo(&exception);
-  configure_info=GetConfigureInfoList(option,&number_options,&exception);
-  DestroyExceptionInfo(&exception);
+  exception=AcquireExceptionInfo();
+  configure_info=GetConfigureInfoList(option,&number_options,exception);
+  exception=DestroyExceptionInfo(exception);
   if (configure_info == (const ConfigureInfo **) NULL)
     return((char *) NULL);
   value=AcquireString(configure_info[0]->value);
@@ -309,11 +479,11 @@ WandExport char **MagickQueryConfigureOptions(const char *pattern,
     **options;
 
   ExceptionInfo
-    exception;
+    *exception;
 
-  GetExceptionInfo(&exception);
-  options=GetConfigureList(pattern,number_options,&exception);
-  DestroyExceptionInfo(&exception);
+  exception=AcquireExceptionInfo();
+  options=GetConfigureList(pattern,number_options,exception);
+  exception=DestroyExceptionInfo(exception);
   return(options);
 }
 
@@ -328,7 +498,7 @@ WandExport char **MagickQueryConfigureOptions(const char *pattern,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  MagickQueryFontMetrics() returns a 7 element array representing the
+%  MagickQueryFontMetrics() returns a 13 element array representing the
 %  following font metrics:
 %
 %    Element Description
@@ -340,6 +510,12 @@ WandExport char **MagickQueryConfigureOptions(const char *pattern,
 %          4 text width
 %          5 text height
 %          6 maximum horizontal advance
+%          7 bounding box: x1
+%          8 bounding box: y1
+%          9 bounding box: x2
+%         10 bounding box: y2
+%         11 origin: x
+%         12 origin: y
 %
 %  The format of the MagickQueryFontMetrics method is:
 %
@@ -371,17 +547,17 @@ WandExport double *MagickQueryFontMetrics(MagickWand *wand,
     metrics;
 
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   assert(drawing_wand != (const DrawingWand *) NULL);
   if (wand->images == (Image *) NULL)
     {
-      (void) ThrowMagickException(&wand->exception,GetMagickModule(),WandError,
+      (void) ThrowMagickException(wand->exception,GetMagickModule(),WandError,
         "ContainsNoImages","`%s'",wand->name);
       return((double *) NULL);
     }
-  font_metrics=(double *) AcquireMagickMemory(7*sizeof(*font_metrics));
+  font_metrics=(double *) AcquireQuantumMemory(13UL,sizeof(*font_metrics));
   if (font_metrics == (double *) NULL)
     return((double *) NULL);
   draw_info=PeekDrawingWand(drawing_wand);
@@ -396,7 +572,7 @@ WandExport double *MagickQueryFontMetrics(MagickWand *wand,
   draw_info=DestroyDrawInfo(draw_info);
   if (status == MagickFalse)
     {
-      InheritException(&wand->exception,&wand->images->exception);
+      InheritException(wand->exception,&wand->images->exception);
       font_metrics=(double *) RelinquishMagickMemory(font_metrics);
       return((double *) NULL);
     }
@@ -407,6 +583,12 @@ WandExport double *MagickQueryFontMetrics(MagickWand *wand,
   font_metrics[4]=metrics.width;
   font_metrics[5]=metrics.height;
   font_metrics[6]=metrics.max_advance;
+  font_metrics[7]=metrics.bounds.x1;
+  font_metrics[8]=metrics.bounds.y1;
+  font_metrics[9]=metrics.bounds.x2;
+  font_metrics[10]=metrics.bounds.y2;
+  font_metrics[11]=metrics.origin.x;
+  font_metrics[12]=metrics.origin.y;
   return(font_metrics);
 }
 
@@ -421,7 +603,7 @@ WandExport double *MagickQueryFontMetrics(MagickWand *wand,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  MagickQueryMultilineFontMetrics() returns a 7 element array representing the
+%  MagickQueryMultilineFontMetrics() returns a 13 element array representing the
 %  following font metrics:
 %
 %    Element Description
@@ -433,6 +615,12 @@ WandExport double *MagickQueryFontMetrics(MagickWand *wand,
 %          4 text width
 %          5 text height
 %          6 maximum horizontal advance
+%          7 bounding box: x1
+%          8 bounding box: y1
+%          9 bounding box: x2
+%         10 bounding box: y2
+%         11 origin: x
+%         12 origin: y
 %
 %  This method is like MagickQueryFontMetrics() but it returns the maximum text
 %  width and height for multiple lines of text.
@@ -467,17 +655,17 @@ WandExport double *MagickQueryMultilineFontMetrics(MagickWand *wand,
     metrics;
 
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   assert(drawing_wand != (const DrawingWand *) NULL);
   if (wand->images == (Image *) NULL)
     {
-      (void) ThrowMagickException(&wand->exception,GetMagickModule(),WandError,
+      (void) ThrowMagickException(wand->exception,GetMagickModule(),WandError,
         "ContainsNoImages","`%s'",wand->name);
       return((double *) NULL);
     }
-  font_metrics=(double *) AcquireMagickMemory(7*sizeof(*font_metrics));
+  font_metrics=(double *) AcquireQuantumMemory(13UL,sizeof(*font_metrics));
   if (font_metrics == (double *) NULL)
     return((double *) NULL);
   draw_info=PeekDrawingWand(drawing_wand);
@@ -492,7 +680,7 @@ WandExport double *MagickQueryMultilineFontMetrics(MagickWand *wand,
   draw_info=DestroyDrawInfo(draw_info);
   if (status == MagickFalse)
     {
-      InheritException(&wand->exception,&wand->images->exception);
+      InheritException(wand->exception,&wand->images->exception);
       font_metrics=(double *) RelinquishMagickMemory(font_metrics);
       return((double *) NULL);
     }
@@ -503,6 +691,12 @@ WandExport double *MagickQueryMultilineFontMetrics(MagickWand *wand,
   font_metrics[4]=metrics.width;
   font_metrics[5]=metrics.height;
   font_metrics[6]=metrics.max_advance;
+  font_metrics[7]=metrics.bounds.x1;
+  font_metrics[8]=metrics.bounds.y1;
+  font_metrics[9]=metrics.bounds.x2;
+  font_metrics[10]=metrics.bounds.y2;
+  font_metrics[11]=metrics.origin.x;
+  font_metrics[12]=metrics.origin.y;
   return(font_metrics);
 }
 
@@ -539,11 +733,11 @@ WandExport char **MagickQueryFonts(const char *pattern,
     **fonts;
 
   ExceptionInfo
-    exception;
+    *exception;
 
-  GetExceptionInfo(&exception);
-  fonts=GetTypeList(pattern,number_fonts,&exception);
-  DestroyExceptionInfo(&exception);
+  exception=AcquireExceptionInfo();
+  fonts=GetTypeList(pattern,number_fonts,exception);
+  exception=DestroyExceptionInfo(exception);
   return(fonts);
 }
 
@@ -573,7 +767,6 @@ WandExport char **MagickQueryFonts(const char *pattern,
 %    o number_formats:  This integer returns the number of image formats in the
 %      list.
 %
-%
 */
 WandExport char **MagickQueryFormats(const char *pattern,
   unsigned long *number_formats)
@@ -582,11 +775,11 @@ WandExport char **MagickQueryFormats(const char *pattern,
     **formats;
 
   ExceptionInfo
-    exception;
+    *exception;
 
-  GetExceptionInfo(&exception);
-  formats=GetMagickList(pattern,number_formats,&exception);
-  DestroyExceptionInfo(&exception);
+  exception=AcquireExceptionInfo();
+  formats=GetMagickList(pattern,number_formats,exception);
+  exception=DestroyExceptionInfo(exception);
   return(formats);
 }
 
@@ -617,7 +810,6 @@ WandExport char **MagickQueryFormats(const char *pattern,
 WandExport void *MagickRelinquishMemory(void *memory)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  assert(memory != (void *) NULL);
   return(RelinquishMagickMemory(memory));
 }
 
@@ -648,10 +840,11 @@ WandExport void *MagickRelinquishMemory(void *memory)
 WandExport void MagickResetIterator(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
-  wand->iterator=MagickTrue;
+  wand->active=MagickFalse;
+  wand->pend=MagickTrue;
   wand->images=GetFirstImageInList(wand->images);
 }
 
@@ -680,12 +873,63 @@ WandExport void MagickResetIterator(MagickWand *wand)
 WandExport void MagickSetFirstIterator(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
-  wand->iterator=MagickFalse;
+  wand->active=MagickTrue;
+  wand->pend=MagickFalse;
   wand->images=GetFirstImageInList(wand->images);
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a g i c k S e t I t e r a t o r I n d e x                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickSetIteratorIndex() set the iterator to the position in the image list
+%  specified with the index parameter.
+%
+%  The format of the MagickSetIteratorIndex method is:
+%
+%      MagickBooleanType MagickSetIteratorIndex(MagickWand *wand,
+%        const long index)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+%    o index: The scene number.
+%
+*/
+WandExport MagickBooleanType MagickSetIteratorIndex(MagickWand *wand,
+  const long index)
+{
+  Image
+    *image;
+
+  assert(wand != (MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  if (wand->images == (Image *) NULL)
+    return(MagickFalse);
+  image=GetImageFromList(wand->images,index);
+  if (image == (Image *) NULL)
+    {
+      InheritException(wand->exception,&wand->images->exception);
+      return(MagickFalse);
+    }
+  wand->active=MagickTrue;
+  wand->pend=MagickFalse;
+  wand->images=image;
+  return(MagickTrue);
+}
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -711,10 +955,11 @@ WandExport void MagickSetFirstIterator(MagickWand *wand)
 WandExport void MagickSetLastIterator(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
-  wand->iterator=MagickFalse;
+  wand->active=MagickFalse;
+  wand->pend=MagickTrue;
   wand->images=GetLastImageInList(wand->images);
 }
 
@@ -723,6 +968,55 @@ WandExport void MagickSetLastIterator(MagickWand *wand)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M a g i c k W a n d G e n e s i s                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickWandGenesis() initializes the MagickWand environment.
+%
+%  The format of the MaickWandGenesis method is:
+%
+%      void MaickWandGenesis(void)
+%
+*/
+WandExport void MagickWandGenesis(void)
+{
+  if (IsMagickInstantiated() == MagickFalse)
+    MagickCoreGenesis((char *) NULL,MagickFalse);
+  id=AcquireWandId();
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a g i c k W a n d T e r m i n u s                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickWandTerminus() terminates the MagickWand environment.
+%
+%  The format of the MaickWandTerminus method is:
+%
+%      void MaickWandTerminus(void)
+%
+*/
+WandExport void MagickWandTerminus(void)
+{
+  RelinquishWandId(id);
+  MagickCoreTerminus();
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   N e w M a g i c k W a n d                                                 %
 %                                                                             %
 %                                                                             %
@@ -733,8 +1027,7 @@ WandExport void MagickSetLastIterator(MagickWand *wand)
 %
 %  The format of the NewMagickWand method is:
 %
-%      MagickWand NewMagickWand(void)
-%
+%      MagickWand *NewMagickWand(void)
 %
 */
 WandExport MagickWand *NewMagickWand(void)
@@ -748,26 +1041,33 @@ WandExport MagickWand *NewMagickWand(void)
   unsigned long
     depth;
 
-  depth=QuantumDepth;
+  depth=MAGICKCORE_QUANTUM_DEPTH;
   quantum=GetMagickQuantumDepth(&depth);
-  if (depth != QuantumDepth)
+  if (depth != MAGICKCORE_QUANTUM_DEPTH)
     ThrowWandFatalException(WandError,"QuantumDepthMismatch",quantum);
   wand=(MagickWand *) AcquireMagickMemory(sizeof(*wand));
   if (wand == (MagickWand *) NULL)
-    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+        message);
+      message=DestroyString(message);
+    }
   (void) ResetMagickMemory(wand,0,sizeof(*wand));
   wand->id=AcquireWandId();
   (void) FormatMagickString(wand->name,MaxTextExtent,"%s-%lu",MagickWandId,
     wand->id);
-  GetExceptionInfo(&wand->exception);
-  wand->image_info=CloneImageInfo((ImageInfo *) NULL);
+  wand->exception=AcquireExceptionInfo();
+  wand->image_info=AcquireImageInfo();
   wand->quantize_info=CloneQuantizeInfo((QuantizeInfo *) NULL);
   wand->images=NewImageList();
   wand->debug=IsEventLogging();
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
-  wand->signature=MagickSignature;
+  wand->signature=WandSignature;
   return(wand);
 }
 
@@ -782,11 +1082,11 @@ WandExport MagickWand *NewMagickWand(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  NewMagickWandWithImage() returns a wand with an image.
+%  NewMagickWandFromImage() returns a wand with an image.
 %
-%  The format of the NewMagickWandWithImage method is:
+%  The format of the NewMagickWandFromImage method is:
 %
-%      MagickWand NewMagickWandWithImage(const Image *image)
+%      MagickWand NewMagickWandFromImage(const Image *image)
 %
 %  A description of each parameter follows:
 %
@@ -799,6 +1099,6 @@ WandExport MagickWand *NewMagickWandFromImage(const Image *image)
     *wand;
 
   wand=NewMagickWand();
-  wand->images=CloneImage(image,0,0,MagickTrue,&wand->exception);
+  wand->images=CloneImage(image,0,0,MagickTrue,wand->exception);
   return(wand);
 }
